@@ -22,6 +22,7 @@ static ut64 from = 0LL, to = -1;
 static char *mask = NULL;
 static int nonstop = 0;
 static bool identify = false;
+static bool quiet = false;
 static int mode = R_SEARCH_STRING;
 static ut64 cur = 0;
 static ut8 *buf = NULL;
@@ -34,6 +35,10 @@ static RList *keywords;
 
 static int hit(RSearchKeyword *kw, void *user, ut64 addr) {
 	int delta = addr - cur;
+	if (cur > addr && (cur - addr == kw->keyword_length - 1)) {
+		// This case occurs when there is hit in search left over
+		delta = cur - addr;
+	}
 	if (delta < 0 || delta >= bsize) {
 		eprintf ("Invalid delta\n");
 		return 0;
@@ -77,7 +82,7 @@ static int hit(RSearchKeyword *kw, void *user, ut64 addr) {
 }
 
 static int show_help(char *argv0, int line) {
-	printf ("Usage: %s [-mXnzZhv] [-a align] [-b sz] [-f/t from/to] [-[e|s|S] str] [-x hex] file ..\n", argv0);
+	printf ("Usage: %s [-mXnzZhqv] [-a align] [-b sz] [-f/t from/to] [-[e|s|S] str] [-x hex] file|dir ..\n", argv0);
 	if (line) return 0;
 	printf (
 	" -a [align] only accept aligned hits\n"
@@ -93,6 +98,7 @@ static int show_help(char *argv0, int line) {
 	" -s [str]   search for a specific string (can be used multiple times)\n"
 	" -S [str]   search for a specific wide string (can be used multiple times)\n"
 	" -t [to]    stop search at address 'to'\n"
+	" -q         quiet - do not show headings (filenames) above matching contents (default for searching a single file)\n"
 	" -v         print version and exit\n"
 	" -x [hex]   search for hexpair string (909090) (can be used multiple times)\n"
 	" -X         show hexdump of search results\n"
@@ -102,12 +108,18 @@ static int show_help(char *argv0, int line) {
 	return 0;
 }
 
-static int rafind_open(char *file) {
+static int rafind_open(char *file);
+
+static int rafind_open_file(char *file) {
 	const char *kw;
 	RListIter *iter;
 	bool last = false;
 	int ret;
 
+	if (!quiet) {
+		printf ("File: %s\n", file);
+	}
+
 	if (identify) {
 		char *cmd = r_str_newf ("r2 -e search.show=false -e search.maxhits=1 -nqcpm '%s'", file);
 		r_sandbox_system (cmd, 1);
@@ -154,6 +166,15 @@ static int rafind_open(char *file) {
 		free (tostr);
 		return 0;
 	}
+	if (mode == R_SEARCH_ESIL) {
+		char *cmd;
+		r_list_foreach (keywords, iter, kw) {
+			cmd = r_str_newf ("r2 -qc \"/E %s\" %s", kw, file);
+			r_sandbox_system (cmd, 1);
+			free (cmd);
+		}
+		return 0;
+	}
 	if (mode == R_SEARCH_KEYWORD) {
 		r_list_foreach (keywords, iter, kw) {
 			if (hexstr) {
@@ -199,11 +220,44 @@ static int rafind_open(char *file) {
 	return 0;
 }
 
+static int rafind_open_dir(char *dir) {
+	RList *files;
+	RListIter *iter;
+	char *fullpath;
+	char *fname = NULL;
+
+	files = r_sys_dir (dir);
+
+	if (files) {
+		r_list_foreach (files, iter, fname) {
+			/* Filter-out unwanted entries */
+			if (*fname == '.') {
+				continue;
+			}
+
+			fullpath = r_str_newf ("%s"R_SYS_DIR"%s", dir, fname);
+			rafind_open (fullpath);
+			free (fullpath);
+		}
+		r_list_free (files);
+	}
+
+	return 0;
+}
+
+static int rafind_open(char *file) {
+	if (r_file_is_directory (file)) {
+		return rafind_open_dir (file);
+	}
+
+	return rafind_open_file (file);
+}
+
 int main(int argc, char **argv) {
 	int c;
 
 	keywords = r_list_new ();
-	while ((c = getopt (argc, argv, "a:ie:b:mM:s:S:x:Xzf:t:rnhvZ")) != -1) {
+	while ((c = getopt (argc, argv, "a:ie:b:mM:s:S:x:Xzf:t:E:rqnhvZ")) != -1) {
 		switch (c) {
 		case 'a':
 			align = r_num_math (NULL, optarg);
@@ -225,6 +279,10 @@ int main(int argc, char **argv) {
 			hexstr = 0;
 			r_list_append (keywords, optarg);
 			break;
+		case 'E':
+			mode = R_SEARCH_ESIL;
+			r_list_append (keywords, optarg);
+			break;
 		case 's':
 			mode = R_SEARCH_KEYWORD;
 			hexstr = 0;
@@ -259,6 +317,9 @@ int main(int argc, char **argv) {
 		case 'X':
 			pr = r_print_new ();
 			break;
+		case 'q':
+			quiet = true;
+			break;
 		case 'v':
 			printf ("rafind2 v"R2_VERSION"\n");
 			return 0;
@@ -277,6 +338,10 @@ int main(int argc, char **argv) {
 	if (optind == argc) {
 		return show_help (argv[0], 1);
 	}
+	/* Enable quiet mode if searching just a single file */
+	if (optind + 1 == argc && !r_file_is_directory (argv[optind])) {
+		quiet = true;
+	}
 	for (; optind < argc; optind++) {
 		rafind_open (argv[optind]);
 	}
