@@ -23,6 +23,7 @@ if [ -d "$pfx/usr/bin" ]; then
 fi
 
 new_env='
+R2_ENV_IS_SET=1
 LIBR_PLUGINS=${pfx}/lib/radare2
 PATH=$pfx/bin:${PATH}
 LD_LIBRARY_PATH=$pfx/lib:$LD_LIBRARY_PATH
@@ -31,6 +32,11 @@ PKG_CONFIG_PATH=$pfx/lib/pkgconfig:$PKG_CONFIG_PATH
 '
 
 shift
+if [ "$R2_ENV_IS_SET" = 1 ]; then
+	echo "Nested call"
+	exec $*
+	exit $?
+fi
 
 if [ -z "$*" ]; then
 	echo
