@@ -89,7 +89,7 @@ static bool is_bin(const ut8 *buf, int size) {
 // TODO: add is_flag, is comment?
 
 // XXX: optimize by removing all strlens here
-R_API char *r_anal_data_to_string(RAnalData *d, RConsPalette *pal) {
+R_API char *r_anal_data_to_string(RAnalData *d, RConsPrintablePalette *pal) {
 	int i, len, idx, mallocsz = 1024;
 	ut32 n32;
 	char *line;
@@ -98,7 +98,7 @@ R_API char *r_anal_data_to_string(RAnalData *d, RConsPalette *pal) {
 
 	line = malloc (mallocsz);
 	if (!line) {
-		eprintf ("Cannot allocate %d bytes\n", mallocsz);
+		eprintf ("Cannot allocate %d byte(s)\n", mallocsz);
 		return NULL;
 	}
 	if (pal) {
@@ -224,7 +224,7 @@ R_API RAnalData *r_anal_data_new_string(ut64 addr, const char *p, int len, int t
 		ad->buf = malloc (len + 1);
 		if (!ad->buf) {
 			r_anal_data_free (ad);
-			eprintf ("Cannot allocate %d bytes\n", len + 1);
+			eprintf ("Cannot allocate %d byte(s)\n", len + 1);
 			return NULL;
 		}
 		memcpy (ad->buf, ad->str, len + 1);
