@@ -31,46 +31,52 @@ Keys:
 #undef DB
 #define DB a->sdb_meta
 
-#if 0
-// Defined but not used. Shall we remove it?
 static char *meta_inrange_get (RAnal *a, ut64 addr, int size) {
-	char key[64];
-	ut64 base, base2;
-	base = META_RANGE_BASE (addr);
-	base2 = META_RANGE_BASE (addr+size);
+	ut64 base = META_RANGE_BASE (addr);
+	ut64 base2 = META_RANGE_BASE (addr + size) + 1;
+	char *res = NULL;
 	// return string array of all the offsets where there are stuff
-	for (; base<base2; base += META_RANGE_SIZE) {
-		snprintf (key, sizeof (key)-1, "range.0x%"PFMT64x, base);
-		sdb_array_get (DB, key, 0, 0);
+	for (; base < base2; base += META_RANGE_SIZE) {
+		const char *key = sdb_fmt ("range.0x%"PFMT64x, base);
+		// char *r = sdb_array_get (DB, key, 0, 0);
+		char *r = sdb_get (DB, key, 0);
+		if (r) {
+			if (res) {
+				res = r_str_append (res, ",");
+			}
+			res = r_str_append (res, r);
+			free (r);
+		}
 	}
-	return NULL;
+	return res;
 }
-#endif
 
-static int meta_inrange_add (RAnal *a, ut64 addr, int size) {
-	int set = 0;
+static bool meta_inrange_add (RAnal *a, ut64 addr, int size) {
+	bool set = false;
 	char key[64];
 	ut64 base, base2;
 	base = META_RANGE_BASE (addr);
-	base2 = META_RANGE_BASE (addr+size);
-	for (; base<base2; base += META_RANGE_SIZE) {
+	base2 = META_RANGE_BASE (addr + size) + 1;
+	for (; base < base2; base += META_RANGE_SIZE) {
 		snprintf (key, sizeof (key)-1, "range.0x%"PFMT64x, base);
-		if (sdb_array_add_num (DB, key, addr, 0))
-			set = 1;
+		if (sdb_array_add_num (DB, key, addr, 0)) {
+			set = true;
+		}
 	}
 	return set;
 }
 
-static int meta_inrange_del (RAnal *a, ut64 addr, int size) {
-	int set = 0;
+static bool meta_inrange_del (RAnal *a, ut64 addr, int size) {
+	bool set = false;
 	char key[64];
 	ut64 base = META_RANGE_BASE (addr);
-	ut64 base2 = META_RANGE_BASE (addr+size);
+	ut64 base2 = META_RANGE_BASE (addr + size) + 1;
 // TODO: optimize this thing?
-	for (; base<base2; base += META_RANGE_SIZE) {
+	for (; base < base2; base += META_RANGE_SIZE) {
 		snprintf (key, sizeof (key)-1, "range.0x%"PFMT64x, base);
-		if (sdb_array_remove_num (DB, key, addr, 0))
-			set = 1;
+		if (sdb_array_remove_num (DB, key, addr, 0)) {
+			set = true;
+		}
 	}
 	//sdb_array_del (DB);
 	return set;
@@ -215,7 +221,6 @@ R_API int r_meta_del(RAnal *a, int type, ut64 addr, ut64 size) {
 					snprintf (key, sizeof (key)-1,
 						"meta.%c.0x%"PFMT64x,
 						type, sdb_atoi (s));
-					eprintf ("--> %s\n", key);
 					sdb_unset (DB, key, 0);
 					if (!next) break;
 				}
@@ -227,10 +232,14 @@ R_API int r_meta_del(RAnal *a, int type, ut64 addr, ut64 size) {
 	if (type == R_META_TYPE_ANY) {
 		/* special case */
 		r_meta_del (a, R_META_TYPE_COMMENT, addr, size);
+		r_meta_del (a, R_META_TYPE_VARTYPE, addr, size);
+	}
+	if (type == R_META_TYPE_COMMENT || type == R_META_TYPE_VARTYPE) {
+		snprintf (key, sizeof (key)-1, "meta.%c.0x%"PFMT64x, type, addr);
+	} else {
+		snprintf (key, sizeof (key)-1, "meta.0x%"PFMT64x, addr);
 	}
 	meta_inrange_del (a, addr, size);
-	snprintf (key, sizeof (key)-1, type == R_META_TYPE_COMMENT ?
-		"meta.C.0x%"PFMT64x : "meta.0x%"PFMT64x, addr);
 	val = sdb_const_get (DB, key, 0);
 	if (val) {
 		if (type == R_META_TYPE_ANY) {
@@ -266,6 +275,7 @@ R_API int r_meta_del(RAnal *a, int type, ut64 addr, ut64 size) {
 	sdb_unset (DB, key, 0);
 	return false;
 }
+
 R_API int r_meta_var_comment_del(RAnal *a, int type, ut64 idx, ut64 addr) {
 	char *key;
 	key = r_str_newf ("meta.%c.0x%"PFMT64x"0x%"PFMT64x, type, addr, idx);
@@ -369,7 +379,7 @@ static int meta_add(RAnal *a, int type, int subtype, ut64 from, ut64 to, const c
 	}
 	snprintf (val, sizeof (val)-1, "%c", type);
 	sdb_array_add (DB, key, val, 0);
-
+	meta_inrange_add (a, from, to - from);
 	return true;
 }
 
@@ -416,6 +426,27 @@ R_API RAnalMetaItem *r_meta_find(RAnal *a, ut64 at, int type, int where) {
 	return NULL;
 }
 
+R_API RAnalMetaItem *r_meta_find_in(RAnal *a, ut64 at, int type, int where) {
+	char *res = meta_inrange_get (a, at, 1);
+	if (!res) {
+		return NULL;
+	}
+	RList *list = r_str_split_list (res, ",");
+	RListIter *iter;
+	const char *meta;
+	r_list_foreach (list, iter, meta) {
+		ut64 mia = r_num_math (NULL, meta);
+		RAnalMetaItem *mi = r_meta_find (a, mia, type, where);
+		if (mi && (at >= mi->from && at < mi->to)) {
+			free (res);
+			return mi;
+		}
+	}
+	r_list_free (list);
+	free (res);
+	return NULL;
+}
+
 R_API const char *r_meta_type_to_string(int type) {
 	// XXX: use type as '%c'
 	switch (type) {
@@ -448,7 +479,7 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, int rad, bool show_full) {
 			if (!d->subtype) {  /* temporary legacy workaround */
 				esc_bslash = false;
 			}
-			str = r_str_escape_latin1 (d->str, false, esc_bslash);
+			str = r_str_escape_latin1 (d->str, false, esc_bslash, false);
 		}
 	} else {
 		str = r_str_escape (d->str);
