@@ -223,7 +223,7 @@ static int handle_bb_cf_recursive_descent (RAnal *anal, RAnalState *state) {
 	switch (control_type) {
 		case R_ANAL_OP_TYPE_CALL:
 			IFDBG eprintf (" - Handling a call @ 0x%04"PFMT64x".\n", addr);
-			r_anal_fcn_xref_add (anal, state->current_fcn, bb->addr, bb->jump, R_ANAL_REF_TYPE_CALL);
+			r_anal_xrefs_set (anal, bb->addr, bb->jump, R_ANAL_REF_TYPE_CALL);
 			result = R_ANAL_RET_ERROR;
 			break;
 		case R_ANAL_OP_TYPE_JMP:
@@ -406,7 +406,7 @@ static int handle_bb_cf_linear_sweep (RAnal *anal, RAnalState *state) {
 	switch (control_type) {
 		case R_ANAL_OP_TYPE_CALL:
 			IFDBG eprintf (" - Handling a call @ 0x%04"PFMT64x"\n", addr);
-			r_anal_fcn_xref_add (anal, state->current_fcn, bb->addr, bb->jump, R_ANAL_REF_TYPE_CALL);
+			r_anal_xrefs_set (anal, bb->addr, bb->jump, R_ANAL_REF_TYPE_CALL);
 			result = R_ANAL_RET_ERROR;
 			break;
 		case R_ANAL_OP_TYPE_JMP:
@@ -422,9 +422,11 @@ static int handle_bb_cf_linear_sweep (RAnal *anal, RAnalState *state) {
 			IFDBG eprintf (" - Handling a bb->jump @ 0x%04"PFMT64x", adding 0x%04"PFMT64x" for future visit\n", addr, *paddr64);
 			r_list_append (nodes->cfg_node_addrs, paddr64);
 			paddr64 = malloc (sizeof(ut64));
-			*paddr64 = bb->fail;
-			IFDBG eprintf (" - Handling a bb->fail @ 0x%04"PFMT64x", adding 0x%04"PFMT64x" for future visit\n", addr, *paddr64);
-			r_list_append (nodes->cfg_node_addrs, paddr64);
+			if (paddr64) {
+				*paddr64 = bb->fail;
+				IFDBG eprintf (" - Handling a bb->fail @ 0x%04"PFMT64x", adding 0x%04"PFMT64x" for future visit\n", addr, *paddr64);
+				r_list_append (nodes->cfg_node_addrs, paddr64);
+			}
 			result = R_ANAL_RET_END;
 			break;
 		case R_ANAL_OP_TYPE_SWITCH:
@@ -479,7 +481,7 @@ static int analyze_from_code_buffer(RAnal *anal, RAnalFunction *fcn, ut64 addr,
 
 	fcn->name = strdup (gen_name);
 	fcn->dsc = strdup ("unknown");
-	r_anal_fcn_set_size (fcn, code_length);
+	r_anal_fcn_set_size (NULL, fcn, code_length);
 	fcn->type = R_ANAL_FCN_TYPE_FCN;
 	fcn->addr = addr;
 	state = r_anal_state_new (addr, (ut8*) code_buf, code_length);
@@ -491,16 +493,19 @@ static int analyze_from_code_buffer(RAnal *anal, RAnalFunction *fcn, ut64 addr,
 	r_list_foreach (fcn->bbs, bb_iter, bb) {
 		actual_size += bb->size;
 	}
-	r_anal_fcn_set_size (fcn, state->bytes_consumed);
+	r_anal_fcn_set_size (NULL, fcn, state->bytes_consumed);
 	result = state->anal_ret_val;
 	r_list_free (nodes->cfg_node_addrs);
 	free (nodes);
 	//leak to avoid UAF is the easy solution otherwise a whole rewrite is needed
 	//r_anal_state_free (state);
 	if (r_anal_fcn_size (fcn) != code_length) {
+		return R_ANAL_RET_ERROR;
+#if 0
 		eprintf ("WARNING Analysis of %s Incorrect: Code Length: 0x%"PFMT64x", Function size reported 0x%x\n", fcn->name, code_length, r_anal_fcn_size(fcn));
 		eprintf ("Deadcode detected, setting code length to: 0x%"PFMT64x"\n", code_length);
 		r_anal_fcn_set_size (fcn, code_length);
+#endif
 	}
 	return result;
 }
@@ -515,7 +520,7 @@ static int analyze_from_code_attr (RAnal *anal, RAnalFunction *fcn, RBinJavaFiel
 	if (!code_attr) {
 		fcn->name = strdup ("sym.UNKNOWN");
 		fcn->dsc = strdup ("unknown");
-		r_anal_fcn_set_size (fcn, code_length);
+		r_anal_fcn_set_size (NULL, fcn, code_length);
 		fcn->type = R_ANAL_FCN_TYPE_FCN;
 		fcn->addr = 0;
 		return R_ANAL_RET_ERROR;
@@ -523,25 +528,24 @@ static int analyze_from_code_attr (RAnal *anal, RAnalFunction *fcn, RBinJavaFiel
 
 	code_length = code_attr->info.code_attr.code_length;
 	code_addr = code_attr->info.code_attr.code_offset;
-	code_buf = malloc (code_length);
+	code_buf = calloc (1, code_length);
 
 	anal->iob.read_at (anal->iob.io, code_addr + loadaddr, code_buf, code_length);
 	result = analyze_from_code_buffer (anal, fcn, code_addr + loadaddr, code_buf, code_length);
 	free (code_buf);
 
-	{
-		char *cname = NULL;
-		char *name = strdup (method->name);
+	char *name = strdup (method->name);
+	if (name) {
 		r_name_filter (name, 80);
 		free (fcn->name);
 		if (method->class_name) {
-			cname = strdup (method->class_name);
+			char *cname = strdup (method->class_name);
 			r_name_filter (cname, 50);
 			fcn->name = r_str_newf ("sym.%s.%s", cname, name);
+			free (cname);
 		} else {
 			fcn->name = r_str_newf ("sym.%s", name);
 		}
-		free (cname);
 		free (name);
 	}
 
@@ -565,7 +569,6 @@ static int analyze_method(RAnal *anal, RAnalFunction *fcn, RAnalState *state) {
 }
 
 static int java_analyze_fns_from_buffer( RAnal *anal, ut64 start, ut64 end, int reftype, int depth) {
-
 	int result = R_ANAL_RET_ERROR;
 	ut64 addr = start;
 	ut64 offset = 0;
@@ -575,17 +578,17 @@ static int java_analyze_fns_from_buffer( RAnal *anal, ut64 start, ut64 end, int
 
 	if (end == UT64_MAX) {
 		//analyze_all = 1;
-		buf_len = anal->iob.size (anal->iob.io);
+		buf_len = anal->iob.desc_size (anal->iob.io->desc);
 
 		if (buf_len == UT64_MAX) buf_len = 1024;
 
 		end = start + buf_len;
 	}
 
-
 	buffer = malloc (buf_len);
-	if (!buffer) return R_ANAL_RET_ERROR;
-
+	if (!buffer) {
+		return R_ANAL_RET_ERROR;
+	}
 
 	anal->iob.read_at (anal->iob.io, addr, buffer, buf_len);
 
@@ -601,6 +604,7 @@ static int java_analyze_fns_from_buffer( RAnal *anal, ut64 start, ut64 end, int
 			break;
 		}
 		//r_listrange_add (anal->fcnstore, fcn);
+		r_anal_fcn_tree_insert (&anal->fcn_tree, fcn);
 		r_list_append (anal->fcns, fcn);
 		offset += r_anal_fcn_size (fcn);
 		if (!analyze_all) break;
@@ -609,15 +613,12 @@ static int java_analyze_fns_from_buffer( RAnal *anal, ut64 start, ut64 end, int
 	return result;
 }
 
-
 static int java_analyze_fns( RAnal *anal, ut64 start, ut64 end, int reftype, int depth) {
 	//anal->iob.read_at (anal->iob.io, op.jump, bbuf, sizeof (bbuf));
 	RBinJavaObj *bin = NULL;// = get_java_bin_obj (anal);
 	RBinJavaField *method = NULL;
 	RListIter *methods_iter, *bin_obs_iter;
-
-	RList * bin_objs_list = get_java_bin_obj_list (anal),
-		  * methods_list = NULL;// = bin ? r_bin_java_get_methods_list (bin) : NULL;
+	RList * bin_objs_list = get_java_bin_obj_list (anal);
 
 	ut8 analyze_all = 0;
 	//RAnalRef *ref = NULL;
@@ -633,28 +634,29 @@ static int java_analyze_fns( RAnal *anal, ut64 start, ut64 end, int reftype, int
 	r_list_foreach (bin_objs_list, bin_obs_iter, bin) {
 		// loop over all bin object that are loaded
 		java_update_anal_types (anal, bin);
-		methods_list = (RList *) r_bin_java_get_methods_list (bin);
-		if (methods_list) {
-			ut64 loadaddr = bin->loadaddr;
-			// loop over all methods in the binary object and analyse
-			// the functions
-			r_list_foreach ( methods_list, methods_iter, method ) {
-				if ((method && analyze_all) ||
-				    (check_addr_less_start (method, end) ||
-				     check_addr_in_code (method, end))) {
-					RAnalFunction *fcn = r_anal_fcn_new ();
-					fcn->cc = r_str_const (r_anal_cc_default (anal));
-					java_set_function_prototype (anal, fcn, method);
-					result = analyze_from_code_attr (anal, fcn, method, loadaddr);
-					if (result == R_ANAL_RET_ERROR) {
-						eprintf ("Failed to parse java fn: %s @ 0x%04"PFMT64x"\n", fcn->name, fcn->addr);
-						// XXX - TO Stop or not to Stop ??
-					}
-					//r_listrange_add (anal->fcnstore, fcn);
-					r_list_append (anal->fcns, fcn);
+		RList *methods_list = (RList *) r_bin_java_get_methods_list (bin);
+		ut64 loadaddr = bin->loadaddr;
+		// loop over all methods in the binary object and analyse
+		// the functions
+		r_list_foreach (methods_list, methods_iter, method) {
+			if ((method && analyze_all) ||
+			    (check_addr_less_start (method, end) ||
+			     check_addr_in_code (method, end))) {
+				RAnalFunction *fcn = r_anal_fcn_new ();
+				fcn->cc = r_str_const (r_anal_cc_default (anal));
+				java_set_function_prototype (anal, fcn, method);
+				result = analyze_from_code_attr (anal, fcn, method, loadaddr);
+				if (result == R_ANAL_RET_ERROR) {
+					//eprintf ("Failed to parse java fn: %s @ 0x%04"PFMT64x"\n", fcn->name, fcn->addr);
+					return result;
+					// XXX - TO Stop or not to Stop ??
 				}
-			} // End of methods loop
-		}// end of methods_list is valid conditional
+				//r_listrange_add (anal->fcnstore, fcn);
+				r_anal_fcn_update_tinyrange_bbs (fcn);
+				r_anal_fcn_tree_insert (&anal->fcn_tree, fcn);
+				r_list_append (anal->fcns, fcn);
+			}
+		} // End of methods loop
 	}// end of bin_objs list loop
 	return result;
 }
@@ -677,11 +679,11 @@ static int java_switch_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data,
 
 	if (op_byte == 0xaa) {
 		// handle a table switch condition
-		if (pos + 8 > len) {
+		if (pos + 8 + 8 > len) {
 			return op->size;
 		}
-		int min_val = (ut32)(UINT (data, pos + 4)),
-			max_val = (ut32)(UINT (data, pos + 8));
+		const int min_val = (ut32)(UINT (data, pos + 4));
+		const int max_val = (ut32)(UINT (data, pos + 8));
 
 		ut32 default_loc = (ut32) (UINT (data, pos)), cur_case = 0;
 		op->switch_op = r_anal_switch_op_new (addr, min_val, default_loc);
@@ -696,9 +698,12 @@ static int java_switch_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data,
 					break;
 				}
 				int offset = (int)(ut32)(R_BIN_JAVA_UINT (data, pos));
-				caseop = r_anal_switch_op_add_case (op->switch_op, addr+pos, cur_case+min_val, addr+offset);
-				caseop->bb_ref_to = addr+offset;
-				caseop->bb_ref_from = addr; // TODO figure this one out
+				caseop = r_anal_switch_op_add_case (op->switch_op,
+					addr + pos, cur_case + min_val, addr + offset);
+				if (caseop) {
+					caseop->bb_ref_to = addr+offset;
+					caseop->bb_ref_from = addr; // TODO figure this one out
+				}
 			}
 		} else {
 			eprintf ("Invalid switch boundaries at 0x%"PFMT64x"\n", addr);
@@ -709,18 +714,16 @@ static int java_switch_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data,
 }
 
 static int java_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len) {
-	int sz = 1;
-
 	/* get opcode size */
 	//ut8 op_byte = data[0];
 	ut8 op_byte = data[0];
-	sz = JAVA_OPS[op_byte].size;
-	if (!op)	return sz;
-
+	int sz = JAVA_OPS[op_byte].size;
+	if (!op) {
+		return sz;
+	}
 	memset (op, '\0', sizeof (RAnalOp));
-
 	IFDBG {
-		//eprintf ("Extracting op from buffer (%d bytes) @ 0x%04x\n", len, addr);
+		//eprintf ("Extracting op from buffer (%d byte(s)) @ 0x%04x\n", len, addr);
 		//eprintf ("Parsing op: (0x%02x) %s.\n", op_byte, JAVA_OPS[op_byte].name);
 	}
 	op->addr = addr;
@@ -754,7 +757,8 @@ static int java_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len
 	op->eob = r_anal_ex_is_op_type_eop (op->type2);
 	IFDBG {
 		const char *ot_str = r_anal_optype_to_string (op->type);
-		eprintf ("op_type2: %s @ 0x%04"PFMT64x" 0x%08"PFMT64x" op_type: (0x%02"PFMT64x") %s.\n", JAVA_OPS[op_byte].name, addr, op->type2, op->type,  ot_str);
+		eprintf ("op_type2: %s @ 0x%04"PFMT64x" 0x%08"PFMT64x" op_type: (0x%02"PFMT64x") %s.\n",
+			JAVA_OPS[op_byte].name, addr, (ut64)op->type2, (ut64)op->type,  ot_str);
 		//eprintf ("op_eob: 0x%02x.\n", op->eob);
 		//eprintf ("op_byte @ 0: 0x%02x op_byte @ 0x%04x: 0x%02x.\n", data[0], addr, data[addr]);
 	}
@@ -867,24 +871,24 @@ static int java_cmd_ext(RAnal *anal, const char* input) {
 		return -1;
 	}
 	switch (*input) {
-		case 'c':
-			// reset bytes counter for case operations
-			r_java_new_method ();
-			break;
-		case 'u':
-			switch (*(input+1)) {
-				case 't': {java_update_anal_types (anal, obj); return true;}
-				default: break;
-			}
-			break;
-		case 's':
-			switch (*(input+1)) {
-				//case 'e': return java_resolve_cp_idx_b64 (anal, input+2);
-				default: break;
-			}
-			break;
+	case 'c':
+		// reset bytes counter for case operations
+		r_java_new_method ();
+		break;
+	case 'u':
+		switch (*(input+1)) {
+			case 't': {java_update_anal_types (anal, obj); return true;}
+			default: break;
+		}
+		break;
+	case 's':
+		switch (*(input+1)) {
+			//case 'e': return java_resolve_cp_idx_b64 (anal, input+2);
+			default: break;
+		}
+		break;
 
-		default: eprintf("Command not supported"); break;
+	default: eprintf("Command not supported"); break;
 	}
 	return 0;
 }
