@@ -80,18 +80,18 @@ int tms320_c55x_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *buf, int len)
 int tms320_op(RAnal * anal, RAnalOp * op, ut64 addr, const ut8 * buf, int len) {
 	TMS_ANAL_OP_FN aop = tms320_c55x_op;
 
-	if (anal->cpu && strcasecmp(anal->cpu, "c64x") == 0) {
+	if (anal->cpu && r_str_casecmp(anal->cpu, "c64x") == 0) {
 #ifdef CAPSTONE_TMS320C64X_H
 		return tms320c64x_analop (anal, op, addr, buf, len);
 #else
 		return -1;
 #endif
 	}
-	if (anal->cpu && strcasecmp(anal->cpu, "c54x") == 0) {
+	if (anal->cpu && r_str_casecmp(anal->cpu, "c54x") == 0) {
 		aop = tms320_c54x_op;
-	} else if (anal->cpu && strcasecmp(anal->cpu, "c55x") == 0) {
+	} else if (anal->cpu && r_str_casecmp(anal->cpu, "c55x") == 0) {
 		aop = tms320_c55x_op;
-	} else if (anal->cpu && strcasecmp(anal->cpu, "c55x+") == 0) {
+	} else if (anal->cpu && r_str_casecmp(anal->cpu, "c55x+") == 0) {
 		aop = tms320_c55x_plus_op;
 	}
 	return aop (anal, op, addr, buf, len);
@@ -109,7 +109,7 @@ RAnalPlugin r_anal_plugin_tms320 = {
 	.name = "tms320",
 	.arch = "tms320",
 	.bits = 32,
-	.desc = "TMS320 DSP family code analisys plugin",
+	.desc = "TMS320 DSP family code analysis plugin",
 	.init = tms320_init,
 	.fini = tms320_fini,
 	.license = "LGPLv3",
