@@ -21,7 +21,7 @@
    MA 02110-1301, USA.  */
 
 #include "sysdep.h"
-#include "dis-asm.h"
+#include "disas-asm.h"
 #include "opcode/nios2.h"
 #include "libiberty.h"
 #include <string.h>
