@@ -25,7 +25,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 //#include "bfd.h"
 #include "sysdep.h"
-#include "dis-asm.h"
+#include "disas-asm.h"
 //#include "libbfd.h"
 #include "opcode/tricore.h"
 
