@@ -27,9 +27,13 @@
 #include "xtensa-isa.h"
 #include "ansidecl.h"
 #include <setjmp.h>
-#include "dis-asm.h"
+#include "disas-asm.h"
 #include "libiberty.h"
 
+
+#if defined(_MSC_VER)
+__declspec(dllimport)
+#endif
 extern xtensa_isa xtensa_default_isa;
 
 #ifndef MAX
