@@ -8,8 +8,6 @@
 #include <spp/spp.h>
 #include <config.h>
 
-#define R_ASM_OPCODES_PATH "/share/radare2/" R2_VERSION "/opcodes"
-
 R_LIB_VERSION (r_asm);
 
 char *directives[] = {
@@ -137,6 +135,34 @@ static inline int r_asm_pseudo_fill(RAsmOp *op, char *input) {
 	return size;
 }
 
+static inline int r_asm_pseudo_incbin(RAsmOp *op, char *input) {
+	int skip = 0;
+	int count = 0;
+	int bytes_read = 0;
+	r_str_replace_char (input, ',', ' ');
+	// int len = r_str_word_count (input);
+	r_str_word_set0 (input);
+	//const char *filename = r_str_word_get0 (input, 0);
+	skip = (int)r_num_math (NULL, r_str_word_get0 (input, 1));
+	count = (int)r_num_math (NULL,r_str_word_get0 (input, 2));
+	char *content = r_file_slurp (input, &bytes_read);
+	if (skip > 0) {
+		skip = skip > bytes_read ? bytes_read : skip;
+	}
+	if (count > 0) {
+		count = count > bytes_read ? 0 : count;
+	} else {
+		count = bytes_read;
+	}
+	// Need to handle arbitrary amount of data
+	r_buf_free (op->buf_inc);
+	op->buf_inc = r_buf_new_with_string (content + skip);
+	// Terminate the original buffer
+	op->buf_hex[0] = '\0';
+	free (content);
+	return count;
+}
+
 static void plugin_free(RAsmPlugin *p) {
 	if (p && p->fini) {
 		p->fini (NULL);
@@ -275,8 +301,6 @@ R_API bool r_asm_use_assembler(RAsm *a, const char *name) {
 
 // TODO: this can be optimized using r_str_hash()
 R_API int r_asm_use(RAsm *a, const char *name) {
-	const char *dirPrefix = r_sys_prefix (NULL);
-	char file[1024];
 	RAsmPlugin *h;
 	RListIter *iter;
 	if (!a || !name) {
@@ -285,12 +309,13 @@ R_API int r_asm_use(RAsm *a, const char *name) {
 	r_list_foreach (a->plugins, iter, h) {
 		if (!strcmp (h->name, name) && h->arch) {
 			if (!a->cur || (a->cur && strcmp (a->cur->arch, h->arch))) {
-				//const char *dop = r_config_get (core->config, "dir.opcodes");
-				// TODO: allow configurable path for sdb files
-				snprintf (file, sizeof (file), "%s/"R_ASM_OPCODES_PATH"/%s.sdb", dirPrefix, h->arch);
-				sdb_free (a->pair);
+				char *r2prefix = r_str_r2_prefix (R2_SDB_OPCODES);
+				char *file = r_str_newf ("%s/%s.sdb", r_str_get (r2prefix), h->arch);
 				r_asm_set_cpu (a, NULL);
+				sdb_free (a->pair);
 				a->pair = sdb_new (NULL, file, 0);
+				free (r2prefix);
+				free (file);
 			}
 			a->cur = h;
 			return true;
@@ -443,11 +468,16 @@ R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	if (a->ofilter) {
 		r_parse_parse (a->ofilter, op->buf_asm, op->buf_asm);
 	}
-	//XXX check against R_ASM_BUFSIZE other oob write
-	memcpy (op->buf, buf, R_MIN (R_ASM_BUFSIZE - 1, oplen));
-	const int addrbytes = a->user ? ((RCore *)a->user)->io->addrbytes : 1;
-	r_hex_bin2str (buf, R_MIN (addrbytes * oplen,
-		(sizeof (op->buf_hex) - 1) / 2), op->buf_hex);
+	// XXX this is crap and needs proper fix
+	int dstlen = R_MIN ((R_ASM_BUFSIZE -1), oplen);
+	int cpylen = R_MIN (dstlen, len);
+	if (cpylen > 0) {
+		memcpy (op->buf, buf, cpylen);
+		const int addrbytes = a->user ? ((RCore *)a->user)->io->addrbytes : 1;
+		int len = R_MIN (addrbytes * oplen, (sizeof (op->buf_hex) - 1) / 2);
+		// len is the length in input buffer, not len of output, so we must / 2
+		r_hex_bin2str (op->buf, len, op->buf_hex);
+	}
 	return ret;
 }
 
@@ -684,6 +714,8 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 	acode->code_align = 0;
 	memset (&op, 0, sizeof (op));
 
+	/* consider ,, an alias for a newline */
+	lbuf = r_str_replace (lbuf, ",,", "\n", true);
 	/* accept ';' as comments when input is multiline */
 	{
 		char *nl = strchr (lbuf, '\n');
@@ -763,7 +795,7 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 					isavrseparator (*ptr_start); ptr_start++);
 			} else {
 				for (ptr_start = buf_token; *ptr_start &&
-					ISSEPARATOR (*ptr_start); ptr_start++);
+					IS_SEPARATOR (*ptr_start); ptr_start++);
 			}
 			if (!strncmp (ptr_start, "/*", 2)) {
 				if (!strstr (ptr_start + 2, "*/")) {
@@ -896,8 +928,15 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 					acode->code_offset = a->pc;
 				} else if (!strncmp (ptr, ".data", 5)) {
 					acode->data_offset = a->pc;
+				} else if (!strncmp (ptr, ".incbin", 7)) {
+					if (ptr[7] != ' ') {
+						eprintf ("incbin missing filename\n");
+						continue;
+					}
+					ret = r_asm_pseudo_incbin (&op, ptr + 8);
 				} else {
 					eprintf ("Unknown directive (%s)\n", ptr);
+					free(lbuf);
 					return r_asm_code_free (acode);
 				}
 				if (!ret) {
@@ -905,6 +944,7 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 				}
 				if (ret < 0) {
 					eprintf ("!!! Oops\n");
+					free(lbuf);
 					return r_asm_code_free (acode);
 				}
 			} else { /* Instruction */
@@ -930,21 +970,35 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 			if (stage == STAGES - 1) {
 				if (ret < 1) {
 					eprintf ("Cannot assemble '%s' at line %d\n", ptr_start, linenum);
+					free(lbuf);
 					return r_asm_code_free (acode);
 				}
 				acode->len = idx + ret;
 				char *newbuf = realloc (acode->buf, (idx + ret) * 2);
 				if (!newbuf) {
+					free(lbuf);
 					return r_asm_code_free (acode);
 				}
 				acode->buf = (ut8*)newbuf;
-				newbuf = realloc (acode->buf_hex, strlen (acode->buf_hex) + strlen (op.buf_hex) + 1);
+				newbuf = realloc (acode->buf_hex, strlen (acode->buf_hex) + strlen (op.buf_hex) + r_buf_size (op.buf_inc) + 1);
 				if (!newbuf) {
+					free(lbuf);
 					return r_asm_code_free (acode);
 				}
 				acode->buf_hex = newbuf;
 				memcpy (acode->buf + idx, op.buf, ret);
 				strcat (acode->buf_hex, op.buf_hex);
+				if (r_buf_size (op.buf_inc) > 1) {
+					if (strlen (acode->buf_hex) > 0) {
+						strcat (acode->buf_hex, "\n");
+					}
+					char *s = r_buf_free_to_string (op.buf_inc);
+					if (s) {
+						strcat (acode->buf_hex, s);
+						free (s);
+					}
+				}
+
 			}
 		}
 	}
@@ -1021,12 +1075,13 @@ R_API char *r_asm_to_string(RAsm *a, ut64 addr, const ut8 *b, int l) {
 }
 
 R_API ut8 *r_asm_from_string(RAsm *a, ut64 addr, const char *b, int *l) {
-	RAsmCode *code;
 	r_asm_set_pc (a, addr);
-	code = r_asm_massemble (a, b);
+	RAsmCode *code = r_asm_massemble (a, b);
 	if (code) {
 		ut8 *buf = code->buf;
-		if (l) *l = code->len;
+		if (l) {
+			*l = code->len;
+		}
 		r_asm_code_free (code);
 		return buf;
 	}
