@@ -7,7 +7,7 @@
 #include <r_util.h>
 #include <r_lib.h>
 #include <r_asm.h>
-#include "dis-asm.h"
+#include "disas-asm.h"
 #include "../arch/arm/gnu/opcode-arm.h"
 
 #if 0
