@@ -9,7 +9,7 @@
 #include <r_util.h>
 #include <r_asm.h>
 
-#include "dis-asm.h"
+#include "disas-asm.h"
 
 
 int print_insn_big_nios2 (bfd_vma address, disassemble_info *info);
