@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2012-2015 - pancake */
+/* radare - LGPL - Copyright 2012-2018 - pancake */
 
 #include <stdio.h>
 #include <string.h>
@@ -10,11 +10,14 @@
 
 static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	struct v850_cmd cmd = {
+		.addr = a->pc,
 		.instr = "",
 		.operands = ""
 	};
-	if (len < 2) return -1;
-	int ret = v850_decode_command (buf, &cmd);
+	if (len < 2) {
+		return -1;
+	}
+	int ret = v850_decode_command (buf, len, &cmd);
 	if (ret > 0) {
 		snprintf (op->buf_asm, R_ASM_BUFSIZE, "%s %s",
 			  cmd.instr, cmd.operands);
