@@ -973,15 +973,17 @@ static void dump_r_bin_dwarf_debug_abbrev(FILE *f, RBinDwarfDebugAbbrev *da) {
 				"has children" : "no children");
 		fprintf (f, "Offset 0x%"PFMT64x"\n", da->decls[i].offset);
 
-		for (j = 0; j < da->decls[i].length; j++) {
-			attr_name = da->decls[i].specs[j].attr_name;
-			attr_form = da->decls[i].specs[j].attr_form;
-			if (attr_name && attr_form &&
-				attr_name <= DW_AT_vtable_elem_location &&
-				attr_form <= DW_FORM_indirect) {
+		if (da->decls[i].specs) {
+			for (j = 0; j < da->decls[i].length; j++) {
+				attr_name = da->decls[i].specs[j].attr_name;
+				attr_form = da->decls[i].specs[j].attr_form;
+				if (attr_name && attr_form &&
+					attr_name <= DW_AT_vtable_elem_location &&
+					attr_form <= DW_FORM_indirect) {
 					fprintf (f, "    %s %s\n",
-						dwarf_attr_encodings[attr_name],
-						dwarf_attr_form_encodings[attr_form]);
+							dwarf_attr_encodings[attr_name],
+							dwarf_attr_form_encodings[attr_form]);
+				}
 			}
 		}
 	}
@@ -1359,7 +1361,11 @@ static const ut8 *r_bin_dwarf_parse_comp_unit(Sdb *s, const ut8 *obuf,
 					&cu->hdr, debug_str, debug_str_len);
 			if (cu->dies[cu->length].attr_values[i].name == DW_AT_comp_dir) {
 				const char *name = cu->dies[cu->length].attr_values[i].encoding.str_struct.string;
-				sdb_set (s, "DW_AT_comp_dir", name, 0);
+				if ((size_t)name > 1024) { // solve some null derefs
+					sdb_set (s, "DW_AT_comp_dir", name, 0);
+				} else {
+					eprintf ("Invalid string pointer at %p\n", name);
+				}
 			}
 			cu->dies[cu->length].length++;
 		}
