@@ -100,6 +100,9 @@ struct PE_(r_bin_pe_obj_t) {
 	PE_(image_metadata_header) * metadata_header;
 	PE_(image_metadata_stream) * *streams;
 
+	/* store the section information for future use */
+	struct r_bin_pe_section_t *sections;
+
 	// these values define the real offset into the untouched binary
 	ut64 nt_header_offset;
 	ut64 section_header_offset;
@@ -138,7 +141,6 @@ char* PE_(r_bin_pe_get_os)(struct PE_(r_bin_pe_obj_t)* bin);
 char* PE_(r_bin_pe_get_class)(struct PE_(r_bin_pe_obj_t)* bin);
 int PE_(r_bin_pe_get_bits)(struct PE_(r_bin_pe_obj_t)* bin);
 int PE_(r_bin_pe_get_section_alignment)(struct PE_(r_bin_pe_obj_t)* bin);
-struct r_bin_pe_section_t* PE_(r_bin_pe_get_sections)(struct PE_(r_bin_pe_obj_t)* bin);
 char* PE_(r_bin_pe_get_subsystem)(struct PE_(r_bin_pe_obj_t)* bin);
 int PE_(r_bin_pe_is_dll)(struct PE_(r_bin_pe_obj_t)* bin);
 int PE_(r_bin_pe_is_big_endian)(struct PE_(r_bin_pe_obj_t)* bin);
