@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2011-2017 - pancake, h4ng3r */
+/* radare - LGPL - Copyright 2011-2018 - pancake, h4ng3r */
 
 #include <r_cons.h>
 #include <r_types.h>
@@ -407,7 +407,7 @@ static void dex_parse_debug_item(RBinFile *binfile, RBinDexObj *bin,
 		parameters_size--;
 	}
 
-	if (!p4) {
+	if (!p4 || p4 >= p4_end) {
 		free (debug_positions);
 		free (params);
 		free (debug_locals);
@@ -620,7 +620,8 @@ static void dex_parse_debug_item(RBinFile *binfile, RBinDexObj *bin,
 #endif
 		RBinDwarfRow *rbindwardrow = R_NEW0 (RBinDwarfRow);
 		if (!rbindwardrow) {
-			return;
+			dexdump = false;
+			break;
 		}
 		if (line) {
 			rbindwardrow->file = strdup (line);
@@ -721,6 +722,10 @@ static void *load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sd
 	return res;
 }
 
+static void * load_buffer(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
+	return r_bin_dex_new_buf (buf);
+}
+
 static bool load(RBinFile *bf) {
 	const ut8 *bytes = bf ? r_buf_buffer (bf->buf) : NULL;
 	ut64 sz = bf ? r_buf_size (bf->buf): 0;
@@ -1089,10 +1094,10 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 	int i, left;
 	ut64 omi = 0;
 	bool catchAll;
-	ut16 regsz, ins_size, outs_size, tries_size;
-	ut16 handler_off, start_addr, insn_count;
-	ut32 debug_info_off, insns_size;
-	const ut8 *encoded_method_addr;
+	ut16 regsz = 0, ins_size = 0, outs_size = 0, tries_size = 0;
+	ut16 handler_off, start_addr, insn_count = 0;
+	ut32 debug_info_off = 0, insns_size = 0;
+	const ut8 *encoded_method_addr = NULL;
 
 	if (DM > 4096) {
 		eprintf ("This DEX is probably corrupted. Chopping DM from %d to 4KB\n", (int)DM);
@@ -1128,7 +1133,7 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 		// TODO: check size
 		// ut64 prolog_size = 2 + 2 + 2 + 2 + 4 + 4;
 		ut64 v2, handler_type, handler_addr;
-		int t;
+		int t = 0;
 		if (MC > 0) {
 			// TODO: parse debug info
 			// XXX why binfile->buf->base???
@@ -1273,7 +1278,7 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 			// if method has code *addr points to code
 			// otherwise it points to the encoded method
 			if (MC > 0) {
-				sym->type = r_str_const ("FUNC");
+				sym->type = r_str_const (R_BIN_TYPE_FUNC_STR);
 				sym->paddr = MC;// + 0x10;
 				sym->vaddr = MC;// + 0x10;
 			} else {
@@ -1282,9 +1287,9 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 				sym->vaddr = encoded_method_addr - binfile->buf->buf;
 			}
 			if ((MA & 0x1) == 0x1) {
-				sym->bind = r_str_const ("GLOBAL");
+				sym->bind = r_str_const (R_BIN_BIND_GLOBAL_STR);
 			} else {
-				sym->bind = r_str_const ("LOCAL");
+				sym->bind = r_str_const (R_BIN_BIND_LOCAL_STR);
 			}
 
 			sym->method_flags = get_method_flags (MA);
@@ -1328,7 +1333,7 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 				if (!mdb) {
 					mdb = sdb_new0 ();
 				}
-				sdb_num_set (mdb, sdb_fmt (0, "method.%d", MI), sym->paddr, 0);
+				sdb_num_set (mdb, sdb_fmt ("method.%d", MI), sym->paddr, 0);
 				// -----------------
 				// WORK IN PROGRESS
 				// -----------------
@@ -1338,7 +1343,7 @@ static const ut8 *parse_dex_class_method(RBinFile *binfile, RBinDexObj *bin,
 						if (!cdb) {
 							cdb = sdb_new0 ();
 						}
-						sdb_num_set (cdb, sdb_fmt (0, "%d", c->class_id), sym->paddr, 0);
+						sdb_num_set (cdb, sdb_fmt ("%d", c->class_id), sym->paddr, 0);
 					}
 				}
 #endif
@@ -1480,13 +1485,20 @@ static void parse_class(RBinFile *binfile, RBinDexObj *bin, RBinDexClass *c,
 	    bin->header.data_offset < c->interfaces_offset &&
 	    c->interfaces_offset <
 		    bin->header.data_offset + bin->header.data_size) {
-		p = r_buf_get_at (binfile->buf, c->interfaces_offset, NULL);
+		int left;
+		p = r_buf_get_at (binfile->buf, c->interfaces_offset, &left);
+		if (left < 4) {
+			return;
+		}
 		int types_list_size = r_read_le32 (p);
 		if (types_list_size < 0 || types_list_size >= bin->header.types_size ) {
 			return;
 		}
 		for (z = 0; z < types_list_size; z++) {
-			int t = r_read_le16 (p + 4 + z * 2);
+			ut16 le16;
+			ut32 off = c->interfaces_offset + 4 + (z * 2);
+			r_buf_read_at (binfile->buf, off, (ut8*)&le16, sizeof (le16));
+			int t = r_read_le16 (&le16);
 			if (t > 0 && t < bin->header.types_size ) {
 				int tid = bin->types[t].descriptor_id;
 				if (dexdump) {
@@ -1701,14 +1713,14 @@ static int dex_loadcode(RBinFile *bf, RBinDexObj *bin) {
 					return false;
 				}
 				sym->name = r_str_newf ("imp.%s", imp->name);
-				sym->type = r_str_const ("FUNC");
+				sym->type = r_str_const (R_BIN_TYPE_FUNC_STR);
 				sym->bind = r_str_const ("NONE");
 				//XXX so damn unsafe check buffer boundaries!!!!
 				//XXX use r_buf API!!
 				sym->paddr = sym->vaddr = bin->b->base + bin->header.method_offset + (sizeof (struct dex_method_t) * i) ;
 				sym->ordinal = sym_count++;
 				r_list_append (bin->methods_list, sym);
-				sdb_num_set (mdb, sdb_fmt (0, "method.%d", i), sym->paddr, 0);
+				sdb_num_set (mdb, sdb_fmt ("method.%d", i), sym->paddr, 0);
 
 			}
 			free (signature);
@@ -1785,7 +1797,7 @@ static RList *entries(RBinFile *bf) {
 	// STEP 1. ".onCreate(Landroid/os/Bundle;)V"
 	r_list_foreach (bin->methods_list, iter, m) {
 		if (strlen (m->name) > 30 && m->bind &&
-			(!strcmp (m->bind, "LOCAL") || !strcmp (m->bind, "GLOBAL")) &&
+			(!strcmp (m->bind, R_BIN_BIND_LOCAL_STR) || !strcmp (m->bind, R_BIN_BIND_GLOBAL_STR)) &&
 		    !strcmp (m->name + strlen (m->name) - 31,
 			     ".onCreate(Landroid/os/Bundle;)V")) {
 			if (!already_entry (ret, m->paddr)) {
@@ -1828,7 +1840,7 @@ static RList *entries(RBinFile *bf) {
 
 static ut64 offset_of_method_idx(RBinFile *bf, struct r_bin_dex_obj_t *dex, int idx) {
 	// ut64 off = dex->header.method_offset + idx;
-	return sdb_num_get (mdb, sdb_fmt (0, "method.%d", idx), 0);
+	return sdb_num_get (mdb, sdb_fmt ("method.%d", idx), 0);
 }
 
 // TODO: change all return type for all getoffset
@@ -1902,7 +1914,7 @@ static RList *sections(RBinFile *bf) {
 		strcpy (ptr->name, "header");
 		ptr->size = ptr->vsize = sizeof (struct dex_header_t);
 		ptr->paddr= ptr->vaddr = 0;
-		ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_MAP;
+		ptr->srwx = R_BIN_SCN_READABLE;
 		ptr->add = true;
 		r_list_append (ret, ptr);
 	}
@@ -1912,7 +1924,7 @@ static RList *sections(RBinFile *bf) {
 		ptr->paddr= ptr->vaddr = sizeof (struct dex_header_t);
 		ptr->size = bin->code_from - ptr->vaddr; // fix size
 		ptr->vsize = ptr->size;
-		ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_MAP;
+		ptr->srwx = R_BIN_SCN_READABLE;
 		ptr->add = true;
 		r_list_append (ret, ptr);
 	}
@@ -1921,7 +1933,7 @@ static RList *sections(RBinFile *bf) {
 		ptr->vaddr = ptr->paddr = bin->code_from; //ptr->vaddr = fsym;
 		ptr->size = bin->code_to - ptr->paddr;
 		ptr->vsize = ptr->size;
-		ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+		ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
 		ptr->add = true;
 		r_list_append (ret, ptr);
 	}
@@ -1937,7 +1949,7 @@ static RList *sections(RBinFile *bf) {
 			// hacky workaround
 			//ptr->size = ptr->vsize = 1024;
 		}
-		ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_MAP; //|2;
+		ptr->srwx = R_BIN_SCN_READABLE; //|2;
 		ptr->add = true;
 		r_list_append (ret, ptr);
 	}
@@ -2011,6 +2023,7 @@ RBinPlugin r_bin_plugin_dex = {
 	.get_sdb = &get_sdb,
 	.load = &load,
 	.load_bytes = load_bytes,
+	.load_buffer = &load_buffer,
 	.check_bytes = check_bytes,
 	.baddr = baddr,
 	.entries = entries,
