@@ -66,7 +66,7 @@ static RList* entries(RBinFile *bf) {
 		ret->free = free;
 		if (!(ptr = R_NEW0 (RBinAddr)))
 			return ret;
-		ptr->paddr = ptr->vaddr = ptr->haddr = 0x100;
+		ptr->paddr = ptr->vaddr = ptr->hpaddr = 0x100;
 		r_list_append (ret, ptr);
 	}
 	return ret;
@@ -108,7 +108,7 @@ static RList* sections(RBinFile *bf){
 	rombank[0]->size = 0x4000;
 	rombank[0]->vsize = 0x4000;
 	rombank[0]->vaddr = 0;
-	rombank[0]->srwx = r_str_rwx ("mrx");
+	rombank[0]->srwx = r_str_rwx ("rx");
 	rombank[0]->add = true;
 
 	r_list_append (ret, rombank[0]);
@@ -119,7 +119,7 @@ static RList* sections(RBinFile *bf){
 		rombank[i]->paddr = i*0x4000;
 		rombank[i]->vaddr = i*0x10000-0xc000;			//spaaaaaaaaaaaaaaaace!!!
 		rombank[i]->size = rombank[i]->vsize = 0x4000;
-		rombank[i]->srwx = r_str_rwx ("mrx");
+		rombank[i]->srwx = r_str_rwx ("rx");
 		rombank[i]->add = true;
 		r_list_append (ret,rombank[i]);
 	}
@@ -284,7 +284,7 @@ RList *mem (RBinFile *bf) {
 	return ret;
 }
 
-struct r_bin_plugin_t r_bin_plugin_ningb = {
+RBinPlugin r_bin_plugin_ningb = {
 	.name = "ningb",
 	.desc = "Gameboy format r_bin plugin",
 	.license = "LGPL3",
