@@ -90,7 +90,7 @@ static void addrom(RList *ret, const char *name, int i, ut64 paddr, ut64 vaddr,
 	ptr->paddr = paddr;
 	ptr->vaddr = vaddr;
 	ptr->size = ptr->vsize = size;
-	ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+	ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
 	ptr->add = true;
 	r_list_append (ret, ptr);
 }
