@@ -65,7 +65,7 @@ static void * load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, S
 			}
 		}
 		if (i >= MACHINES_MAX) {
-			eprintf("Unsupported machine: %s\n", machine);
+			eprintf ("Unsupported machine type\n");
 			free (res);
 			return NULL;
 		}
@@ -122,7 +122,7 @@ static RList *mem(RBinFile *bf) {
 	m->name = strdup ("RAM");
 	m->addr = 0;	// start address
 	m->size = _machines[vsf_obj->machine_idx].ram_size;
-	m->perms = r_str_rwx ("mrwx");
+	m->perms = r_str_rwx ("rwx");
 	r_list_append (ret, m);
 	return ret;
 }
@@ -157,7 +157,7 @@ static RList* sections(RBinFile* bf) {
 			ptr->size = 1024 * 8; // (8k)
 			ptr->vaddr = 0xa000;
 			ptr->vsize = 1024 * 8;	// BASIC size (8k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -172,7 +172,7 @@ static RList* sections(RBinFile* bf) {
 			ptr->size = 1024 * 8; // (8k)
 			ptr->vaddr = 0xe000;
 			ptr->vsize = 1024 * 8;	// KERNAL size (8k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -190,7 +190,7 @@ static RList* sections(RBinFile* bf) {
 			ptr->size = 1024 * 28; // (28k)
 			ptr->vaddr = 0x4000;
 			ptr->vsize = 1024 * 28;	// BASIC size (28k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -206,7 +206,7 @@ static RList* sections(RBinFile* bf) {
 			ptr->size = 1024 * 4; // (4k)
 			ptr->vaddr = 0xb000;
 			ptr->vsize = 1024 * 4;	// BASIC size (4k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -221,7 +221,7 @@ static RList* sections(RBinFile* bf) {
 			ptr->size = 1024 * 4; // (4k)
 			ptr->vaddr = 0xc000;
 			ptr->vsize = 1024 * 4;	// BASIC size (4k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -236,7 +236,7 @@ static RList* sections(RBinFile* bf) {
 			ptr->size = 1024 * 8; // (8k)
 			ptr->vaddr = 0xe000;
 			ptr->vsize = 1024 * 8;	// KERNAL size (8k)
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -257,7 +257,7 @@ static RList* sections(RBinFile* bf) {
 			ptr->size = size;
 			ptr->vaddr = 0x0;
 			ptr->vsize = size;
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE | R_BIN_SCN_EXECUTABLE;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 		} else {
@@ -274,7 +274,7 @@ static RList* sections(RBinFile* bf) {
 			ptr->size = size;
 			ptr->vaddr = 0x0;
 			ptr->vsize = size;
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE | R_BIN_SCN_EXECUTABLE;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 
@@ -286,7 +286,7 @@ static RList* sections(RBinFile* bf) {
 			ptr->size = size;
 			ptr->vaddr = 0x0;
 			ptr->vsize = size;
-			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE | R_BIN_SCN_EXECUTABLE | R_BIN_SCN_MAP;
+			ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE | R_BIN_SCN_EXECUTABLE;
 			ptr->add = true;
 			r_list_append (ret, ptr);
 		}
@@ -538,7 +538,7 @@ static RList* entries(RBinFile *bf) {
 	return ret;
 }
 
-struct r_bin_plugin_t r_bin_plugin_vsf = {
+RBinPlugin r_bin_plugin_vsf = {
 	.name = "vsf",
 	.desc = "VICE Snapshot File",
 	.license = "LGPL3",
