@@ -19,11 +19,7 @@ static bool checkExtract() {
 }
 
 static bool checkCurl() {
-#if __WINDOWS__ && !__CYGWIN__
-	const char nul[] = "nul";
-#else
-	const char nul[] = "/dev/null";
-#endif
+	const char nul[] = R_SYS_DEVNULL;
 	if (r_sys_cmdf ("curl --version > %s", nul) != 0) {
 		return false;
 	}
@@ -37,7 +33,13 @@ static int download(struct SPDBDownloader *pd) {
 	char *abspath_to_archive = NULL;
 	char *archive_name = NULL;
 	size_t archive_name_len = 0;
-	const char *basepath = ".";
+	char *symbol_store_path = NULL;
+	char *dbg_file = NULL;
+	char *guid = NULL;
+	char *archive_name_escaped  = NULL;
+	char *user_agent = NULL;
+	char *symbol_server = NULL;
+
 	int res = 0;
 	int cmd_ret;
 	if (!opt->dbg_file || !*opt->dbg_file) {
@@ -55,42 +57,49 @@ static int download(struct SPDBDownloader *pd) {
 	}
 	memcpy (archive_name, opt->dbg_file, archive_name_len + 1);
 	archive_name[archive_name_len - 1] = '_';
-	if (opt->path && *opt->path) {
-		basepath = opt->path;
-	}
+	symbol_store_path = r_str_escape (opt->symbol_store_path);
+	dbg_file = r_str_escape (opt->dbg_file);
+	guid = r_str_escape (opt->guid);
+	archive_name_escaped = r_str_escape (archive_name);
+	user_agent = r_str_escape (opt->user_agent);
+	symbol_server = r_str_escape (opt->symbol_server);
+
 	if (checkExtract () || opt->extract == 0) {
 		res = 1;
-		abspath_to_archive = r_str_newf ("%s%s%s", basepath,
-		                                 R_SYS_DIR, archive_name);
-		curl_cmd = r_str_newf ("curl -sfA \"%s\" \"%s/%s/%s/%s\" -o \"%s\"",
-		                       opt->user_agent,
-		                       opt->symbol_server,
-		                       opt->dbg_file,
-		                       opt->guid,
-		                       archive_name,
+		abspath_to_archive = r_str_newf ("%s%s%s%s%s%s%s",
+						    symbol_store_path, R_SYS_DIR,
+						    dbg_file, R_SYS_DIR,
+						    guid, R_SYS_DIR,
+						    archive_name_escaped);
+
+		curl_cmd = r_str_newf ("curl -sfLA \"%s\" \"%s/%s/%s/%s\" --create-dirs -o \"%s\"",
+		                       user_agent,
+		                       symbol_server,
+							   dbg_file,
+							   guid,
+		                       archive_name_escaped,
 		                       abspath_to_archive);
 #if __WINDOWS__ && !__CYGWIN__
 		const char *cabextractor = "expand";
 		const char *format = "%s %s %s";
 		char *abspath_to_file = strdup (abspath_to_archive);
-		if (abspath_to_file) {
-			int abspath_to_archive_len = archive_name_len + strlen (basepath) + 2;
-			abspath_to_file[abspath_to_archive_len - 2] = 'b';
-			// extractor_cmd -> %1 %2 %3
-			// %1 - 'expand'
-			// %2 - absolute path to archive
-			// %3 - absolute path to file that will be dearchive
-			extractor_cmd = r_str_newf (format, cabextractor,
-				abspath_to_archive, abspath_to_file);
-		}
+		abspath_to_file[strlen (abspath_to_file) - 1] = 'b';
+
+		// extractor_cmd -> %1 %2 %3
+		// %1 - 'expand'
+		// %2 - absolute path to archive
+		// %3 - absolute path to file that will be dearchive
+		extractor_cmd = r_str_newf (format, cabextractor,
+			abspath_to_archive, abspath_to_file);
 #else
 		const char *cabextractor = "cabextract";
 		const char *format = "%s -d \"%s\" \"%s\"";
+		char *abspath_to_dir = r_file_dirname (abspath_to_archive);
 		// cabextract -d %1 %2
 		// %1 - path to directory where to extract all files from cab archive
 		// %2 - absolute path to cab archive
-		extractor_cmd = r_str_newf (format,
-		                            cabextractor, basepath, abspath_to_archive);
+		extractor_cmd = r_str_newf (format, cabextractor, abspath_to_dir, abspath_to_archive);
+		R_FREE (abspath_to_dir);
 #endif
 		if ((cmd_ret = r_sys_cmd (curl_cmd) != 0)) {
 			eprintf("curl exited with error %d\n", cmd_ret);
@@ -110,14 +119,20 @@ static int download(struct SPDBDownloader *pd) {
 		eprintf ("Falling back to uncompressed pdb\n");
 		res = 1;
 		archive_name[archive_name_len - 1] = 'b';
-		abspath_to_archive = r_str_newf ("%s%s%s", basepath,
-		                                 R_SYS_DIR, archive_name);
-		curl_cmd = r_str_newf ("curl -sfA \"%s\" \"%s/%s/%s/%s\" -o \"%s\"",
+		R_FREE (archive_name_escaped);
+		archive_name_escaped = r_str_escape (archive_name);
+		abspath_to_archive = r_str_newf("%s%s%s%s%s%s%s",
+		    symbol_store_path, R_SYS_DIR,
+		    dbg_file, R_SYS_DIR,
+		    guid, R_SYS_DIR,
+		    archive_name_escaped);
+
+		curl_cmd = r_str_newf ("curl -sfLA \"%s\" \"%s/%s/%s/%s\" --create-dirs -o \"%s\"",
 		                       opt->user_agent,
 		                       opt->symbol_server,
 		                       opt->dbg_file,
 		                       opt->guid,
-		                       archive_name,
+		                       archive_name_escaped,
 		                       abspath_to_archive);
 		if ((cmd_ret = r_sys_cmd (curl_cmd) != 0)) {
 			eprintf("curl exited with error %d\n", cmd_ret);
@@ -128,6 +143,12 @@ static int download(struct SPDBDownloader *pd) {
 	}
 	R_FREE (archive_name);
 	R_FREE (extractor_cmd);
+	R_FREE (symbol_store_path);
+	R_FREE (dbg_file);
+	R_FREE (guid);
+	R_FREE (archive_name_escaped);
+	R_FREE (user_agent);
+	R_FREE (symbol_server);
 	return res;
 }
 
@@ -140,7 +161,7 @@ void init_pdb_downloader(SPDBDownloaderOpt *opt, SPDBDownloader *pd) {
 	pd->opt->guid = strdup (opt->guid);
 	pd->opt->symbol_server = strdup (opt->symbol_server);
 	pd->opt->user_agent = strdup (opt->user_agent);
-	pd->opt->path = strdup (opt->path);
+	pd->opt->symbol_store_path = strdup (opt->symbol_store_path);
 	pd->opt->extract = opt->extract;
 	pd->download = download;
 }
@@ -150,14 +171,13 @@ void deinit_pdb_downloader(SPDBDownloader *pd) {
 	R_FREE (pd->opt->guid);
 	R_FREE (pd->opt->symbol_server);
 	R_FREE (pd->opt->user_agent);
-	R_FREE (pd->opt->path);
+	R_FREE (pd->opt->symbol_store_path);
 	R_FREE (pd->opt);
 	pd->download = 0;
 }
 
 int r_bin_pdb_download(RCore *core, int isradjson, int *actions_done, SPDBOptions *options) {
 	int ret;
-	char *path;
 	SPDBDownloaderOpt opt;
 	SPDBDownloader pdb_downloader;
 	RBinInfo *info = r_bin_get_info (core->bin);
@@ -172,13 +192,11 @@ int r_bin_pdb_download(RCore *core, int isradjson, int *actions_done, SPDBOption
 		return 1;
 	}
 
-	path = info->file ? r_file_dirname (info->file) : strdup (".");
-
 	opt.dbg_file = info->debug_file_name;
 	opt.guid = info->guid;
 	opt.symbol_server = options->symbol_server;
 	opt.user_agent = options->user_agent;
-	opt.path = path;
+	opt.symbol_store_path = options->symbol_store_path;
 	opt.extract = options->extract;
 
 	init_pdb_downloader (&opt, &pdb_downloader);
@@ -195,6 +213,5 @@ int r_bin_pdb_download(RCore *core, int isradjson, int *actions_done, SPDBOption
 	}
 	deinit_pdb_downloader (&pdb_downloader);
 
-	free (path);
 	return 0;
 }
