@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2016 pancake */
+/* radare - LGPL - Copyright 2009-2018 pancake */
 
 #include <r_bp.h>
 #include <config.h>
@@ -25,7 +25,7 @@ R_API void r_bp_restore_one(RBreakpoint *bp, RBreakpointItem *b, bool set) {
  * reflect all r_bp stuff in the process using dbg->bp_write or ->breakpoint
  */
 R_API int r_bp_restore(RBreakpoint *bp, bool set) {
-	return r_bp_restore_except (bp, set, 0);
+	return r_bp_restore_except (bp, set, UT64_MAX);
 }
 
 /**
