@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2008-2017 - pancake, Jody Frankowski */
+/* radare2 - LGPL - Copyright 2008-2018 - pancake, Jody Frankowski */
 
 #include <r_cons.h>
 #include <r_print.h>
@@ -15,7 +15,8 @@
 
 R_LIB_VERSION (r_cons);
 
-static RCons r_cons_instance;
+static RConsContext r_cons_context_default = {{{{0}}}};
+static RCons r_cons_instance = {0};
 #define I r_cons_instance
 
 //this structure goes into cons_stack when r_cons_push/pop
@@ -28,8 +29,8 @@ typedef struct {
 
 typedef struct {
 	bool breaked;
-	void *data;
 	RConsEvent event_interrupt;
+	void *event_interrupt_data;
 } RConsBreakStack;
 
 static void break_stack_free(void *ptr) {
@@ -40,16 +41,80 @@ static void break_stack_free(void *ptr) {
 static void cons_stack_free(void *ptr) {
 	RConsStack *s = (RConsStack *)ptr;
 	free (s->buf);
+	if (s->grep) {
+		free (s->grep->str);
+	}
 	free (s->grep);
 	free (s);
 }
 
-static void break_signal(int sig) {
-	I.breaked = true;
-	r_print_set_interrupted (I.breaked);
-	if (I.event_interrupt) {
-		I.event_interrupt (I.data);
+static RConsStack *cons_stack_dump(bool recreate) {
+	RConsStack *data = R_NEW0 (RConsStack);
+	if (!data) {
+		return NULL;
 	}
+
+	if (I.context->buffer) {
+		data->buf = I.context->buffer;
+		data->buf_len = I.context->buffer_len;
+		data->buf_size = I.context->buffer_sz;
+	}
+
+	data->grep = R_NEW0 (RConsGrep);
+	if (data->grep) {
+		memcpy (data->grep, &I.context->grep, sizeof (RConsGrep));
+		if (I.context->grep.str) {
+			data->grep->str = strdup (I.context->grep.str);
+		}
+	}
+
+	if (recreate && I.context->buffer_sz > 0) {
+		I.context->buffer = malloc (I.context->buffer_sz);
+		if (!I.context->buffer) {
+			I.context->buffer = data->buf;
+			free (data);
+			return NULL;
+		}
+	} else {
+		I.context->buffer = NULL;
+	}
+
+	return data;
+}
+
+static void cons_stack_load(RConsStack *data, bool free_current) {
+	if (free_current) {
+		free (I.context->buffer);
+	}
+	I.context->buffer = data->buf;
+	data->buf = NULL;
+	I.context->buffer_len = data->buf_len;
+	I.context->buffer_sz = data->buf_size;
+
+	if (data->grep) {
+		free (I.context->grep.str);
+		memcpy (&I.context->grep, data->grep, sizeof (RConsGrep));
+	}
+}
+
+static void cons_context_init(RConsContext *context) {
+	context->breaked = false;
+	context->buffer = NULL;
+	context->buffer_sz = 0;
+	context->buffer_len = 0;
+	context->cons_stack = r_stack_newf (6, cons_stack_free);
+	context->break_stack = r_stack_newf (6, break_stack_free);
+	context->event_interrupt = NULL;
+	context->event_interrupt_data = NULL;
+}
+
+static void cons_context_deinit(RConsContext *context) {
+	r_stack_free (context->cons_stack);
+	r_stack_free (context->break_stack);
+}
+
+static void break_signal(int sig) {
+	r_cons_context_break (&r_cons_context_default);
 }
 
 static inline void r_cons_write(const char *buf, int len) {
@@ -71,67 +136,33 @@ static inline void r_cons_write(const char *buf, int len) {
 #endif
 }
 
-R_API char *r_cons_color_random_string(int bg) {
-	int r, g, b;
-	if (I.truecolor > 0) {
-		char out[32];
-		r = r_num_rand (0xff);
-		g = r_num_rand (0xff);
-		b = r_num_rand (0xff);
-		r_cons_rgb_str (out, r, g, b, bg);
-		return strdup (out);
+R_API RColor r_cons_color_random(ut8 alpha) {
+	RColor rcolor = {0};
+	if (I.color > COLOR_MODE_16) {
+		rcolor.r = r_num_rand (0xff);
+		rcolor.g = r_num_rand (0xff);
+		rcolor.b = r_num_rand (0xff);
+		rcolor.a = alpha;
+		return rcolor;
 	}
-	// random ansi
-	const char *color = "white";
-	r = r_num_rand (8);
+	int r = r_num_rand (16);
 	switch (r) {
-	case 0: color = "red"; break;
-	case 1: color = "white"; break;
-	case 2: color = "green"; break;
-	case 3: color = "magenta"; break;
-	case 4: color = "yellow"; break;
-	case 5: color = "cyan"; break;
-	case 6: color = "blue"; break;
-	case 7: color = "gray"; break;
+	case 0: case 1: rcolor = (RColor) RColor_RED; break;
+	case 2: case 3: rcolor = (RColor) RColor_WHITE; break;
+	case 4: case 5: rcolor = (RColor) RColor_GREEN; break;
+	case 6: case 7: rcolor = (RColor) RColor_MAGENTA; break;
+	case 8: case 9: rcolor = (RColor) RColor_YELLOW; break;
+	case 10: case 11: rcolor = (RColor) RColor_CYAN; break;
+	case 12: case 13: rcolor = (RColor) RColor_BLUE; break;
+	case 14: case 15: rcolor = (RColor) RColor_GRAY; break;
 	}
-	return strdup (color);
+	if (r & 1) {
+		rcolor.attr = R_CONS_ATTR_BOLD;
+	}
+	return rcolor;
 }
 
-R_API char *r_cons_color_random(int bg) {
-	int r, g, b;
-	if (I.truecolor > 0) {
-		char out[32];
-		r = r_num_rand (0xff);
-		g = r_num_rand (0xff);
-		b = r_num_rand (0xff);
-		r_cons_rgb_str (out, r, g, b, bg);
-		return strdup (out);
-	}
-	const char *color = Color_RESET;
-	// random ansi
-	r = r_num_rand (16);
-	switch (r) {
-	case 0: color = Color_RED; break;
-	case 1: color = Color_BRED; break;
-	case 2: color = Color_WHITE; break;
-	case 3: color = Color_BWHITE; break;
-	case 4: color = Color_GREEN; break;
-	case 5: color = Color_BGREEN; break;
-	case 6: color = Color_MAGENTA; break;
-	case 7: color = Color_BMAGENTA; break;
-	case 8: color = Color_YELLOW; break;
-	case 9: color = Color_BYELLOW; break;
-	case 10: color = Color_CYAN; break;
-	case 11: color = Color_BCYAN; break;
-	case 12: color = Color_BLUE; break;
-	case 13: color = Color_BBLUE; break;
-	case 14: color = Color_GRAY; break;
-	case 15: color = Color_BGRAY; break;
-	}
-	return strdup (color);
-}
-
-R_API void r_cons_color (int fg, int r, int g, int b) {
+R_API void r_cons_color(int fg, int r, int g, int b) {
 	int k;
 	r = R_DIM (r, 0, 255);
 	g = R_DIM (g, 0, 255);
@@ -155,7 +186,7 @@ R_API void r_cons_println(const char* str) {
 R_API void r_cons_strcat_justify(const char *str, int j, char c) {
 	int i, o, len;
 	for (o = i = len = 0; str[i]; i++, len++) {
-		if (str[i]=='\n') {
+		if (str[i] == '\n') {
 			r_cons_memset (' ', j);
 			if (c) {
 				r_cons_memset (c, 1);
@@ -174,64 +205,81 @@ R_API void r_cons_strcat_justify(const char *str, int j, char c) {
 	}
 }
 
-R_API RCons *r_cons_singleton () {
+R_API RCons *r_cons_singleton() {
 	return &I;
 }
 
 R_API void r_cons_break_clear() {
-	I.breaked = false;
+	I.context->breaked = false;
+}
+
+R_API void r_cons_context_break_push(RConsContext *context, RConsBreak cb, void *user, bool sig) {
+	if (!context->break_stack) {
+		return;
+	}
+
+	//if we don't have any element in the stack start the signal
+	RConsBreakStack *b = R_NEW0 (RConsBreakStack);
+	if (!b) return;
+	if (r_stack_is_empty (context->break_stack)) {
+#if __UNIX__ || __CYGWIN__
+		if (sig && r_cons_context_is_main ()) {
+			signal (SIGINT, break_signal);
+		}
+#endif
+		context->breaked = false;
+	}
+	//save the actual state
+	b->event_interrupt = context->event_interrupt;
+	b->event_interrupt_data = context->event_interrupt_data;
+	r_stack_push (context->break_stack, b);
+	//configure break
+	context->event_interrupt = cb;
+	context->event_interrupt_data = user;
+}
+
+R_API void r_cons_context_break_pop(RConsContext *context, bool sig) {
+	if (!context->break_stack) {
+		return;
+	}
+	//restore old state
+	RConsBreakStack *b = NULL;
+	b = r_stack_pop (context->break_stack);
+	if (b) {
+		context->event_interrupt = b->event_interrupt;
+		context->event_interrupt_data = b->event_interrupt_data;
+		break_stack_free (b);
+	} else {
+		//there is not more elements in the stack
+#if __UNIX__ || __CYGWIN__
+		if (sig && r_cons_context_is_main ()) {
+			signal (SIGINT, SIG_IGN);
+		}
+#endif
+		context->breaked = false;
+	}
 }
 
 R_API void r_cons_break_push(RConsBreak cb, void *user) {
-	if (I.break_stack) {
-		//if we don't have any element in the stack start the signal
-		RConsBreakStack *b = R_NEW0 (RConsBreakStack);
-		if (!b) return;
-		if (r_stack_is_empty (I.break_stack)) {
-#if __UNIX__ || __CYGWIN__
-			signal (SIGINT, break_signal);
-#endif
-			I.breaked = false;
-		}
-		//save the actual state
-		b->event_interrupt = I.event_interrupt;
-		b->data = I.data;
-		r_stack_push (I.break_stack, b);
-		//configure break
-		I.event_interrupt = cb;
-		I.data = user;
-	}
+	r_cons_context_break_push (I.context, cb, user, true);
 }
 
 R_API void r_cons_break_pop() {
-	//restore old state
-	if (I.break_stack) {
-		RConsBreakStack *b = NULL;
-		r_print_set_interrupted (I.breaked);
-		b = r_stack_pop (I.break_stack);
-		if (b) {
-			I.event_interrupt = b->event_interrupt;
-			I.data = b->data;
-			break_stack_free (b);
-		} else {
-			//there is not more elements in the stack
-#if __UNIX__ || __CYGWIN__
-			signal (SIGINT, SIG_IGN);
-#endif
-			I.breaked = false;
-		}
-	}
+	r_cons_context_break_pop (I.context, true);
 }
 
 R_API bool r_cons_is_breaked() {
+	if (I.cb_break) {
+		I.cb_break (I.user);
+	}
 	if (I.timeout) {
 		if (r_sys_now () > I.timeout) {
-			I.breaked = true;
+			I.context->breaked = true;
 			eprintf ("\nTimeout!\n");
 			I.timeout = 0;
 		}
 	}
-	return I.breaked;
+	return I.context->breaked;
 }
 
 R_API void r_cons_break_timeout(int timeout) {
@@ -247,22 +295,35 @@ R_API void r_cons_break_timeout(int timeout) {
 }
 
 R_API void r_cons_break_end() {
-	I.breaked = false;
+	I.context->breaked = false;
 	I.timeout = 0;
-	r_print_set_interrupted (I.breaked);
 #if __UNIX__ || __CYGWIN__
 	signal (SIGINT, SIG_IGN);
 #endif
-	if (!r_stack_is_empty (I.break_stack)) {
+	if (!r_stack_is_empty (I.context->break_stack)) {
 		//free all the stack
-		r_stack_free (I.break_stack);
+		r_stack_free (I.context->break_stack);
 		//create another one
-		I.break_stack = r_stack_newf (6, break_stack_free);
-		I.data = NULL;
-		I.event_interrupt = NULL;
+		I.context->break_stack = r_stack_newf (6, break_stack_free);
+		I.context->event_interrupt_data = NULL;
+		I.context->event_interrupt = NULL;
 	}
 }
 
+R_API void *r_cons_sleep_begin(void) {
+	if (!I.cb_sleep_begin) {
+		return NULL;
+	}
+	return I.cb_sleep_begin (I.user);
+}
+
+R_API void r_cons_sleep_end(void *user) {
+	if (!I.cb_sleep_end) {
+		return;
+	}
+	I.cb_sleep_end (I.user, user);
+}
+
 #if __WINDOWS__ && !__CYGWIN__
 static HANDLE h;
 static BOOL __w32_control(DWORD type) {
@@ -274,7 +335,7 @@ static BOOL __w32_control(DWORD type) {
 	return false;
 }
 #elif __UNIX__ || __CYGWIN__
-static void resize (int sig) {
+static void resize(int sig) {
 	if (I.event_resize) {
 		I.event_resize (I.event_data);
 	}
@@ -295,14 +356,6 @@ R_API bool r_cons_enable_mouse(const bool enable) {
 #endif
 }
 
-static void r_cons_pal_null() {
-	int i;
-	RCons *cons = r_cons_singleton ();
-	for (i = 0; i < R_CONS_PALETTE_LIST_SIZE; i++){
-		cons->pal.list[i] = NULL;
-	}
-}
-
 R_API RCons *r_cons_new() {
 	I.refcnt++;
 	if (I.refcnt != 1) {
@@ -311,10 +364,9 @@ R_API RCons *r_cons_new() {
 	I.rgbstr = r_cons_rgb_str_off;
 	I.line = r_line_new ();
 	I.highlight = NULL;
-	I.event_interrupt = NULL;
 	I.is_wine = -1;
 	I.fps = 0;
-	I.use_color = false;
+	I.color = COLOR_MODE_DISABLED;
 	I.blankline = true;
 	I.teefile = NULL;
 	I.fix_columns = 0;
@@ -323,19 +375,18 @@ R_API RCons *r_cons_new() {
 	I.force_rows = 0;
 	I.force_columns = 0;
 	I.event_resize = NULL;
-	I.data = NULL;
 	I.event_data = NULL;
 	I.is_interactive = true;
 	I.noflush = false;
 	I.linesleep = 0;
 	I.fdin = stdin;
 	I.fdout = 1;
-	I.breaked = false;
 	I.break_lines = false;
-	//I.lines = 0;
-	I.buffer = NULL;
-	I.buffer_sz = 0;
-	I.buffer_len = 0;
+	I.lines = 0;
+
+	I.context = &r_cons_context_default;
+	cons_context_init (I.context);
+
 	r_cons_get_size (&I.pagesize);
 	I.num = NULL;
 	I.null = 0;
@@ -362,14 +413,13 @@ R_API RCons *r_cons_new() {
 	}
 #endif
 	I.pager = NULL; /* no pager by default */
-	I.truecolor = 0;
 	I.mouse = 0;
-	I.cons_stack = r_stack_newf (6, cons_stack_free);
-	I.break_stack = r_stack_newf (6, break_stack_free);
-	r_cons_pal_null ();
-	r_cons_pal_init (NULL);
-	r_cons_rgb_init ();
 	r_cons_reset ();
+	r_cons_rgb_init ();
+	r_cons_pal_init ();
+
+	r_print_set_is_interrupted_cb (r_cons_is_breaked);
+
 	return &I;
 }
 
@@ -383,13 +433,12 @@ R_API RCons *r_cons_free() {
 		r_line_free ();
 		I.line = NULL;
 	}
-	if (I.buffer) {
-		free (I.buffer);
-		I.buffer = NULL;
+	if (I.context->buffer) {
+		free (I.context->buffer);
+		I.context->buffer = NULL;
 	}
 	R_FREE (I.break_word);
-	r_stack_free (I.cons_stack);
-	r_stack_free (I.break_stack);
+	cons_context_deinit (I.context);
 	return NULL;
 }
 
@@ -399,7 +448,7 @@ static bool palloc(int moar) {
 	if (moar <= 0) {
 		return false;
 	}
-	if (!I.buffer) {
+	if (!I.context->buffer) {
 		int new_sz;
 		if ((INT_MAX - MOAR) < moar) {
 			return false;
@@ -407,22 +456,22 @@ static bool palloc(int moar) {
 		new_sz = moar + MOAR;
 		temp = calloc (1, new_sz);
 		if (temp) {
-			I.buffer_sz = new_sz;
-			I.buffer = temp;
-			I.buffer[0] = '\0';
+			I.context->buffer_sz = new_sz;
+			I.context->buffer = temp;
+			I.context->buffer[0] = '\0';
 		}
-	} else if (moar + I.buffer_len > I.buffer_sz) {
+	} else if (moar + I.context->buffer_len > I.context->buffer_sz) {
 		char *new_buffer;
-		int old_buffer_sz = I.buffer_sz;
-		if ((INT_MAX - MOAR - moar) < I.buffer_sz) {
+		int old_buffer_sz = I.context->buffer_sz;
+		if ((INT_MAX - MOAR - moar) < I.context->buffer_sz) {
 			return false;
 		}
-		I.buffer_sz += moar + MOAR;
-		new_buffer = realloc (I.buffer, I.buffer_sz);
+		I.context->buffer_sz += moar + MOAR;
+		new_buffer = realloc (I.context->buffer, I.context->buffer_sz);
 		if (new_buffer) {
-			I.buffer = new_buffer;
+			I.context->buffer = new_buffer;
 		} else {
-			I.buffer_sz = old_buffer_sz;
+			I.context->buffer_sz = old_buffer_sz;
 			return false;
 		}
 	}
@@ -462,11 +511,11 @@ R_API void r_cons_fill_line() {
 R_API void r_cons_clear_line(int std_err) {
 #if __WINDOWS__
 	if (I.ansicon) {
-		fprintf (std_err? stderr: stdout,"\x1b[0K\r");
+		fprintf (std_err? stderr: stdout,"%s", R_CONS_CLEAR_LINE);
 	} else {
 		char white[1024];
 		memset (&white, ' ', sizeof (white));
-		if (I.columns > 0 && I.columns < sizeof(white)) {
+		if (I.columns > 0 && I.columns < sizeof (white)) {
 			white[I.columns - 1] = 0;
 		} else if (I.columns == 0) {
 			white[0] = 0;
@@ -476,7 +525,7 @@ R_API void r_cons_clear_line(int std_err) {
 		fprintf (std_err? stderr: stdout, "\r%s\r", white);
 	}
 #else
-	fprintf (std_err? stderr: stdout,"\x1b[0K\r");
+	fprintf (std_err? stderr: stdout,"%s", R_CONS_CLEAR_LINE);
 #endif
 	fflush (std_err? stderr: stdout);
 }
@@ -495,101 +544,135 @@ R_API void r_cons_clear() {
 	I.lines = 0;
 }
 
+static void cons_grep_reset(RConsGrep *grep) {
+	grep->strings[0][0] = '\0';
+	grep->nstrings = 0; // XXX
+	grep->line = -1;
+	grep->sort = -1;
+	grep->sort_invert = false;
+	R_FREE (grep->str);
+	ZERO_FILL (grep->tokens);
+	grep->tokens_used = 0;
+}
+
 R_API void r_cons_reset() {
-	if (I.buffer) {
-		I.buffer[0] = '\0';
+	if (I.context->buffer) {
+		I.context->buffer[0] = '\0';
 	}
-	I.buffer_len = 0;
+	I.context->buffer_len = 0;
 	I.lines = 0;
-	I.lastline = I.buffer;
-	I.grep.strings[0][0] = '\0';
-	I.grep.nstrings = 0; // XXX
-	I.grep.line = -1;
-	I.grep.sort = -1;
-	I.grep.sort_invert = false;
-	R_FREE (I.grep.str);
-	ZERO_FILL (I.grep.tokens);
-	I.grep.tokens_used = 0;
+	I.lastline = I.context->buffer;
+	cons_grep_reset (&I.context->grep);
 }
 
 R_API const char *r_cons_get_buffer() {
 	//check len otherwise it will return trash
-	return I.buffer_len? I.buffer : NULL;
+	return I.context->buffer_len? I.context->buffer : NULL;
 }
 
 R_API void r_cons_filter() {
 	/* grep */
-	if (I.filter || I.grep.nstrings > 0 || I.grep.tokens_used || I.grep.less || I.grep.json) {
-		r_cons_grepbuf (I.buffer, I.buffer_len);
+	if (I.filter || I.context->grep.nstrings > 0 || I.context->grep.tokens_used || I.context->grep.less || I.context->grep.json) {
+		r_cons_grepbuf (I.context->buffer, I.context->buffer_len);
 		I.filter = false;
 	}
 	/* html */
 	if (I.is_html) {
 		int newlen = 0;
-		char *input = r_str_ndup (I.buffer, I.buffer_len);
+		char *input = r_str_ndup (I.context->buffer, I.context->buffer_len);
 		char *res = r_cons_html_filter (input, &newlen);
-		free (I.buffer);
+		free (I.context->buffer);
 		free (input);
-		I.buffer = res;
-		I.buffer_len = newlen;
-		I.buffer_sz = newlen;
+		I.context->buffer = res;
+		I.context->buffer_len = newlen;
+		I.context->buffer_sz = newlen;
 	}
 	/* TODO */
 }
 
 R_API void r_cons_push() {
-	if (I.cons_stack) {
-		RConsStack *data = R_NEW0 (RConsStack);
-		if (!data) {
-			return;
-		}
-		if (I.buffer) {
-			data->buf = malloc (I.buffer_sz);
-			if (!data->buf) {
-				free (data);
-				return;
-			}
-			memcpy (data->buf, I.buffer, I.buffer_sz);
-			data->buf_len = I.buffer_len;
-			data->buf_size = I.buffer_sz;
-		}
-		data->grep = R_NEW0 (RConsGrep);
-		if (data->grep) {
-			memcpy (data->grep, &I.grep, sizeof (RConsGrep));
-			if (I.grep.str) {
-				data->grep->str = strdup (I.grep.str);
-			}
-		}
-		r_stack_push (I.cons_stack, data);
-		I.buffer_len = 0;
-		if (I.buffer) {
-			memset (I.buffer, 0, I.buffer_sz);
-		}
+	if (!I.context->cons_stack) {
+		return;
+	}
+	RConsStack *data = cons_stack_dump (true);
+	if (!data) {
+		return;
+	}
+	r_stack_push (I.context->cons_stack, data);
+	I.context->buffer_len = 0;
+	if (I.context->buffer) {
+		memset (I.context->buffer, 0, I.context->buffer_sz);
 	}
 }
 
 R_API void r_cons_pop() {
-	if (I.cons_stack) {
-		RConsStack *data = (RConsStack *)r_stack_pop (I.cons_stack);
-		if (!data) {
-			return;
-		}
-		free (I.buffer);
-		I.buffer = data->buf ? malloc (data->buf_size) : NULL;
-		I.buffer_len = data->buf_len;
-		I.buffer_sz = data->buf_size;
-		if (I.buffer) {
-			memcpy (I.buffer, data->buf, data->buf_size);
-		}
-		if (data->grep) {
-			memcpy (&I.grep, data->grep, sizeof (RConsGrep));
-			if (data->grep->str) {
-				char *old = I.grep.str;
-				I.grep.str = strdup (data->grep->str);
-				R_FREE (old);
-			}
-		}
-		cons_stack_free ((void *)data);
+	if (!I.context->cons_stack) {
+		return;
+	}
+	RConsStack *data = (RConsStack *)r_stack_pop (I.context->cons_stack);
+	if (!data) {
+		return;
+	}
+	cons_stack_load (data, true);
+	cons_stack_free ((void *)data);
+}
+
+R_API RConsContext *r_cons_context_new(void) {
+	RConsContext *context = R_NEW0 (RConsContext);
+	if (!context) {
+		return NULL;
+	}
+	cons_context_init (context);
+	return context;
+	/*
+	RStack *stack = r_stack_newf (6, cons_stack_free);
+	if (!stack) {
+		return NULL;
+	}
+
+	RConsStack *data = R_NEW0 (RConsStack);
+	if (!data) {
+		r_stack_free (stack);
+		return NULL;
+	}
+
+	data->grep = R_NEW0 (RConsGrep);
+	if (!data->grep) {
+		R_FREE (data);
+		r_stack_free (stack);
+		return NULL;
+	}
+	cons_grep_reset (data->grep);
+
+	r_stack_push (stack, data);
+
+	return stack;*/
+}
+
+R_API void r_cons_context_free(RConsContext *context) {
+	if (!context) {
+		return;
+	}
+	cons_context_deinit (context);
+	free (context);
+}
+
+R_API void r_cons_context_load(RConsContext *context) {
+	I.context = context;
+}
+
+R_API void r_cons_context_reset() {
+	I.context = &r_cons_context_default;
+}
+
+R_API bool r_cons_context_is_main() {
+	return I.context == &r_cons_context_default;
+}
+
+R_API void r_cons_context_break(RConsContext *context) {
+	context->breaked = true;
+	if (context->event_interrupt) {
+		context->event_interrupt (context->event_interrupt_data);
 	}
 }
 
@@ -605,17 +688,17 @@ R_API void r_cons_flush() {
 	r_cons_filter ();
 	if (I.is_interactive && I.fdout == 1) {
 		/* Use a pager if the output doesn't fit on the terminal window. */
-		if (I.pager && *I.pager && I.buffer_len > 0
-				&& r_str_char_count (I.buffer, '\n') >= I.rows) {
-			I.buffer[I.buffer_len-1] = 0;
-			r_sys_cmd_str_full (I.pager, I.buffer, NULL, NULL, NULL);
+		if (I.pager && *I.pager && I.context->buffer_len > 0
+				&& r_str_char_count (I.context->buffer, '\n') >= I.rows) {
+			I.context->buffer[I.context->buffer_len-1] = 0;
+			r_sys_cmd_str_full (I.pager, I.context->buffer, NULL, NULL, NULL);
 			r_cons_reset ();
 
-		} else if (I.buffer_len > CONS_MAX_USER) {
+		} else if (I.context->buffer_len > CONS_MAX_USER) {
 #if COUNT_LINES
 			int i, lines = 0;
-			for (i = 0; I.buffer[i]; i++) {
-				if (I.buffer[i] == '\n') {
+			for (i = 0; I.context->buffer[i]; i++) {
+				if (I.context->buffer[i] == '\n') {
 					lines ++;
 				}
 			}
@@ -625,7 +708,7 @@ R_API void r_cons_flush() {
 			}
 #else
 			char buf[64];
-			char *buflen = r_num_units (buf, I.buffer_len);
+			char *buflen = r_num_units (buf, I.context->buffer_len);
 			if (buflen && !r_cons_yesno ('n',"Do you want to print %s chars? (y/N)", buflen)) {
 				r_cons_reset ();
 				return;
@@ -638,7 +721,7 @@ R_API void r_cons_flush() {
 	if (tee && *tee) {
 		FILE *d = r_sandbox_fopen (tee, "a+");
 		if (d) {
-			if (I.buffer_len != fwrite (I.buffer, 1, I.buffer_len, d)) {
+			if (I.context->buffer_len != fwrite (I.context->buffer, 1, I.context->buffer_len, d)) {
 				eprintf ("r_cons_flush: fwrite: error (%s)\n", tee);
 			}
 			fclose (d);
@@ -652,10 +735,10 @@ R_API void r_cons_flush() {
 		if (I.linesleep > 0 && I.linesleep < 1000) {
 			int i = 0;
 			int pagesize = R_MAX (1, I.pagesize);
-			char *ptr = I.buffer;
+			char *ptr = I.context->buffer;
 			char *nl = strchr (ptr, '\n');
-			int len = I.buffer_len;
-			I.buffer[I.buffer_len] = 0;
+			int len = I.context->buffer_len;
+			I.context->buffer[I.context->buffer_len] = 0;
 			r_cons_break_push (NULL, NULL);
 			while (nl && !r_cons_is_breaked ()) {
 				r_cons_write (ptr, nl - ptr + 1);
@@ -666,13 +749,13 @@ R_API void r_cons_flush() {
 				nl = strchr (ptr, '\n');
 				i++;
 			}
-			r_cons_write (ptr, I.buffer + len - ptr);
+			r_cons_write (ptr, I.context->buffer + len - ptr);
 			r_cons_break_pop ();
 		} else {
-			r_cons_write (I.buffer, I.buffer_len);
+			r_cons_write (I.context->buffer, I.context->buffer_len);
 		}
 	} else {
-		r_cons_write (I.buffer, I.buffer_len);
+		r_cons_write (I.context->buffer, I.context->buffer_len);
 	}
 
 	r_cons_reset ();
@@ -691,12 +774,12 @@ R_API void r_cons_visual_flush() {
 /* TODO: this ifdef must go in the function body */
 #if __WINDOWS__ && !__CYGWIN__
 		if (I.ansicon) {
-			r_cons_visual_write (I.buffer);
+			r_cons_visual_write (I.context->buffer);
 		} else {
-			r_cons_w32_print ((const ut8*)I.buffer, I.buffer_len, 1);
+			r_cons_w32_print ((const ut8*)I.context->buffer, I.context->buffer_len, 1);
 		}
 #else
-		r_cons_visual_write (I.buffer);
+		r_cons_visual_write (I.context->buffer);
 #endif
 	}
 	r_cons_reset ();
@@ -730,7 +813,7 @@ static int real_strlen(const char *ptr, int len) {
 	return ansilen - diff;
 }
 
-R_API void r_cons_visual_write (char *buffer) {
+R_API void r_cons_visual_write(char *buffer) {
 	char white[1024];
 	int cols = I.columns;
 	int alen, plen, lines = I.rows;
@@ -817,16 +900,16 @@ R_API void r_cons_printf_list(const char *format, va_list ap) {
 	if (strchr (format, '%')) {
 		palloc (MOAR + strlen (format) * 20);
 club:
-		size = I.buffer_sz - I.buffer_len - 1; /* remaining space in I.buffer */
-		written = vsnprintf (I.buffer + I.buffer_len, size, format, ap3);
+		size = I.context->buffer_sz - I.context->buffer_len - 1; /* remaining space in I.context->buffer */
+		written = vsnprintf (I.context->buffer + I.context->buffer_len, size, format, ap3);
 		if (written >= size) { /* not all bytes were written */
 			palloc (written);
 			va_end (ap3);
 			va_copy (ap3, ap2);
 			goto club;
 		}
-		I.buffer_len += written;
-		I.buffer[I.buffer_len] = 0;
+		I.context->buffer_len += written;
+		I.context->buffer[I.context->buffer_len] = 0;
 	} else {
 		r_cons_strcat (format);
 	}
@@ -845,17 +928,17 @@ R_API void r_cons_printf(const char *format, ...) {
 }
 
 R_API int r_cons_get_column() {
-	char *line = strrchr (I.buffer, '\n');
+	char *line = strrchr (I.context->buffer, '\n');
 	if (!line) {
-		line = I.buffer;
+		line = I.context->buffer;
 	}
-	I.buffer[I.buffer_len] = 0;
+	I.context->buffer[I.context->buffer_len] = 0;
 	return r_str_ansi_len (line);
 }
 
 /* final entrypoint for adding stuff in the buffer screen */
 R_API int r_cons_memcat(const char *str, int len) {
-	if (len < 0 || (I.buffer_len + len) < 0) {
+	if (len < 0 || (I.context->buffer_len + len) < 0) {
 		return -1;
 	}
 	if (I.echo) {
@@ -863,9 +946,9 @@ R_API int r_cons_memcat(const char *str, int len) {
 	}
 	if (str && len > 0 && !I.null) {
 		if (palloc (len + 1)) {
-			memcpy (I.buffer + I.buffer_len, str, len);
-			I.buffer_len += len;
-			I.buffer[I.buffer_len] = 0;
+			memcpy (I.context->buffer + I.context->buffer_len, str, len);
+			I.context->buffer_len += len;
+			I.context->buffer[I.context->buffer_len] = 0;
 		}
 	}
 	if (I.flush) {
@@ -873,7 +956,7 @@ R_API int r_cons_memcat(const char *str, int len) {
 	}
 	if (I.break_word && str && len > 0) {
 		if (r_mem_mem ((const ut8*)str, len, (const ut8*)I.break_word, I.break_word_len)) {
-			I.breaked = true;
+			I.context->breaked = true;
 		}
 	}
 	return len;
@@ -882,9 +965,9 @@ R_API int r_cons_memcat(const char *str, int len) {
 R_API void r_cons_memset(char ch, int len) {
 	if (!I.null && len > 0) {
 		palloc (len + 1);
-		memset (I.buffer + I.buffer_len, ch, len);
-		I.buffer_len += len;
-		I.buffer[I.buffer_len] = 0;
+		memset (I.context->buffer + I.context->buffer_len, ch, len);
+		I.context->buffer_len += len;
+		I.context->buffer[I.context->buffer_len] = 0;
 	}
 }
 
@@ -914,15 +997,16 @@ R_API void r_cons_newline() {
 }
 
 /* return the aproximated x,y of cursor before flushing */
+// XXX this function is a huge bottleneck
 R_API int r_cons_get_cursor(int *rows) {
 	int i, col = 0;
 	int row = 0;
 	// TODO: we need to handle GOTOXY and CLRSCR ansi escape code too
-	for (i = 0; i < I.buffer_len; i++) {
+	for (i = 0; i < I.context->buffer_len; i++) {
 		// ignore ansi chars, copypasta from r_str_ansi_len
-		if (I.buffer[i] == 0x1b) {
-			char ch2 = I.buffer[i + 1];
-			char *str = I.buffer;
+		if (I.context->buffer[i] == 0x1b) {
+			char ch2 = I.context->buffer[i + 1];
+			char *str = I.context->buffer;
 			if (ch2 == '\\') {
 				i++;
 			} else if (ch2 == ']') {
@@ -931,7 +1015,7 @@ R_API int r_cons_get_cursor(int *rows) {
 			} else if (ch2 == '[') {
 				for (++i; str[i] && str[i] != 'J' && str[i] != 'm' && str[i] != 'H'; i++);
 			}
-		} else if (I.buffer[i] == '\n') {
+		} else if (I.context->buffer[i] == '\n') {
 			row++;
 			col = 0;
 		} else {
@@ -1053,7 +1137,7 @@ R_API int r_cons_get_size(int *rows) {
 	return R_MAX (0, I.columns);
 }
 
-R_API void r_cons_show_cursor (int cursor) {
+R_API void r_cons_show_cursor(int cursor) {
 #if __WINDOWS__ && !__CYGWIN__
 	// TODO
 #else
@@ -1088,8 +1172,8 @@ R_API void r_cons_set_raw(bool is_raw) {
 	/* do nothing here */
 #elif __UNIX__ || __CYGWIN__
 	// enforce echo off
-	I.term_raw.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
 	if (is_raw) {
+		I.term_raw.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
 		tcsetattr (0, TCSANOW, &I.term_raw);
 	} else {
 		tcsetattr (0, TCSANOW, &I.term_buf);
@@ -1151,12 +1235,12 @@ R_API void r_cons_set_cup(int enable) {
 }
 
 R_API void r_cons_column(int c) {
-	char *b = malloc (I.buffer_len + 1);
+	char *b = malloc (I.context->buffer_len + 1);
 	if (!b) {
 		return;
 	}
-	memcpy (b, I.buffer, I.buffer_len);
-	b[I.buffer_len] = 0;
+	memcpy (b, I.context->buffer, I.context->buffer_len);
+	b[I.context->buffer_len] = 0;
 	r_cons_reset ();
 	// align current buffer N chars right
 	r_cons_strcat_justify (b, c, 0);
@@ -1186,9 +1270,9 @@ R_API void r_cons_zero() {
 	write (1, "", 1);
 }
 
-R_API void r_cons_highlight (const char *word) {
-	int l, *cpos;
-	char *rword, *res, *clean;
+R_API void r_cons_highlight(const char *word) {
+	int l, *cpos = NULL;
+	char *rword = NULL, *res, *clean = NULL;
 	char *inv[2] = {
 		R_CONS_INVERT (true, true),
 		R_CONS_INVERT (false, true)
@@ -1198,12 +1282,12 @@ R_API void r_cons_highlight (const char *word) {
 		strlen (inv[1])
 	};
 
-	if (word && *word && I.buffer) {
+	if (word && *word && I.context->buffer) {
 		int word_len = strlen (word);
 		char *orig;
-		clean = I.buffer;
-		l = r_str_ansi_filter (clean, &orig, &cpos, 0);
-		I.buffer = orig;
+		clean = r_str_ndup (I.context->buffer, I.context->buffer_len);
+		l = r_str_ansi_filter (clean, &orig, &cpos, -1);
+		I.context->buffer = orig;
 		if (I.highlight) {
 			if (strcmp (word, I.highlight)) {
 				free (I.highlight);
@@ -1215,31 +1299,32 @@ R_API void r_cons_highlight (const char *word) {
 		rword = malloc (word_len + linv[0] + linv[1] + 1);
 		if (!rword) {
 			free (cpos);
+			free (clean);
 			return;
 		}
 		strcpy (rword, inv[0]);
 		strcpy (rword + linv[0], word);
 		strcpy (rword + linv[0] + word_len, inv[1]);
-		res = r_str_replace_thunked (I.buffer, clean, cpos,
+		res = r_str_replace_thunked (I.context->buffer, clean, cpos,
 					     l, word, rword, 1);
 		if (res) {
-			I.buffer = res;
-			I.buffer_len = I.buffer_sz = strlen (res);
+			I.context->buffer = res;
+			I.context->buffer_len = I.context->buffer_sz = strlen (res);
 		}
 		free (rword);
 		free (clean);
 		free (cpos);
 		/* don't free orig - it's assigned
-		 * to I.buffer and possibly realloc'd */
+		 * to I.context->buffer and possibly realloc'd */
 	} else {
 		free (I.highlight);
 		I.highlight = NULL;
 	}
 }
 
-R_API char *r_cons_lastline (int *len) {
-	char *b = I.buffer + I.buffer_len;
-	while (b > I.buffer) {
+R_API char *r_cons_lastline(int *len) {
+	char *b = I.context->buffer + I.context->buffer_len;
+	while (b > I.context->buffer) {
 		if (*b == '\n') {
 			b++;
 			break;
@@ -1247,12 +1332,53 @@ R_API char *r_cons_lastline (int *len) {
 		b--;
 	}
 	if (len) {
-		int delta = b - I.buffer;
-		*len = I.buffer_len - delta;
+		int delta = b - I.context->buffer;
+		*len = I.context->buffer_len - delta;
 	}
 	return b;
 }
 
+// same as r_cons_lastline(), but len will be the number of
+// utf-8 characters excluding ansi escape sequences as opposed to just bytes
+R_API char *r_cons_lastline_utf8_ansi_len(int *len) {
+	if (!len) {
+		return r_cons_lastline (0);
+	}
+
+	char *b = I.context->buffer + I.context->buffer_len;
+	int l = 0;
+	int last_possible_ansi_end = 0;
+	char ch = '\0';
+	char ch2;
+	while (b > I.context->buffer) {
+		ch2 = ch;
+		ch = *b;
+
+		if (ch == '\n') {
+			b++;
+			l--;
+			break;
+		}
+
+		// utf-8
+		if ((ch & 0xc0) != 0x80) {
+			l++;
+		}
+
+		// ansi
+		if (ch == 'J' || ch == 'm' || ch == 'H') {
+			last_possible_ansi_end = l - 1;
+		} else if (ch == '\x1b' && ch2 == '[') {
+			l = last_possible_ansi_end;
+		}
+
+		b--;
+	}
+
+	*len = l;
+	return b;
+}
+
 /* swap color from foreground to background, returned value must be freed */
 R_API char *r_cons_swap_ground(const char *col) {
 	if (!col) {
@@ -1274,22 +1400,22 @@ R_API char *r_cons_swap_ground(const char *col) {
 	return strdup (col);
 }
 
-R_API bool r_cons_drop (int n) {
-	if (n > I.buffer_len) {
-		I.buffer_len = 0;
+R_API bool r_cons_drop(int n) {
+	if (n > I.context->buffer_len) {
+		I.context->buffer_len = 0;
 		return false;
 	}
-	I.buffer_len -= n;
+	I.context->buffer_len -= n;
 	return true;
 }
 
-R_API void r_cons_chop () {
-	while (I.buffer_len > 0) {
-		char ch = I.buffer[I.buffer_len - 1];
+R_API void r_cons_chop() {
+	while (I.context->buffer_len > 0) {
+		char ch = I.context->buffer[I.context->buffer_len - 1];
 		if (ch != '\n' && !IS_WHITESPACE (ch)) {
 			break;
 		}
-		I.buffer_len--;
+		I.context->buffer_len--;
 	}
 }
 
@@ -1353,7 +1479,7 @@ R_API void r_cons_cmd_help(const char *help[], bool use_color) {
 	for (i = 0; help[i]; i += 3) {
 		if (i) {
 			int padding = max_length - (strlen (help[i]) + strlen (help[i + 1]));
-			r_cons_printf("| %s%s%s%*s  %s%s%s\n",
+			r_cons_printf ("| %s%s%s%*s  %s%s%s\n",
 					help[i],
 					pal_args_color, help[i + 1],
 					padding, "",
