@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2017 - pancake */
+/* radare - LGPL - Copyright 2007-2018 - pancake */
 /* dietline is a lightweight and portable library similar to GNU readline */
 
 #include <r_cons.h>
@@ -17,55 +17,30 @@
 #endif
 
 static char *r_line_nullstr = "";
-static const char dl_basic_word_break_characters[] =  " \t\n\"\\'`@$><=;|&{(";
-
-#define ONLY_VALID_CHARS 1
-
-#if ONLY_VALID_CHARS
-static inline int is_valid_char(unsigned char ch) {
-	if (ch >= 32 && ch <= 127) {
-		return true;
-	}
-	switch (ch) {
-	// case 0: // wat
-	case 1:	// ^a
-	case 2:	// ^b -> emacs left
-	case 4:	// ^d
-	case 5:	// ^e
-	case 6:	// ^f -> emacs right
-	case 8:	// backspace
-	case 9:	// tab
-	case 10:// newline
-	case 13:// carriage return
-	case 23:// ^w
-	case 27:// arrow
-		return true;
-	}
-	return false;
-}
-#endif
+static const char word_break_characters[] = "\t\n ~`!@#$%^&*()-_=+[]{}\\|;:\"'<>,./";
 
 static inline bool is_word_break_char(char ch) {
 	int i;
 	int len =
-		sizeof (dl_basic_word_break_characters) /
-		sizeof (dl_basic_word_break_characters[0]);
+		sizeof (word_break_characters) /
+		sizeof (word_break_characters[0]);
 	for (i = 0; i < len; ++i) {
-		if (ch == dl_basic_word_break_characters[i]) {
+		if (ch == word_break_characters[i]) {
 			return true;
 		}
 	}
 	return false;
 }
 
-static void unix_word_rubout() {
-	int i;
+/* https://www.gnu.org/software/bash/manual/html_node/Commands-For-Killing.html */
+static void backward_kill_word() {
+	int i, len;
 	if (I.buffer.index > 0) {
 		for (i = I.buffer.index - 1; i > 0 && is_word_break_char (I.buffer.data[i]); i--) {
-			/*nothing to see here*/
+			/* Move the cursor index back until we hit a non-word-break-character */
 		}
-		for (; i && !is_word_break_char (I.buffer.data[i]); i--) {
-			/*nothing to see here*/
+		for (; i > 0 && !is_word_break_char (I.buffer.data[i]); i--) {
+			/* Move the cursor index back until we hit a word-break-character */
 		}
 		if (i > 0) {
 			i++;
@@ -75,6 +50,9 @@ static void unix_word_rubout() {
 		if (I.buffer.index > I.buffer.length) {
 			I.buffer.length = I.buffer.index;
 		}
+		len = I.buffer.index - i + 1;
+		free (I.clipboard);
+		I.clipboard = r_str_ndup (I.buffer.data + i, len);
 		memmove (I.buffer.data + i, I.buffer.data + I.buffer.index,
 				I.buffer.length - I.buffer.index + 1);
 		I.buffer.length = strlen (I.buffer.data);
@@ -82,6 +60,63 @@ static void unix_word_rubout() {
 	}
 }
 
+static void kill_word() {
+	int i, len;
+	for (i = I.buffer.index + 1; i < I.buffer.length && is_word_break_char (I.buffer.data[i]); i++) {
+		/* Move the cursor index forward until we hit a non-word-break-character */
+	}
+	for (; i < I.buffer.length && !is_word_break_char (I.buffer.data[i]); i++) {
+		/* Move the cursor index forward we hit a word-break-character */
+	}
+	if (I.buffer.index >= I.buffer.length) {
+		I.buffer.length = I.buffer.index;
+	}
+	len = i - I.buffer.index + 1;
+	free (I.clipboard);
+	I.clipboard = r_str_ndup (I.buffer.data + I.buffer.index, len);
+	memmove (I.buffer.data + I.buffer.index, I.buffer.data + i, len);
+	I.buffer.length = strlen (I.buffer.data);
+}
+
+static void paste() {
+	if (I.clipboard) {
+		char *cursor = I.buffer.data + I.buffer.index;
+		int dist = (I.buffer.data + I.buffer.length) - cursor;
+		int len = strlen (I.clipboard);
+		I.buffer.length += len;
+		memmove (cursor + len, cursor, dist);
+		memcpy (cursor, I.clipboard, len);
+		I.buffer.index += len;
+	}
+}
+
+static void unix_word_rubout() {
+	int i;
+	if (I.buffer.index > 0) {
+		for (i = I.buffer.index - 1; i > 0 && I.buffer.data[i] == ' '; i--) {
+			/* Move cursor backwards until we hit a non-space character or EOL */
+			/* This removes any trailing spaces from the input */
+		}
+		for (; i > 0 && I.buffer.data[i] != ' '; i--) {
+			/* Move cursor backwards until we hit a space character or EOL */
+			/* This deletes everything back to the previous space character */
+		}
+		if (i > 0) {
+			i++;
+		} else if (i < 0) {
+			i = 0;
+		}
+		if (I.buffer.index > I.buffer.length) {
+			I.buffer.length = I.buffer.index;
+		}
+		memmove (I.buffer.data + i,
+			I.buffer.data + I.buffer.index,
+			I.buffer.length - I.buffer.index + 1);
+		I.buffer.length = strlen (I.buffer.data);
+		I.buffer.index = i;
+	}
+}
+
 static int inithist() {
 	ZERO_FILL (I.history);
 	if ((I.history.size + 1024) * sizeof (char *) < I.history.size) {
@@ -109,15 +144,22 @@ R_API int r_line_dietline_init() {
 /* read utf8 char into 's', return the length in bytes */
 static int r_line_readchar_utf8(ut8 *s, int slen) {
 	// TODO: add support for w32
-	ssize_t len, t, i;
+	ssize_t len, i;
 	if (slen < 1) {
 		return 0;
 	}
+	int ch = r_cons_readchar ();
+	if (ch == -1) {
+		return -1;
+	}
+	*s = ch;
+#if 0
 	if ((t = read (0, s, 1)) != 1) {
 		return t;
 	}
-	s[0] = r_cons_controlz (s[0]);
-	if (s[0] < 0x80) {
+#endif
+	*s = r_cons_controlz (*s);
+	if (*s < 0x80) {
 		len = 1;
 	} else if ((s[0] & 0xe0) == 0xc0) {
 		len = 2;
@@ -132,8 +174,9 @@ static int r_line_readchar_utf8(ut8 *s, int slen) {
 		return -1;
 	}
 	for (i = 1; i < len; i++) {
-		if ((t = read (0, s + i, 1)) != 1) {
-			return t;
+		int ch = r_cons_readchar ();
+		if (ch != -1) {
+			s[i] = ch;
 		}
 		if ((s[i] & 0xc0) != 0x80) {
 			return -1;
@@ -145,122 +188,112 @@ static int r_line_readchar_utf8(ut8 *s, int slen) {
 
 #if __WINDOWS__ && !__CYGWIN__
 static int r_line_readchar_win(int *vch) { // this function handle the input in console mode
-	INPUT_RECORD irInBuf[128];
+	INPUT_RECORD irInBuf;
 	BOOL ret, bCtrl = FALSE;
 	DWORD mode, out;
 	ut8 buf[2];
 	HANDLE h;
 	int i;
+	void *bed;
 
 	if (I.zerosep) {
 		*vch = 0;
 		buf[0] = 0;
-		read (0, buf, 1);
+		bed = r_cons_sleep_begin ();
+		int rsz = read (0, buf, 1);
+		r_cons_sleep_end (bed);
+		if (rsz != 1)
+			return -1;
 		return buf[0];
 	}
 
 	*buf = '\0';
-do_it_again:
+
 	h = GetStdHandle (STD_INPUT_HANDLE);
 	GetConsoleMode (h, &mode);
 	SetConsoleMode (h, 0);	// RAW
+do_it_again:
 	*vch = 0;
-	ret = ReadConsoleInput (h, irInBuf, 128, &out);
+	bed = r_cons_sleep_begin ();
+	ret = ReadConsoleInput (h, &irInBuf, 1, &out);
+	r_cons_sleep_end (bed);
 	if (ret < 1) {
 		return 0;
 	}
-	for (i = 0; i < out; i++) {
-		if (irInBuf[i].EventType != KEY_EVENT) {
-			continue;
-		}
-		if (!irInBuf[i].Event.KeyEvent.bKeyDown) {
-			continue;
-		}
-		*buf = irInBuf[i].Event.KeyEvent.uChar.AsciiChar;
-		bCtrl = irInBuf[i].Event.KeyEvent.dwControlKeyState & 8;
-		if (irInBuf[i].Event.KeyEvent.uChar.AsciiChar) {
-			continue;
-		}
-		switch (irInBuf[i].Event.KeyEvent.wVirtualKeyCode) {
-		case VK_DOWN: *vch = bCtrl? 140: 40; break;
-		case VK_UP: *vch = bCtrl? 138: 38; break;
-		case VK_RIGHT: *vch = bCtrl? 139: 39; break;
-		case VK_LEFT: *vch = bCtrl? 137: 37; break;
-		case 46: *vch = bCtrl? 146: 46; break;	// SUPR KEY
-		case VK_PRIOR: *vch = bCtrl? 136: 36; break;	// HOME KEY
-		case VK_NEXT: *vch = bCtrl? 135: 35; break;	// END KEY
-		default: *vch = *buf = 0; break;
+	if (irInBuf.EventType == KEY_EVENT) {
+		if (irInBuf.Event.KeyEvent.bKeyDown) {
+			if (irInBuf.Event.KeyEvent.uChar.AsciiChar) {
+				*buf = irInBuf.Event.KeyEvent.uChar.AsciiChar;
+				bCtrl = irInBuf.Event.KeyEvent.dwControlKeyState & 8;
+			}
+			else {
+				switch (irInBuf.Event.KeyEvent.wVirtualKeyCode) {
+				case VK_DOWN: *vch = bCtrl ? 140 : 40; break;
+				case VK_UP: *vch = bCtrl ? 138 : 38; break;
+				case VK_RIGHT: *vch = bCtrl ? 139 : 39; break;
+				case VK_LEFT: *vch = bCtrl ? 137 : 37; break;
+				case VK_DELETE: *vch = bCtrl ? 146 : 46; break;	// SUPR KEY
+				case VK_HOME: *vch = bCtrl ? 136 : 36; break;	// HOME KEY
+				case VK_END: *vch = bCtrl ? 135 : 35; break;	// END KEY
+				default: *vch = *buf = 0; break;
+				}
+			}
 		}
 	}
-	SetConsoleMode (h, mode);
 	if (buf[0] == 0 && *vch == 0) {
 		goto do_it_again;
 	}
+	SetConsoleMode (h, mode);
 	return buf[0];
 }
 #endif
-static int r_line_readchar() {
-	ut8 buf[2];
-	*buf = '\0';
-#if __WINDOWS__ && !__CYGWIN__
-#if 1		// new implementation for read input at windows by skuater. If something fail set this to 0
-	int dummy = 0;
-	return r_line_readchar_win (&dummy);
-#endif
-	BOOL ret;
-	DWORD mode, out;
-	HANDLE h;
-#else
-	int ret;
-#endif
 
-do_it_again:
-#if __WINDOWS__ && !__CYGWIN__
-	h = GetStdHandle (STD_INPUT_HANDLE);
-	GetConsoleMode (h, &mode);
-	SetConsoleMode (h, 0);	// RAW
-	ret = ReadConsole (h, buf, 1, &out, NULL);
-	// wine hack-around
-	if (!ret && read (0, buf, 1) != 1) {
-		return -1;
+R_API int r_line_set_hist_callback(RLine *line, RLineHistoryUpCb up, RLineHistoryDownCb down) {
+	line->cb_history_up = up;
+	line->cb_history_down = down;
+	line->offset_hist_index = 0;
+	line->file_hist_index = 0;
+	line->sdbshell_hist_iter = line->sdbshell_hist? r_list_head (line->sdbshell_hist): NULL;
+	return 1;
+}
+
+R_API int cmd_history_up(RLine *line) {
+	if (line->hist_up) {
+		return line->hist_up (line->user);
 	}
-	SetConsoleMode (h, mode);
-#else
-	do {
-		buf[0] = 0;
-		ret = read (0, buf, 1);
-		buf[0] = r_cons_controlz (buf[0]);
-		// VTE HOME/END support
-		if (buf[0] == 79) {
-			if (read (0, buf, 1) != 1) {
-				return -1;
-			}
-			if (buf[0] == 70) {
-				return 5;
-			} else if (buf[0] == 72) {
-				return 1;
-			}
-			return 0;
-		}
-		if (ret == -1) {
-			return 0;	// read no char
-		}
-		if (!buf[0] || !ret) {
-			return -1;	// eof
-		}
-		// TODO: add support for other invalid chars
-		if (*buf == 0xc2 || *buf == 0xc3) {
-			read (0, buf + 1, 1);
-			*buf = '\0';
-		}
-	} while (*buf == '\0');
-#endif
-#if ONLY_VALID_CHARS
-	if (!is_valid_char (buf[0])) {
-		goto do_it_again;
+	if (!line->history.data) {
+		inithist ();
 	}
-#endif
-	return buf[0];
+	if (line->history.index > 0 && line->history.data) {
+		strncpy (line->buffer.data, line->history.data[--line->history.index], R_LINE_BUFSIZE - 1);
+		line->buffer.index = line->buffer.length = strlen (line->buffer.data);
+		return true;
+	}
+	return false;
+}
+
+R_API int cmd_history_down(RLine *line) {
+	if (line->hist_down) {
+		return line->hist_down (line->user);
+	}
+	line->buffer.index = 0;
+	if (!line->history.data) {
+		inithist ();
+	}
+	if (line->history.index == line->history.top) {
+		return false;
+	}
+	line->history.index++;
+	if (line->history.index == line->history.top) {
+		line->buffer.data[0] = '\0';
+		line->buffer.index = line->buffer.length = 0;
+		return false;
+	}
+	if (line->history.data && line->history.data[line->history.index]) {
+		strncpy (line->buffer.data, line->history.data[line->history.index], R_LINE_BUFSIZE - 1);
+		line->buffer.index = line->buffer.length = strlen (line->buffer.data);
+	}
+	return true;
 }
 
 R_API int r_line_hist_add(const char *line) {
@@ -292,42 +325,17 @@ R_API int r_line_hist_add(const char *line) {
 }
 
 static int r_line_hist_up() {
-	if (I.hist_up) {
-		return I.hist_up (I.user);
+	if (!I.cb_history_up) {
+		r_line_set_hist_callback (&I, &cmd_history_up, &cmd_history_down);
 	}
-	if (!I.history.data) {
-		inithist ();
-	}
-	if (I.history.index > 0) {
-		strncpy (I.buffer.data, I.history.data[--I.history.index], R_LINE_BUFSIZE - 1);
-		I.buffer.index = I.buffer.length = strlen (I.buffer.data);
-		return true;
-	}
-	return false;
+	return I.cb_history_up (&I);
 }
 
 static int r_line_hist_down() {
-	if (I.hist_down) {
-		return I.hist_down (I.user);
+	if (!I.cb_history_down) {
+		r_line_set_hist_callback (&I, &cmd_history_up, &cmd_history_down);
 	}
-	I.buffer.index = 0;
-	if (!I.history.data) {
-		inithist ();
-	}
-	if (I.history.index == I.history.top) {
-		return false;
-	}
-	I.history.index++;
-	if (I.history.index == I.history.top) {
-		I.buffer.data[0] = '\0';
-		I.buffer.index = I.buffer.length = 0;
-		return false;
-	}
-	if (I.history.data[I.history.index]) {
-		strncpy (I.buffer.data, I.history.data[I.history.index], R_LINE_BUFSIZE - 1);
-		I.buffer.index = I.buffer.length = strlen (I.buffer.data);
-	}
-	return true;
+	return I.cb_history_down (&I);
 }
 
 R_API const char *r_line_hist_get(int n) {
@@ -335,6 +343,7 @@ R_API const char *r_line_hist_get(int n) {
 	if (!I.history.data) {
 		inithist ();
 	}
+	n--;
 	if (I.history.data) {
 		for (i = 0; i < I.history.size && I.history.data[i]; i++) {
 			if (n == i) {
@@ -368,6 +377,7 @@ R_API void r_line_hist_free() {
 		}
 	}
 	R_FREE (I.history.data);
+	R_FREE (I.sdbshell_hist);
 	I.history.index = 0;
 }
 
@@ -402,7 +412,10 @@ R_API int r_line_hist_save(const char *file) {
 		p = (char *) r_str_lastbut (path, R_SYS_DIR[0], NULL);	// TODO: use fs
 		if (p) {
 			*p = 0;
-			r_sys_mkdirp (path);
+			if (!r_sys_mkdirp (path)) {
+				eprintf ("could not save history into %s\n", path);
+				goto end;
+			}
 			*p = R_SYS_DIR[0];
 		}
 		fd = fopen (path, "w");
@@ -419,6 +432,7 @@ R_API int r_line_hist_save(const char *file) {
 			}
 		}
 	}
+end:
 	free (path);
 	return ret;
 }
@@ -428,7 +442,122 @@ R_API int r_line_hist_chop(const char *file, int limit) {
 	return 0;
 }
 
+static void selection_widget_draw() {
+	RCons *cons = r_cons_singleton ();
+	RSelWidget *sel_widget = I.sel_widget;
+	int y, pos_y = cons->rows, pos_x = r_str_ansi_len (I.prompt);
+
+	for (y = 0; y < sel_widget->options_len; y++) {
+		sel_widget->w = R_MAX (sel_widget->w, strlen (sel_widget->options[y]));
+	}
+	sel_widget->w = R_MIN (sel_widget->w, R_SELWIDGET_MAXW);
+
+	char *background_color = cons->color ? cons->pal.widget_bg : Color_INVERT_RESET;
+	char *selected_color = cons->color ? cons->pal.widget_sel : Color_INVERT;
+	bool scrollbar = sel_widget->options_len > R_SELWIDGET_MAXH;
+	int scrollbar_y = 0, scrollbar_l = 0;
+	if (scrollbar) {
+		scrollbar_y = (R_SELWIDGET_MAXH * (sel_widget->selection - sel_widget->scroll)) / sel_widget->options_len;
+		scrollbar_l = (R_SELWIDGET_MAXH * R_SELWIDGET_MAXH) / sel_widget->options_len;
+	}
+
+	for (y = 0; y < R_MIN (sel_widget->h, R_SELWIDGET_MAXH); y++) {
+		r_cons_gotoxy (pos_x + 1, pos_y - y - 1);
+		int scroll = R_MAX (0, sel_widget->selection - sel_widget->scroll);
+		const char *option = y < sel_widget->options_len ? sel_widget->options[y + scroll] : "";
+		r_cons_printf ("%s", sel_widget->selection == y + scroll ? selected_color : background_color);
+		r_cons_printf ("%-*.*s", sel_widget->w, sel_widget->w, option);
+		if (scrollbar && R_BETWEEN (scrollbar_y, y, scrollbar_y + scrollbar_l)) {
+			r_cons_memcat (Color_INVERT" "Color_INVERT_RESET, 10);
+		} else {
+			r_cons_memcat (" ", 1);
+		}
+	}
+
+	r_cons_gotoxy (pos_x + I.buffer.length, pos_y);
+	r_cons_memcat (Color_RESET_BG, 5);
+	r_cons_flush ();
+}
+
+static void selection_widget_up(int steps) {
+	RSelWidget *sel_widget = I.sel_widget;
+	if (sel_widget) {
+		int height = R_MIN (sel_widget->h, R_SELWIDGET_MAXH - 1);
+		sel_widget->selection = R_MIN (sel_widget->selection + steps, sel_widget->options_len - 1);
+		if (steps == 1) {
+			sel_widget->scroll = R_MIN (sel_widget->scroll + 1, R_SELWIDGET_MAXH - 1);
+		} else if (sel_widget->selection + (height - sel_widget->scroll) > sel_widget->options_len - 1) {
+			sel_widget->scroll = height - (sel_widget->options_len - 1 - sel_widget->selection);
+		}
+	}
+}
+
+static void selection_widget_down(int steps) {
+	RSelWidget *sel_widget = I.sel_widget;
+	if (sel_widget) {
+		sel_widget->selection = R_MAX (sel_widget->selection - steps, 0);
+		if (steps == 1) {
+			sel_widget->scroll = R_MAX (sel_widget->scroll - 1, 0);
+		} else if (sel_widget->selection - sel_widget->scroll <= 0) {
+			sel_widget->scroll = sel_widget->selection;
+		}
+	}
+}
+
+static void print_rline_task(void *core) {
+	r_cons_clear_line (0);
+	r_cons_printf ("%s%s%s", Color_RESET, I.prompt,  I.buffer.data); 
+	r_cons_flush ();
+}
+
+static void selection_widget_erase() {
+	RSelWidget *sel_widget = I.sel_widget;
+	if (sel_widget) {
+		sel_widget->options_len = 0;
+		sel_widget->selection = -1;
+		selection_widget_draw ();
+		R_FREE (I.sel_widget);
+		RCons *cons = r_cons_singleton ();
+		if (cons->event_resize && cons->event_data) {
+			cons->event_resize (cons->event_data);
+			cons->cb_task_oneshot (cons->user, print_rline_task, NULL);
+		}
+	}
+}
+
+static void selection_widget_select() {
+	RSelWidget *sel_widget = I.sel_widget;
+	if (sel_widget && sel_widget->selection < sel_widget->options_len) {
+		I.buffer.length = R_MIN (strlen (sel_widget->options[sel_widget->selection]), R_LINE_BUFSIZE - 1);
+		memcpy (I.buffer.data, sel_widget->options[sel_widget->selection], I.buffer.length);
+		I.buffer.data[I.buffer.length] = '\0';
+		I.buffer.index = I.buffer.length;
+		selection_widget_erase ();
+	}
+}
+
+static void selection_widget_update() {
+	if (I.completion.argc == 0 ||
+		(I.completion.argc == 1 && I.buffer.length >= strlen (I.completion.argv[0]))) {
+		selection_widget_erase ();
+		return;
+	}
+	if (!I.sel_widget) {
+		RSelWidget *sel_widget = R_NEW0 (RSelWidget);
+		I.sel_widget = sel_widget;
+	}
+	I.sel_widget->scroll = 0;
+	I.sel_widget->selection = 0;
+	I.sel_widget->options_len = I.completion.argc;
+	I.sel_widget->options = I.completion.argv;
+	I.sel_widget->h = R_MAX (I.sel_widget->h, I.completion.argc);
+	selection_widget_draw ();
+	r_cons_flush ();
+	return;
+}
+
 R_API void r_line_autocomplete() {
+
 	int argc = 0;
 	char *p;
 	const char **argv = NULL;
@@ -442,6 +571,11 @@ R_API void r_line_autocomplete() {
 		argv = I.completion.argv;
 	}
 
+	if (I.sel_widget && !I.sel_widget->complete_common) {
+		selection_widget_update ();
+		return;
+	}
+
 	p = (char *) r_sub_str_lchr (I.buffer.data, 0, I.buffer.index, ' ');
 	if (!p) {
 		p = (char *) r_sub_str_lchr (I.buffer.data, 0, I.buffer.index, '@');	// HACK FOR r2
@@ -516,6 +650,14 @@ R_API void r_line_autocomplete() {
 		}
 	}
 
+	if (I.offset_prompt || I.file_prompt) {
+		selection_widget_update ();
+		if (I.sel_widget) {
+			I.sel_widget->complete_common = false;
+		}
+		return;
+	}
+
 	/* show options */
 	if (opt > 1 && I.echo) {
 		const int sep = 3;
@@ -548,6 +690,7 @@ R_API void r_line_autocomplete() {
 R_API const char *r_line_readline() {
 	return r_line_readline_cb (NULL, NULL);
 }
+
 #if __WINDOWS__ && !__CYGWIN__
 R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 	int columns = r_cons_get_size (NULL) - 2;
@@ -560,6 +703,7 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 	char *tmp_ed_cmd, prev = 0;
 	HANDLE hClipBoard;
 	char *clipText;
+	int prev_buflen = 0;
 
 	I.buffer.index = I.buffer.length = 0;
 	I.buffer.data[0] = '\0';
@@ -582,7 +726,7 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 
 	if (I.echo) {
 		r_cons_clear_line (0);
-		printf ("\x1b[0K\r%s%s", I.prompt, I.buffer.data);
+		printf ("%s%s%s", Color_RESET, I.prompt, I.buffer.data);
 		fflush (stdout);
 	}
 	r_cons_break_push (NULL, NULL);
@@ -617,7 +761,10 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 			I.buffer.index = I.buffer.index? I.buffer.index - 1: 0;
 			break;
 		case 38:	// up arrow
-			if (gcomp) {
+			if (I.sel_widget) {
+				selection_widget_up (1);
+				selection_widget_draw ();
+			} else if (gcomp) {
 				gcomp_idx++;
 			} else if (r_line_hist_up () == -1) {
 				r_cons_break_pop ();
@@ -629,7 +776,10 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 					 I.buffer.index + 1: I.buffer.length;
 			break;
 		case 40:// down arrow
-			if (gcomp) {
+			if (I.sel_widget) {
+				selection_widget_up (1);
+				selection_widget_draw ();
+			} else if (gcomp) {
 				if (gcomp_idx > 0) {
 					gcomp_idx--;
 				}
@@ -712,7 +862,7 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 		case 5:	// ^E
 			if (prev == 24) {	// ^X = 0x18
 				I.buffer.data[I.buffer.length] = 0;	// probably unnecessary
-				tmp_ed_cmd = I.editor_cb (I.user, I.buffer.data);
+				tmp_ed_cmd = I.cb_editor (I.user, I.buffer.data);
 				if (tmp_ed_cmd) {
 					/* copied from yank (case 25) */
 					I.buffer.length = strlen (tmp_ed_cmd);
@@ -813,19 +963,13 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 		case 24:// ^X -- do nothing but store in prev = *buf
 			break;
 		case 25:// ^Y - paste
-			if (I.clipboard != NULL) {
-				I.buffer.length += strlen (I.clipboard);
-				// TODO: support endless strings
-				if (I.buffer.length < R_LINE_BUFSIZE) {
-					I.buffer.index = I.buffer.length;
-					strcat (I.buffer.data, I.clipboard);
-				} else {
-					I.buffer.length -= strlen (I.clipboard);
-				}
-			}
+			paste ();
 			break;
 		case 14:// ^n
-			if (gcomp) {
+			if (I.sel_widget) {
+				selection_widget_down (1);
+				selection_widget_draw ();
+			} else if (gcomp) {
 				if (gcomp_idx > 0) {
 					gcomp_idx--;
 				}
@@ -834,7 +978,10 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 			}
 			break;
 		case 16:// ^p
-			if (gcomp) {
+			if (I.sel_widget) {
+				selection_widget_down (1);
+				selection_widget_draw ();
+			} else if (gcomp) {
 				gcomp_idx++;
 			} else {
 				r_line_hist_up ();
@@ -868,10 +1015,17 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 			break;
 		/* tab */
 		case 9:	// tab
+			if (I.sel_widget) {
+				I.sel_widget->complete_common = true;
+			}
 			r_line_autocomplete ();
 			break;
 		/* enter */
 		case 13:
+			if (I.sel_widget) {
+				selection_widget_select ();
+				break;
+			}
 			if (gcomp && I.buffer.length > 0) {
 				strncpy (I.buffer.data, gcomp_line, R_LINE_BUFSIZE - 1);
 				I.buffer.data[R_LINE_BUFSIZE - 1] = '\0';
@@ -899,6 +1053,10 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 			I.buffer.index++;
 			break;
 		}
+		if (I.sel_widget && I.buffer.length != prev_buflen) {
+			prev_buflen = I.buffer.length;
+			r_line_autocomplete ();
+		}
 		prev = buf[0];
 		if (I.echo) {
 			if (gcomp) {
@@ -921,7 +1079,7 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 				int chars = R_MAX (1, strlen (I.buffer.data));	// wtf?
 				int len, cols = R_MAX (1, columns - r_str_ansi_len (I.prompt) - 2);
 				/* print line */
-				printf ("\r%s", I.prompt);
+				printf ("\r%s%s", Color_RESET, I.prompt);
 				fwrite (I.buffer.data, 1, R_MIN (cols, chars), stdout);
 				/* place cursor */
 				printf ("\r%s", I.prompt);
@@ -950,6 +1108,8 @@ _end:
 		fflush (stdout);
 	}
 
+	R_FREE (I.sel_widget);
+
 	// should be here or not?
 	if (!memcmp (I.buffer.data, "!history", 8)) {
 		r_line_hist_list ();
@@ -961,9 +1121,8 @@ _end:
 
 R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 #if __WINDOWS__ && !__CYGWIN__
-#if 1		// new implementation for read input at windows by skuater. If something fail set this to 0
+	// new implementation for read input at windows by skuater. If something fail set this to 0
 	return r_line_readline_cb_win (cb, user);
-#endif
 #endif
 	int columns = r_cons_get_size (NULL) - 2;
 	const char *gcomp_line = "";
@@ -975,6 +1134,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 #endif
 	int ch, i = 0;	/* grep completion */
 	char *tmp_ed_cmd, prev = 0;
+	int prev_buflen = -1;
 
 	I.buffer.index = I.buffer.length = 0;
 	I.buffer.data[0] = '\0';
@@ -997,13 +1157,12 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 
 	if (I.echo) {
 		r_cons_clear_line (0);
-		printf ("\x1b[0K\r%s%s", I.prompt, I.buffer.data);
+		printf ("%s%s%s", Color_RESET, I.prompt, I.buffer.data);
 		fflush (stdout);
 	}
 	r_cons_break_push (NULL, NULL);
 	for (;;) {
 		if (r_cons_is_breaked ()) {
-			eprintf ("CATCH\n");
 			break;
 		}
 		I.buffer.data[I.buffer.length] = '\0';
@@ -1019,7 +1178,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 		}
 		buf[utflen] = 0;
 #else
-		ch = r_line_readchar ();
+		ch = r_cons_readchar ();
 		if (ch == -1) {
 			r_cons_break_pop ();
 			return NULL;
@@ -1041,6 +1200,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			printf ("\r\x1b[2K\r");	// %*c\r", columns, ' ');
 		}
 #endif
+
 		switch (*buf) {
 		case 0:	// control-space
 			/* ignore atm */
@@ -1081,7 +1241,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 				gcomp = false;
 			} else if (prev == 24) {// ^X = 0x18
 				I.buffer.data[I.buffer.length] = 0;	// probably unnecessary
-				tmp_ed_cmd = I.editor_cb (I.user, I.buffer.data);
+				tmp_ed_cmd = I.cb_editor (I.user, I.buffer.data);
 				if (tmp_ed_cmd) {
 					/* copied from yank (case 25) */
 					I.buffer.length = strlen (tmp_ed_cmd);
@@ -1183,50 +1343,19 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			I.buffer.length = 0;
 			I.buffer.index = 0;
 			break;
-		case 23:// ^W ^w
-			if (I.buffer.index > 0) {
-				for (i = I.buffer.index - 1; i > 0 && I.buffer.data[i] == ' '; i--) {
-					/*nothing to see here*/
-				}
-				for (; i && I.buffer.data[i] != ' '; i--) {
-					/*nothing to see here*/
-				}
-				if (!i) {
-					for (; i > 0 && I.buffer.data[i] == ' '; i--) {
-						/*nothing to see here*/
-					}
-				}
-				if (i > 0) {
-					i++;
-				} else if (i < 0) {
-					i = 0;
-				}
-				if (I.buffer.index > I.buffer.length) {
-					I.buffer.length = I.buffer.index;
-				}
-				memmove (I.buffer.data + i,
-					I.buffer.data + I.buffer.index,
-					I.buffer.length - I.buffer.index + 1);
-				I.buffer.length = strlen (I.buffer.data);
-				I.buffer.index = i;
-			}
+		case 23:// ^W ^w unix-word-rubout
+			unix_word_rubout ();
 			break;
 		case 24:// ^X -- do nothing but store in prev = *buf
 			break;
 		case 25:// ^Y - paste
-			if (I.clipboard != NULL) {
-				I.buffer.length += strlen (I.clipboard);
-				// TODO: support endless strings
-				if (I.buffer.length < R_LINE_BUFSIZE) {
-					I.buffer.index = I.buffer.length;
-					strcat (I.buffer.data, I.clipboard);
-				} else {
-					I.buffer.length -= strlen (I.clipboard);
-				}
-			}
+			paste ();
 			break;
 		case 14:// ^n
-			if (gcomp) {
+			if (I.sel_widget) {
+				selection_widget_down (1);
+				selection_widget_draw ();
+			} else if (gcomp) {
 				if (gcomp_idx > 0) {
 					gcomp_idx--;
 				}
@@ -1235,17 +1364,20 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			}
 			break;
 		case 16:// ^p
-			if (gcomp) {
+			if (I.sel_widget) {
+				selection_widget_up (1);
+				selection_widget_draw ();
+			} else if (gcomp) {
 				gcomp_idx++;
 			} else {
 				r_line_hist_up ();
 			}
 			break;
-		case 27:// esc-5b-41-00-00
-			buf[0] = r_line_readchar ();
+		case 27: // esc-5b-41-00-00 alt/meta key
+			buf[0] = r_cons_readchar ();
 			switch (buf[0]) {
 			case 127: // alt+bkspace
-				unix_word_rubout ();
+				backward_kill_word ();
 				break;
 			case -1:
 				r_cons_break_pop ();
@@ -1260,7 +1392,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			case 'b':
 				// previous word
 				for (i = I.buffer.index - 2; i >= 0; i--) {
-					if (I.buffer.data[i] == ' ' && I.buffer.data[i + 1] != ' ') {
+					if (is_word_break_char (I.buffer.data[i]) && !is_word_break_char (I.buffer.data[i + 1])) {
 						I.buffer.index = i + 1;
 						break;
 					}
@@ -1269,11 +1401,15 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 					I.buffer.index = 0;
 				}
 				break;
+			case 'D':
+			case 'd':
+				kill_word ();
+				break;
 			case 'F':
 			case 'f':
 				// next word
 				for (i = I.buffer.index + 1; i < I.buffer.length; i++) {
-					if (I.buffer.data[i] != ' ' && I.buffer.data[i - 1] == ' ') {
+					if (!is_word_break_char (I.buffer.data[i]) && is_word_break_char (I.buffer.data[i - 1])) {
 						I.buffer.index = i;
 						break;
 					}
@@ -1283,7 +1419,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 				}
 				break;
 			default:
-				buf[1] = r_line_readchar ();
+				buf[1] = r_cons_readchar ();
 				if (buf[1] == -1) {
 					r_cons_break_pop ();
 					return NULL;
@@ -1296,15 +1432,32 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 								I.buffer.data + I.buffer.index + 1,
 								strlen (I.buffer.data + I.buffer.index + 1) + 1);
 						}
-						buf[1] = r_line_readchar ();
+						buf[1] = r_cons_readchar ();
 						if (buf[1] == -1) {
 							r_cons_break_pop ();
 							return NULL;
 						}
 						break;
+					case '5': // pag up
+						buf[1] = r_cons_readchar ();
+						if (I.sel_widget) {
+							selection_widget_up (R_MIN (I.sel_widget->h, R_SELWIDGET_MAXH));
+							selection_widget_draw ();
+						}
+						break;
+					case '6': // pag down
+						buf[1] = r_cons_readchar ();
+						if (I.sel_widget) {
+							selection_widget_down (R_MIN (I.sel_widget->h, R_SELWIDGET_MAXH));
+							selection_widget_draw ();
+						}
+						break;
 					/* arrows */
 					case 'A':	// up arrow
-						if (gcomp) {
+						if (I.sel_widget) {
+							selection_widget_up (1);
+							selection_widget_draw ();
+						} else if (gcomp) {
 							gcomp_idx++;
 						} else if (r_line_hist_up () == -1) {
 							r_cons_break_pop ();
@@ -1312,7 +1465,10 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 						}
 						break;
 					case 'B':	// down arrow
-						if (gcomp) {
+						if (I.sel_widget) {
+							selection_widget_down (1);
+							selection_widget_draw ();
+						} else if (gcomp) {
 							if (gcomp_idx > 0) {
 								gcomp_idx--;
 							}
@@ -1407,12 +1563,22 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 					case 0x37:	// HOME xrvt-unicode
 						r_cons_readchar ();
 					case 0x48:	// HOME
+						if (I.sel_widget) {
+							selection_widget_up (I.sel_widget->options_len - 1);
+							selection_widget_draw ();
+							break;
+						}
 						I.buffer.index = 0;
 						break;
 					case 0x34:	// END
 					case 0x38:	// END xrvt-unicode
 						r_cons_readchar ();
 					case 0x46:	// END
+						if (I.sel_widget) {
+							selection_widget_down (I.sel_widget->options_len - 1);
+							selection_widget_draw ();
+							break;
+						}
 						I.buffer.index = I.buffer.length;
 						break;
 					}
@@ -1476,9 +1642,16 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 				I.buffer.length++;
 				I.buffer.index++;
 			}
+			if (I.sel_widget) {
+				I.sel_widget->complete_common = true;
+			}
 			r_line_autocomplete ();
 			break;
-		case 13:
+		case 13: // enter
+			if (I.sel_widget) {
+				selection_widget_select ();
+				break;
+			}
 			if (gcomp && I.buffer.length > 0) {
 				strncpy (I.buffer.data, gcomp_line, R_LINE_BUFSIZE - 1);
 				I.buffer.data[R_LINE_BUFSIZE - 1] = '\0';
@@ -1532,6 +1705,10 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 #endif
 			break;
 		}
+		if (I.sel_widget && I.buffer.length != prev_buflen) {
+			prev_buflen = I.buffer.length;
+			r_line_autocomplete ();
+		}
 		prev = buf[0];
 		if (I.echo) {
 			if (gcomp) {
@@ -1554,7 +1731,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 				int chars = R_MAX (1, strlen (I.buffer.data));	// wtf?
 				int len, cols = R_MAX (1, columns - r_str_ansi_len (I.prompt) - 2);
 				/* print line */
-				printf ("\r%s", I.prompt);
+				printf ("\r%s%s", Color_RESET, I.prompt);
 				fwrite (I.buffer.data, 1, R_MIN (cols, chars), stdout);
 				/* place cursor */
 				printf ("\r%s", I.prompt);
@@ -1583,6 +1760,8 @@ _end:
 		fflush (stdout);
 	}
 
+	R_FREE (I.sel_widget);
+
 	// should be here or not?
 	if (!memcmp (I.buffer.data, "!history", 8)) {
 		// if (I.buffer.data[0]=='!' && I.buffer.data[1]=='\0') {
