@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2009-2017 - pancake */
+/* radare2 - LGPL - Copyright 2009-2018 - pancake */
 
 #include <stddef.h>
 #include <stdbool.h>
@@ -17,6 +17,7 @@ static const char *help_msg_e[] = {
 	"e!", "a", "invert the boolean value of 'a' var",
 	"ec", " [k] [color]", "set color for given key (prompt, offset, ...)",
 	"ee", "var", "open editor to change the value of var",
+	"ed", "", "open editor to change the ~/.radare2rc",
 	"ej", "", "list config vars in JSON",
 	"env", " [k[=v]]", "get/set environment variable",
 	"er", " [key]", "set config key as readonly. no way back",
@@ -27,11 +28,34 @@ static const char *help_msg_e[] = {
 	NULL
 };
 
+static const char *help_msg_ec[] = {
+	"Usage ec[s?] [key][[=| ]fg] [bg]", "", "",
+	"ec", "", "list all color keys",
+	"ec*", "", "same as above, but using r2 commands",
+	"ecd", "", "set default palette",
+	"ecr", "", "set random palette (see also scr.randpal)",
+	"ecs", "", "show a colorful palette",
+	"ecj", "", "show palette in JSON",
+	"ecc", " [prefix]", "show palette in CSS",
+	"eco", " dark|white", "load white color scheme template",
+	"ecp", "", "load previous color theme",
+	"ecn", "", "load next color theme",
+	"ecH", "[?]", "highlight word or instruction",
+	"ec", " prompt red", "change color of prompt",
+	"ec", " prompt red blue", "change color and background of prompt",
+	"", " ", "",
+	"colors:", "", "rgb:000, red, green, blue, #ff0000, ...",
+	"e scr.color", "=0", "use more colors (0: no color 1: ansi 16, 2: 256, 3: 16M)",
+	"$DATADIR/radare2/cons", "", R_JOIN_2_PATHS ("~", R2_HOME_THEMES) " ./",
+	NULL
+};
+
 static char *curtheme = NULL;
 static bool getNext = false;
 
 static void cmd_eval_init(RCore *core) {
 	DEFINE_CMD_DESCRIPTOR (core, e);
+	DEFINE_CMD_DESCRIPTOR (core, ec);
 }
 
 static bool load_theme(RCore *core, const char *path) {
@@ -78,32 +102,34 @@ static bool nextpal_item(RCore *core, int mode, const char *file, int ctr) {
 	return true;
 }
 
-R_API RList *r_core_list_themes(RCore *core) {
-	RList *files = NULL;
+static void list_themes_in_path(RList *list, const char *path) {
 	RListIter *iter;
 	const char *fn;
-	char *home = r_str_home (".config/radare2/cons/");
-
-	RList *list = r_list_new ();
-	getNext = false;
-	if (home) {
-		files = r_sys_dir (home);
-		r_list_foreach (files, iter, fn) {
-			if (*fn && *fn != '.') {
-				r_list_append (list, strdup (fn));
-			}
-		}
-		r_list_free (files);
-		R_FREE (home);
-	}
-	files = r_sys_dir (R2_DATDIR"/radare2/"R2_VERSION"/cons/");
+	RList *files = r_sys_dir (path);
 	r_list_foreach (files, iter, fn) {
 		if (*fn && *fn != '.') {
 			r_list_append (list, strdup (fn));
 		}
 	}
 	r_list_free (files);
-	files = NULL;
+}
+
+R_API RList *r_core_list_themes(RCore *core) {
+	RList *list = r_list_newf (free);
+	getNext = false;
+
+	char *path = r_str_home (R2_HOME_THEMES R_SYS_DIR);
+	if (path) {
+		list_themes_in_path (list, path);
+		R_FREE (path);
+	}
+
+	path = r_str_r2_prefix (R2_THEMES R_SYS_DIR);
+	if (path) {
+		list_themes_in_path (list, path);
+		R_FREE (path);
+	}
+
 	return list;
 }
 
@@ -113,7 +139,7 @@ static void nextpal(RCore *core, int mode) {
 	RListIter *iter;
 	const char *fn;
 	int ctr = 0;
-	char *home = r_str_home (".config/radare2/cons/");
+	char *home = r_str_home (R2_HOME_THEMES R_SYS_DIR);
 
 	getNext = false;
 	if (mode == 'j') {
@@ -152,29 +178,34 @@ static void nextpal(RCore *core, int mode) {
 		r_list_free (files);
 		R_FREE (home);
 	}
-	files = r_sys_dir (R2_DATDIR"/radare2/"R2_VERSION"/cons/");
-	r_list_foreach (files, iter, fn) {
-		if (*fn && *fn != '.') {
-			if (mode == 'p') {
-				const char *nfn = iter->n? iter->n->data: NULL;
-				if (!curtheme) {
-					free (home);
-					r_list_free (files);
-					return;
-				}
-				eprintf ("%s %s %s\n", nfn, curtheme, fn);
-				if (nfn && !strcmp (nfn, curtheme)) {
-					free (curtheme);
-					curtheme = strdup (fn);
-					goto done;
-				}
-			} else {
-				if (!nextpal_item (core, mode, fn, ctr++)) {
-					goto done;
+
+	char *path = r_str_r2_prefix (R2_THEMES R_SYS_DIR);
+	if (path) {
+		files = r_sys_dir (path);
+		r_list_foreach (files, iter, fn) {
+			if (*fn && *fn != '.') {
+				if (mode == 'p') {
+					const char *nfn = iter->n? iter->n->data: NULL;
+					if (!curtheme) {
+						free (home);
+						r_list_free (files);
+						return;
+					}
+					eprintf ("%s %s %s\n", nfn, curtheme, fn);
+					if (nfn && !strcmp (nfn, curtheme)) {
+						free (curtheme);
+						curtheme = strdup (fn);
+						goto done;
+					}
+				} else {
+					if (!nextpal_item (core, mode, fn, ctr++)) {
+						goto done;
+					}
 				}
 			}
 		}
 	}
+
 done:
 	if (getNext) {
 		R_FREE (curtheme);
@@ -183,7 +214,7 @@ done:
 	}
 	if (mode == 'l' && !curtheme && !r_list_empty (files)) {
 		//nextpal (core, mode);
-	} else {
+	} else if (mode == 'n' || mode == 'p') {
 		if (curtheme) {
 			r_core_cmdf (core, "eco %s", curtheme);
 		}
@@ -196,10 +227,20 @@ done:
 }
 
 static int cmd_eval(void *data, const char *input) {
-	char *p;
 	RCore *core = (RCore *)data;
 	switch (input[0]) {
-	case 't': // env
+	case '\0': // "e"
+		r_config_list (core->config, NULL, 0);
+		break;
+	case '?': // "e?"
+	default:
+		switch (input[1]) {
+		case '\0': r_core_cmd_help (core, help_msg_e); break;
+		case '?': r_config_list (core->config, input + 2, 2); break;
+		default: r_config_list (core->config, input + 1, 2); break;
+		}
+		break;
+	case 't': // "et"
 		if (input[1] == 'a') {
 			r_cons_printf ("%s\n", (r_num_rand (10) % 2)? "wen": "son");
 		} else if (input[1]==' ' && input[2]) {
@@ -214,7 +255,7 @@ static int cmd_eval(void *data, const char *input) {
 			eprintf ("Usage: et [varname]  ; show type of eval var\n");
 		}
 		break;
-	case 'n': // env
+	case 'n': // "en"
 		if (!strchr (input, '=')) {
 			char *var, *p;
 			var = strchr (input, ' ');
@@ -253,49 +294,29 @@ static int cmd_eval(void *data, const char *input) {
 	case 'q': // quiet list of eval keys
 		r_config_list (core->config, NULL, 'q');
 		break;
-	case '\0': // "e"
-		r_config_list (core->config, NULL, 0);
-		break;
 	case 'c': // "ec"
 		switch (input[1]) {
-		case 'd':
-			r_cons_pal_init (NULL);
+		case 'd': // "ecd"
+			r_cons_pal_init ();
 			break;
-		case '?': {
-			const char *helpmsg[] = {
-			"Usage ec[s?] [key][[=| ]fg] [bg]","","",
-			"ec","","list all color keys",
-			"ec*","","same as above, but using r2 commands",
-			"ecd","","set default palette",
-			"ecr","","set random palette (see also scr.randpal)",
-			"ecs","","show a colorful palette",
-			"ecj","","show palette in JSON",
-			"ecc"," [prefix]","show palette in CSS",
-			"eco"," dark|white","load white color scheme template",
-			"ecp","","load previous color theme",
-			"ecn","","load next color theme",
-			"ecH","[?]","highlight word or instruction",
-			"ec"," prompt red","change color of prompt",
-			"ec"," prompt red blue","change color and background of prompt",
-			""," ","",
-			"colors:","","rgb:000, red, green, blue, ...",
-			"e scr.rgbcolor","=1|0","for 256 color cube (boolean)",
-			"e scr.truecolor","=1|0","for 256*256*256 colors (boolean)",
-			"$DATADIR/radare2/cons","","~/.config/radare2/cons ./",
-			NULL};
-			r_core_cmd_help (core, helpmsg);
-			}
+		case '?':
+			r_core_cmd_help (core, help_msg_ec);
 			break;
 		case 'o': // "eco"
 			if (input[2] == 'j') {
 				nextpal (core, 'j');
 			} else if (input[2] == ' ') {
 				bool failed = false;
-				char *home, path[512];
-				snprintf (path, sizeof (path), ".config/radare2/cons/%s", input + 3);
-				home = r_str_home (path);
-				snprintf (path, sizeof (path), R2_DATDIR"/radare2/"
-					R2_VERSION"/cons/%s", input + 3);
+				char *home, *path, *tmp;
+
+				tmp = r_str_newf (R_JOIN_2_PATHS (R2_HOME_THEMES, "%s"), input + 3);
+				home = tmp ? r_str_home (tmp) : NULL;
+				free (tmp);
+
+				tmp = r_str_newf (R_JOIN_2_PATHS (R2_THEMES, "%s"), input + 3);
+				path = tmp ? r_str_r2_prefix (tmp) : NULL;
+				free (tmp);
+
 				if (!load_theme (core, home)) {
 					if (load_theme (core, path)) {
 						//curtheme = r_str_dup (curtheme, path);
@@ -312,11 +333,14 @@ static int cmd_eval(void *data, const char *input) {
 					}
 				}
 				free (home);
+				free (path);
 				if (failed) {
 					eprintf ("Something went wrong\n");
 				}
 			} else if (input[2] == '?') {
-				eprintf ("Usage: eco [themename]  ;load theme from "R2_DATDIR"/radare2/"R2_VERSION"/cons/\n");
+				eprintf ("Usage: eco [themename]  ;load theme from "
+					R_JOIN_3_PATHS ("%s", R2_THEMES, "") " (see dir.prefix)\n",
+					r_sys_prefix (NULL));
 
 			} else {
 				nextpal (core, 'l');
@@ -325,11 +349,13 @@ static int cmd_eval(void *data, const char *input) {
 		case 's': r_cons_pal_show (); break; // "ecs"
 		case '*': r_cons_pal_list (1, NULL); break; // "ec*"
 		case 'h': // echo
-			if (( p = strchr (input, ' ') )) {
-				r_cons_strcat (p+1);
-				r_cons_newline ();
+			if (input[2] == 'o') {
+				char *p = strchr (input, ' ');
+				if (p) {
+					r_cons_strcat (p + 1);
+					r_cons_newline ();
+				}
 			} else {
-				// "ech"
 				r_cons_pal_list ('h', NULL);
 			}
 			break;
@@ -372,10 +398,10 @@ static int cmd_eval(void *data, const char *input) {
 				r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, "");
 				return false;
 			case '\0':
-			case 'i': // "ecHi
+			case 'i': // "ecHi"
 				if (argc) {
 					char *dup = r_str_newf ("bgonly %s", argv[0]);
-					color_code = r_cons_pal_parse (dup);
+					color_code = r_cons_pal_parse (dup, NULL);
 					R_FREE (dup);
 				}
 				break;
@@ -388,7 +414,7 @@ static int cmd_eval(void *data, const char *input) {
 				word = strdup (argv[0]);
 				if (argc > 1) {
 					char *dup = r_str_newf ("bgonly %s", argv[1]);
-					color_code = r_cons_pal_parse (dup);
+					color_code = r_cons_pal_parse (dup, NULL);
 					if (!color_code) {
 						eprintf ("Unknown color %s\n", argv[1]);
 						r_str_argv_free (argv);
@@ -405,7 +431,7 @@ static int cmd_eval(void *data, const char *input) {
 				return true;
 			}
 			char *str = r_meta_get_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset);
-			char *dup = r_str_newf ("%s \"%s%s\"", str?str:"", word?word:"", color_code?color_code:r_cons_pal_get ("highlight"));
+			char *dup = r_str_newf ("%s \"%s%s\"", str?str:"", word?word:"", color_code?color_code:r_cons_singleton ()->pal.wordhl);
 			r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, dup);
 			r_str_argv_free (argv);
 			R_FREE (word);
@@ -419,20 +445,49 @@ static int cmd_eval(void *data, const char *input) {
 				q = strchr (p, ' ');
 			}
 			if (q) {
-				// set
+				// Set color
 				*q++ = 0;
-				r_cons_pal_set (p, q);
-			} else {
-				const char *k = r_cons_pal_get (p);
-				if (k) {
-					eprintf ("(%s)(%sCOLOR"Color_RESET")\n", p, k);
+				if (r_cons_pal_set (p, q)) {
+					r_cons_pal_update_event ();
 				}
+			} else {
+				char color[32];
+				RColor rcolor = r_cons_pal_get (p);
+				r_cons_rgb_str (color, sizeof (color), &rcolor);
+				eprintf ("(%s)(%sCOLOR"Color_RESET")\n", p, color);
 			}
 			free (p);
 		}
 		}
 		break;
-	case 'e':
+	case 'd': // "ed"
+		if (input[1] == '?') {
+			eprintf ("Usage: ed[-][?] - edit ~/.radare2rc with cfg.editor\n");
+			eprintf ("NOTE: ~ is HOME and this can be changed with %%HOME=/tmp\n");
+			eprintf ("  ed    : ${cfg.editor} ~/.radare2rc\n");
+			eprintf ("  ed-   : rm ~/.radare2rc\n");
+		} else if (input[1] == '-') {
+			char *file = r_str_home (".radare2rc");
+			r_cons_printf ("rm %s\n", file);
+			// r_file_rm (file);
+			free (file);
+		} else {
+			char *file = r_str_home (".radare2rc");
+			if (r_config_get_i (core->config, "scr.interactive")) {
+				r_file_touch (file);
+				char * res = r_cons_editor (file, NULL);
+				if (res) {
+					if (r_cons_yesno ('y', "Reload? (Y/n)")) {
+						r_core_run_script (core, file);
+					}
+				}
+			} else {
+				r_core_run_script (core, file);
+			}
+			free (file);
+		}
+		break;
+	case 'e': // "ee"
 		if (input[1] == ' ') {
 			char *p;
 			const char *val, *input2 = strchr (input+2, ' ');
@@ -447,28 +502,22 @@ static int cmd_eval(void *data, const char *input) {
 			eprintf ("Usage: ee varname\n");
 		}
 		break;
-	case '!':
+	case '!': // "e!"
 		input = r_str_trim_ro (input+1);
 		if (!r_config_toggle (core->config, input))
 			eprintf ("r_config: '%s' is not a boolean variable.\n", input);
 		break;
-	case 's':
+	case 's': // "es"
 		r_config_list (core->config, (input[1])? input + 1: NULL, 's');
 		break;
-	case '-':
+	case '-': // "e-"
 		r_core_config_init (core);
 		//eprintf ("BUG: 'e-' command locks the eval hashtable. patches are welcome :)\n");
 		break;
-	case '*': r_config_list (core->config, NULL, 1); break;
-	case '?':
-		switch (input[1]) {
-		case '?': r_config_list (core->config, input+2, 2); break;
-		default: r_config_list (core->config, input+1, 2); break;
-		case 0:
-			r_core_cmd_help (core, help_msg_e);
-		}
+	case '*': // "e*"
+		r_config_list (core->config, NULL, 1);
 		break;
-	case 'r':
+	case 'r': // "er"
 		if (input[1]) {
 			const char *key = input+((input[1]==' ')?2:1);
 			if (!r_config_readonly (core->config, key)) {
@@ -479,7 +528,6 @@ static int cmd_eval(void *data, const char *input) {
 		}
 		break;
 	case ' ': r_config_eval (core->config, input+1); break;
-	default: r_config_eval (core->config, input); break;
 	}
 	return 0;
 }
