@@ -25,7 +25,7 @@ static const char *help_msg_i[] = {
 	"idp", "", "Load pdb file information",
 	"iD", " lang sym", "demangle symbolname for given language",
 	"ie", "", "Entrypoint",
-	"iee", "", "Show Entry and Exit (init and fini)",
+	"iee", "", "Show Entry and Exit (preinit, init and fini)",
 	"iE", "", "Exports (global symbols)",
 	"iE.", "", "Current export",
 	"ih", "", "Headers (alias for iH)",
@@ -38,13 +38,16 @@ static const char *help_msg_i[] = {
 	"im", "", "Show info about predefined memory allocation",
 	"iM", "", "Show main address",
 	"io", " [file]", "Load info from file (or last opened) use bin.baddr",
+	"iO", "[?]", "Perform binary operation (dump, resize, change sections, ...)",
 	"ir", "", "Relocs",
 	"iR", "", "Resources",
 	"is", "", "Symbols",
 	"is.", "", "Current symbol",
 	"iS ", "[entropy,sha1]", "Sections (choose which hash algorithm to use)",
 	"iS.", "", "Current section",
+	"iSS", " [entropy,sha1]", "Segments",
 	"iV", "", "Display file version info",
+	"iX", "", "Display source files used (via dwarf)",
 	"iz|izj", "", "Strings in data sections (in JSON/Base64)",
 	"izz", "", "Search for Strings in the whole binary",
 	"izzz", "", "Dump Strings from whole binary to r2 shell (for huge files)",
@@ -95,6 +98,7 @@ static bool demangle_internal(RCore *core, const char *lang, const char *s) {
 	case R_BIN_NM_OBJC: res = r_bin_demangle_objc (NULL, s); break;
 	case R_BIN_NM_SWIFT: res = r_bin_demangle_swift (s, core->bin->demanglercmd); break;
 	case R_BIN_NM_DLANG: res = r_bin_demangle_plugin (core->bin, "dlang", s); break;
+	case R_BIN_NM_MSVC: res = r_bin_demangle_msvc (s); break;
 	default:
 		r_bin_demangle_list (core->bin);
 		return true;
@@ -150,7 +154,7 @@ static void r_core_file_info(RCore *core, int mode) {
 	if (info) {
 		fn = info->file;
 		if (mode == R_CORE_BIN_JSON) {
-			r_cons_printf ("\"type\":\"%s\"", STR (info->type));
+			r_cons_printf ("\"type\":\"%s\",", STR (info->type));
 		}
 	} else {
 		fn = desc ? desc->name: NULL;
@@ -166,7 +170,7 @@ static void r_core_file_info(RCore *core, int mode) {
 		}
 		{
 			char *escapedFile = r_str_utf16_encode (uri, -1);
-			r_cons_printf (",\"file\":\"%s\"", escapedFile);
+			r_cons_printf ("\"file\":\"%s\"", escapedFile);
 			free (escapedFile);
 		}
 		if (dbg) {
@@ -210,11 +214,11 @@ static void r_core_file_info(RCore *core, int mode) {
 			dbg = R_IO_WRITE | R_IO_EXEC;
 		}
 		if (desc) {
-			pair ("blksz", sdb_fmt (0, "0x%"PFMT64x, (ut64) core->io->desc->obsz));
+			pair ("blksz", sdb_fmt ("0x%"PFMT64x, (ut64) core->io->desc->obsz));
 		}
-		pair ("block", sdb_fmt (0, "0x%x", core->blocksize));
+		pair ("block", sdb_fmt ("0x%x", core->blocksize));
 		if (desc) {
-			pair ("fd", sdb_fmt (0, "%d", desc->fd));
+			pair ("fd", sdb_fmt ("%d", desc->fd));
 		}
 		if (fn || (desc && desc->uri)) {
 			pair ("file", fn? fn: desc->uri);
@@ -235,7 +239,7 @@ static void r_core_file_info(RCore *core, int mode) {
 		if (desc) {
 			ut64 fsz = r_io_desc_size (desc);
 			if (fsz != UT64_MAX) {
-				pair ("size", sdb_fmt (0,"0x%"PFMT64x, fsz));
+				pair ("size", sdb_itoca (fsz));
 				char *humansz = r_num_units (NULL, fsz);
 				if (humansz) {
 					pair ("humansz", humansz);
@@ -271,22 +275,22 @@ static void cmd_info_bin(RCore *core, int va, int mode) {
 	if (core->file) {
 		if ((mode & R_CORE_BIN_JSON) && !(mode & R_CORE_BIN_ARRAY)) {
 			mode = R_CORE_BIN_JSON;
-			r_cons_printf ("{\"core\":");
+			r_cons_strcat ("{\"core\":");
 		}
 		if ((mode & R_CORE_BIN_JSON) && (mode & R_CORE_BIN_ARRAY)) {
 			mode = R_CORE_BIN_JSON;
 			array = 1;
-			r_cons_printf (",\"core\":");
+			r_cons_strcat (",\"core\":");
 		}
 		r_core_file_info (core, mode);
 		if (bin_is_executable (obj)) {
 			if ((mode & R_CORE_BIN_JSON)) {
-				r_cons_printf (",\"bin\":");
+				r_cons_strcat (",\"bin\":");
 			}
 			r_core_bin_info (core, R_CORE_BIN_ACC_INFO, mode, va, NULL, NULL);
 		}
 		if (mode == R_CORE_BIN_JSON && array == 0) {
-			r_cons_printf ("}\n");
+			r_cons_strcat ("}\n");
 		}
 	} else {
 		eprintf ("No file selected\n");
@@ -327,9 +331,12 @@ static int cmd_info(void *data, const char *input) {
 		case 'q': mode = R_CORE_BIN_SIMPLE; break;
 		}
 	}
-
 	if (mode == R_CORE_BIN_JSON) {
-		if (strlen (input + 1) > 1) {
+		int suffix_shift = 0;
+		if (!strncmp (input, "SS", 2) || !strncmp (input, "ee", 2)) {
+			suffix_shift = 1;
+		}
+		if (strlen (input + 1 + suffix_shift) > 1) {
 			is_array = 1;
 		}
 	}
@@ -360,7 +367,7 @@ static int cmd_info(void *data, const char *input) {
 			newline = false;
 		}
 		break;
-		case 'k':
+		case 'k': // "ik"
 		{
 			RBinObject *o = r_bin_cur_object (core->bin);
 			db = o? o->kv: NULL;
@@ -405,7 +412,7 @@ static int cmd_info(void *data, const char *input) {
 			goto done;
 		}
 		break;
-		case 'o':
+		case 'o': // "io"
 		{
 			if (!desc) {
 				eprintf ("Core file not open\n");
@@ -424,7 +431,7 @@ static int cmd_info(void *data, const char *input) {
 				}\
 				if (z) { playMsg (core, n, z);}\
 				r_core_bin_info (core, x, mode, va, NULL, y);
-		case 'A':
+		case 'A': // "iA"
 			newline = false;
 			if (input[1] == 'j') {
 				r_cons_printf ("{");
@@ -434,9 +441,8 @@ static int cmd_info(void *data, const char *input) {
 				r_bin_list_archs (core->bin, 1);
 			}
 			break;
-		case 'E': 
+		case 'E': // "iE"
 		{
-			// case for iEj.
 			if (input[1] == 'j' && input[2] == '.') {	
 				mode = R_CORE_BIN_JSON;
 				RBININFO ("exports", R_CORE_BIN_ACC_EXPORTS, input + 2, 0); 
@@ -447,45 +453,67 @@ static int cmd_info(void *data, const char *input) {
 			input--;
 			break;
 		}
-		case 'Z': RBININFO ("size", R_CORE_BIN_ACC_SIZE, NULL, 0); break;
-		case 'S':
+		case 'Z': // "iZ"
+			RBININFO ("size", R_CORE_BIN_ACC_SIZE, NULL, 0);
+			break;
+		case 'O': // "iO"
+			switch (input[1]) {
+			case ' ':
+			        r_sys_cmdf ("rabin2 -O \"%s\" \"%s\"", r_str_trim_ro (input + 1), desc->name);
+			        break;
+			default:
+			        r_sys_cmdf ("rabin2 -O help");
+			        break;
+			}
+			return 0;
+		case 'S': // "iS"
 			//we comes from ia or iS
 			if ((input[1] == 'm' && input[2] == 'z') || !input[1]) {
 				RBININFO ("sections", R_CORE_BIN_ACC_SECTIONS, NULL, 0);
-			} else {  //iS entropy,sha1
+			} else if (input[1] == 'S' && !input[2]) {  // "iSS"
+				RBININFO ("segments", R_CORE_BIN_ACC_SEGMENTS, NULL, 0);
+			} else {  //iS/iSS entropy,sha1
+				const char *name = "sections";
+				int action = R_CORE_BIN_ACC_SECTIONS;
+				int param_shift = 0;
+				if (input[1] == 'S') {
+					name = "segments";
+					action = R_CORE_BIN_ACC_SEGMENTS;
+					param_shift = 1;
+				}
 				// case for iSj.
 				if (input[1] == 'j' && input[2] == '.') {
 					mode = R_CORE_BIN_JSON;
 				}
 				RBinObject *obj = r_bin_cur_object (core->bin);
 				if (mode == R_CORE_BIN_RADARE || mode == R_CORE_BIN_JSON || mode == R_CORE_BIN_SIMPLE) {
-					RBININFO ("sections", R_CORE_BIN_ACC_SECTIONS, input + 2,
+					RBININFO (name, action, input + 2 + param_shift,
 						obj? r_list_length (obj->sections): 0);
 				} else {
-					RBININFO ("sections", R_CORE_BIN_ACC_SECTIONS, input + 1,
+					RBININFO (name, action, input + 1 + param_shift,
 						obj? r_list_length (obj->sections): 0);
 				}
-				//we move input until get '\0'
-				while (*(++input)) ;
-				//input-- because we are inside a while that does input++
-				// oob read if not input--
-				input--;
 			}
+			//we move input until get '\0'
+			while (*(++input)) ;
+			//input-- because we are inside a while that does input++
+			// oob read if not input--
+			input--;
 			break;
-		case 'H':
+		case 'H': // "iH"
 			if (input[1] == 'H') { // "iHH"
 				RBININFO ("header", R_CORE_BIN_ACC_HEADER, NULL, -1);
 				break;
 			}
-		case 'h': RBININFO ("fields", R_CORE_BIN_ACC_FIELDS, NULL, 0); break;
-		case 'l':
-			  {
-				  RBinObject *obj = r_bin_cur_object (core->bin);
-				  RBININFO ("libs", R_CORE_BIN_ACC_LIBS, NULL, obj? r_list_length (obj->libs): 0);
-			  }
-			  break;
-		case 'L':
-		{
+		case 'h': // "ih"
+			RBININFO ("fields", R_CORE_BIN_ACC_FIELDS, NULL, 0);
+			break;
+		case 'l': { // "il"
+			RBinObject *obj = r_bin_cur_object (core->bin);
+			RBININFO ("libs", R_CORE_BIN_ACC_LIBS, NULL, obj? r_list_length (obj->libs): 0);
+			break;
+		}
+		case 'L': { // "iL"
 			char *ptr = strchr (input, ' ');
 			int json = input[1] == 'j'? 'j': 0;
 
@@ -498,27 +526,23 @@ static int cmd_info(void *data, const char *input) {
 			} else {
 				r_bin_list (core->bin, json);
 			}
-
 			newline = false;
-
 			goto done;
 		}
-		break;
-		case 's':
-			{
-				RBinObject *obj = r_bin_cur_object (core->bin);
-  			// Case for isj.
-				if (input[1] == 'j' && input[2] == '.') {
-					mode = R_CORE_BIN_JSON;
-					RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 2, obj? r_list_length (obj->symbols): 0);
-				} else {
-					RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 1, obj? r_list_length (obj->symbols): 0);
-				}
-				while (*(++input)) ;
-				input--;
-				break;
+		case 's': { // "is"
+			RBinObject *obj = r_bin_cur_object (core->bin);
+			// Case for isj.
+			if (input[1] == 'j' && input[2] == '.') {
+				mode = R_CORE_BIN_JSON;
+				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 2, obj? r_list_length (obj->symbols): 0);
+			} else {
+				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 1, obj? r_list_length (obj->symbols): 0);
 			}
-		case 'R':
+			while (*(++input)) ;
+			input--;
+			break;
+		}
+		case 'R': // "iR"
 			if  (input[1] == '*') {
 				mode = R_CORE_BIN_RADARE;
 			} else if (input[1] == 'j') {
@@ -526,7 +550,12 @@ static int cmd_info(void *data, const char *input) {
 			}
 			RBININFO ("resources", R_CORE_BIN_ACC_RESOURCES, NULL, 0);
 			break;
-		case 'r': RBININFO ("relocs", R_CORE_BIN_ACC_RELOCS, NULL, 0); break;
+		case 'r': // "ir"
+			RBININFO ("relocs", R_CORE_BIN_ACC_RELOCS, NULL, 0);
+			break;
+		case 'X': // "iX"
+			RBININFO ("source", R_CORE_BIN_ACC_SOURCE, NULL, 0);
+			break;
 		case 'd': // "id"
 			if (input[1] == 'p') { // "idp"
 				SPDBOptions pdbopts;
@@ -546,6 +575,7 @@ static int cmd_info(void *data, const char *input) {
 					pdbopts.user_agent = (char*) r_config_get (core->config, "pdb.useragent");
 					pdbopts.symbol_server = (char*) r_config_get (core->config, "pdb.server");
 					pdbopts.extract = r_config_get_i (core->config, "pdb.extract");
+					pdbopts.symbol_store_path = (char*) r_config_get (core->config, "pdb.symstore");
 					int r = r_bin_pdb_download (core, 0, NULL, &pdbopts);
 					if (r > 0) {
 						eprintf ("Error while downloading pdb file");
@@ -584,10 +614,24 @@ static int cmd_info(void *data, const char *input) {
 								filename = strdup (basename);
 							}
 						}
+
+						// Last chance: Check if file is in downstream symbol store
+						if (!file_found) {
+							const char* symstore_path = r_config_get (core->config, "pdb.symstore");
+							char* pdb_path = r_str_newf ("%s" R_SYS_DIR "%s" R_SYS_DIR "%s" R_SYS_DIR "%s",
+										     symstore_path, info->debug_file_name,
+										     info->guid, info->debug_file_name);
+							file_found = r_file_exists (pdb_path);
+							if (file_found) {
+								filename = pdb_path;
+							} else {
+								R_FREE(pdb_path);
+							}
+						}
 					}
 
 					if (!file_found) {
-						eprintf ("File '%s' not found", filename);
+						eprintf ("File '%s' not found in file directory or symbol store", info->debug_file_name);
 						free (filename);
 						break;
 					}
@@ -597,16 +641,7 @@ static int cmd_info(void *data, const char *input) {
 					} else {
 						eprintf ("Warning: Cannot find base address, flags will probably be misplaced\n");
 					}
-					void *res = r_core_file_open (core, filename, R_IO_READ, baddr);
-					if (!res) {
-						eprintf ("Error while opening '%s'", filename);
-						break;
-					}
-					int fd = r_io_fd_get_current (core->io);
-					r_core_bin_load (core, filename, baddr);
-					RCoreBinFilter filter = { 0 };
-					r_core_bin_info (core, R_CORE_BIN_ACC_PDB, mode, true, &filter, NULL);
-					r_core_file_close_fd (core, fd);
+					r_core_pdb_info (core, filename, baddr, mode);
 					free (filename);
 					break;
 				case '?':
@@ -623,26 +658,36 @@ static int cmd_info(void *data, const char *input) {
 				RBININFO ("dwarf", R_CORE_BIN_ACC_DWARF, NULL, -1);
 			}
 			break;
-		case 'i': {
-				  RBinObject *obj = r_bin_cur_object (core->bin);
-				  RBININFO ("imports", R_CORE_BIN_ACC_IMPORTS, NULL,
-						  obj? r_list_length (obj->imports): 0);
-			  }
-			  break;
-		case 'I': RBININFO ("info", R_CORE_BIN_ACC_INFO, NULL, 0); break;
-		case 'e':
-			  if (input[1] == 'e') {
-				  RBININFO ("entries", R_CORE_BIN_ACC_INITFINI, NULL, 0);
-				  input++;
-			  } else {
-				  RBININFO ("entries", R_CORE_BIN_ACC_ENTRIES, NULL, 0);
-			  }
+		case 'i': { // "ii"
+			RBinObject *obj = r_bin_cur_object (core->bin);
+			RBININFO ("imports", R_CORE_BIN_ACC_IMPORTS, NULL,
+				obj? r_list_length (obj->imports): 0);
 			break;
-		case 'M': RBININFO ("main", R_CORE_BIN_ACC_MAIN, NULL, 0); break;
-		case 'm': RBININFO ("memory", R_CORE_BIN_ACC_MEM, NULL, 0); break;
-		case 'V': RBININFO ("versioninfo", R_CORE_BIN_ACC_VERSIONINFO, NULL, 0); break;
-		case 'C': RBININFO ("signature", R_CORE_BIN_ACC_SIGNATURE, NULL, 0); break;
-		case 'z':
+		}
+		case 'I': // "iI"
+			RBININFO ("info", R_CORE_BIN_ACC_INFO, NULL, 0);
+			break;
+		case 'e': // "ie"
+			if (input[1] == 'e') {
+				RBININFO ("initfini", R_CORE_BIN_ACC_INITFINI, NULL, 0);
+				input++;
+			} else {
+				RBININFO ("entries", R_CORE_BIN_ACC_ENTRIES, NULL, 0);
+			}
+			break;
+		case 'M': // "iM"
+			RBININFO ("main", R_CORE_BIN_ACC_MAIN, NULL, 0);
+			break;
+		case 'm': // "im"
+			RBININFO ("memory", R_CORE_BIN_ACC_MEM, NULL, 0);
+			break;
+		case 'V': // "iV"
+			RBININFO ("versioninfo", R_CORE_BIN_ACC_VERSIONINFO, NULL, 0);
+			break;
+		case 'C': // "iC"
+			RBININFO ("signature", R_CORE_BIN_ACC_SIGNATURE, NULL, 0);
+			break;
+		case 'z': // "iz"
 			if (input[1] == '-') { //iz-
 				char *strpurge = core->bin->strpurge;
 				ut64 addr = core->offset;
@@ -691,11 +736,8 @@ static int cmd_info(void *data, const char *input) {
 					RBinFile *bf = r_core_bin_cur (core);
 					int min = r_config_get_i (core->config, "bin.minstr");
 					if (bf) {
-						int tmp = bf->rawstr;
-						bf->rawstr = 2;
 						bf->strmode = mode;
-						r_bin_dump_strings (bf, min);
-						bf->rawstr = tmp;
+						r_bin_dump_strings (bf, min, 2);
 					}
 					goto done;
 				}
@@ -714,7 +756,7 @@ static int cmd_info(void *data, const char *input) {
 				}
 			}
 			break;
-		case 'c': // for r2 `ic`
+		case 'c': // "ic"
 			if (input[1] == '?') {
 				eprintf ("Usage: ic[ljqc*] [class-index or name]\n");
 			} else if (input[1] == ' ' || input[1] == 'q' || input[1] == 'j' || input[1] == 'l' || input[1] == 'c') {
@@ -816,12 +858,12 @@ static int cmd_info(void *data, const char *input) {
 				RBININFO ("classes", R_CORE_BIN_ACC_CLASSES, NULL, len);
 			}
 			break;
-		case 'D':
+		case 'D': // "iD"
 			if (input[1] != ' ' || !demangle (core, input + 2)) {
 				eprintf ("|Usage: iD lang symbolname\n");
 			}
 			return 0;
-		case 'a':
+		case 'a': // "ia"
 			switch (mode) {
 			case R_CORE_BIN_RADARE: cmd_info (core, "IieEcsSmz*"); break;
 			case R_CORE_BIN_JSON: cmd_info (core, "IieEcsSmzj"); break;
@@ -829,17 +871,17 @@ static int cmd_info(void *data, const char *input) {
 			default: cmd_info (core, "IiEecsSmz"); break;
 			}
 			break;
-		case '?':
+		case '?': // "i?"
 			r_core_cmd_help (core, help_msg_i);
-			goto done;
-		case '*':
+			goto redone;
+		case '*': // "i*"
 			mode = R_CORE_BIN_RADARE;
 			goto done;
-		case 'q':
+		case 'q': // "iq"
 			mode = R_CORE_BIN_SIMPLE;
 			cmd_info_bin (core, va, mode);
 			goto done;
-		case 'j':
+		case 'j': // "ij"
 			mode = R_CORE_BIN_JSON;
 			if (is_array > 1) {
 				mode |= R_CORE_BIN_ARRAY;
@@ -867,5 +909,6 @@ done:
 	if (newline) {
 		r_cons_newline ();
 	}
+redone:
 	return 0;
 }
