@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2009-2017 - pancake */
+/* radare2 - LGPL - Copyright 2009-2018 - pancake */
 
 #include "r_anal.h"
 #include "r_bin.h"
@@ -12,12 +12,13 @@ static const char *help_msg_C[] = {
 	"Usage:", "C[-LCvsdfm*?][*?] [...]", " # Metadata management",
 	"C", "", "list meta info in human friendly form",
 	"C*", "", "list meta info in r2 commands",
-	"C[Chsdmf]", "", "list comments/hidden/strings/data/magic/formatted in human friendly form",
-	"C[Chsdmf]*", "", "list comments/hidden/strings/data/magic/formatted in r2 commands",
+	"C[Cthsdmf]", "", "list comments/types/hidden/strings/data/magic/formatted in human friendly form",
+	"C[Cthsdmf]*", "", "list comments/types/hidden/strings/data/magic/formatted in r2 commands",
 	"C-", " [len] [[@]addr]", "delete metadata at given address range",
 	"CL", "[-][*] [file:line] [addr]", "show or add 'code line' information (bininfo)",
 	"CS", "[-][space]", "manage meta-spaces to filter comments, etc..",
 	"CC", "[?] [-] [comment-text] [@addr]", "add/remove comment",
+	"Ct", "[?] [-] [comment-text] [@addr]", "add/remove type analysis comment",
 	"CC.", "[addr]", "show comment in current address",
 	"CC!", " [@addr]", "edit comment with $EDITOR",
 	"CCa", "[-at]|[at] [text] [@addr]", "add/remove comment at given address",
@@ -27,6 +28,7 @@ static const char *help_msg_C[] = {
 	"Cz", "[@addr]", "add string (see Cs?)",
 	"Ch", "[-] [size] [@addr]", "hide data",
 	"Cd", "[-] [size] [repeat] [@addr]", "hexdump data array (Cd 4 10 == dword [10])",
+	"Cd.", " [@addr]", "show size of data at current address",
 	"Cf", "[?][-] [sz] [0|cnt][fmt] [a0 a1...] [@addr]", "format memory (see pf?)",
 	"CF", "[sz] [fcn-sign..] [@addr]", "function signature",
 	"Cm", "[-] [sz] [fmt..] [@addr]", "magic parse (see pm?)",
@@ -517,7 +519,7 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 	return true;
 }
 
-static int cmd_meta_hsdmf(RCore *core, const char *input) {
+static int cmd_meta_others(RCore *core, const char *input) {
 	int n, type = input[0], subtype;
 	char *t = 0, *p, *p2, name[256];
 	int repeat = 1;
@@ -628,7 +630,7 @@ static int cmd_meta_hsdmf(RCore *core, const char *input) {
 			case 0:  /* temporary legacy workaround */
 				esc_bslash = false;
 			default:
-				esc_str = r_str_escape_latin1 (mi.str, false, esc_bslash);
+				esc_str = r_str_escape_latin1 (mi.str, false, esc_bslash, false);
 			}
 			if (esc_str) {
 				r_cons_printf ("\"%s\"\n", esc_str);
@@ -636,6 +638,8 @@ static int cmd_meta_hsdmf(RCore *core, const char *input) {
 			} else {
 				r_cons_println ("<oom>");
 			}
+		} else if (type == 'd') {
+			r_cons_printf ("%"PFMT64u"\n", mi.size);
 		} else {
 			r_cons_println (mi.str);
 		}
@@ -703,11 +707,11 @@ p = t;
 					char tmp[256] = R_EMPTY;
 					int i, j, name_len = 0;
 					if (input[1] == 'a' || input[1] == '8') {
-						(void)r_core_read_at (core, addr, (ut8*)name, sizeof (name) - 1);
+						(void)r_io_read_at (core->io, addr, (ut8*)name, sizeof (name) - 1);
 						name[sizeof (name) - 1] = '\0';
 						name_len = strlen (name);
 					} else {
-						(void)r_core_read_at (core, addr, (ut8*)tmp, sizeof (tmp) - 3);
+						(void)r_io_read_at (core->io, addr, (ut8*)tmp, sizeof (tmp) - 3);
 						name_len = r_str_nlen_w (tmp, sizeof (tmp) - 3);
 						//handle wide strings
 						for (i = 0, j = 0; i < sizeof (name); i++, j++) {
@@ -852,7 +856,7 @@ void r_comment_vars(RCore *core, const char *input) {
 				comment = heap_comment;
 			}
 		}
-		var = r_anal_var_get_byname (core->anal, fcn, name);
+		var = r_anal_var_get_byname (core->anal, fcn->addr, name);
 		if (var) {
 			idx = var->delta;
 		} else if (!strncmp (name, "0x", 2))  {
@@ -885,7 +889,7 @@ void r_comment_vars(RCore *core, const char *input) {
 		}
 		break;
 	case '-':
-		var = r_anal_var_get_byname (core->anal,fcn, name);
+		var = r_anal_var_get_byname (core->anal, fcn->addr, name);
 		if (var) {
 			idx = var->delta;
 		} else if (!strncmp (name, "0x", 2)) {
@@ -906,7 +910,7 @@ void r_comment_vars(RCore *core, const char *input) {
 		break;
 	case '!': {
 		char *comment;
-		var = r_anal_var_get_byname (core->anal,fcn, name);
+		var = r_anal_var_get_byname (core->anal, fcn->addr, name);
 		if (!var) {
 			eprintf ("cant find variable named `%s`\n",name);
 			break;
@@ -953,7 +957,8 @@ static int cmd_meta(void *data, const char *input) {
 	case 'd': /* "Cd" data */
 	case 'm': /* Cm magic */
 	case 'f': /* Cf formatted */
-		cmd_meta_hsdmf (core, input);
+	case 't': /* Ct type analysis commnets */
+		cmd_meta_others (core, input);
 		break;
 	case '-':
 		if (input[1] != '*') {
