@@ -22,9 +22,40 @@ void r_core_hack_help(const RCore *core) {
 		"NOTE:", "", "those operations are only implemented for x86 and arm atm.", //TODO
 		NULL
 	};
-	r_core_cmd_help(core, help_msg);
+	r_core_cmd_help (core, help_msg);
 }
 
+R_API bool r_core_hack_arm64(RCore *core, const char *op, const RAnalOp *analop) {
+	if (!strcmp (op, "nop")) {
+		r_core_cmdf (core, "wx 1f2003d5");
+	} else if (!strcmp (op, "ret")) {
+		r_core_cmdf (core, "wx c0035fd6t");
+	} else if (!strcmp (op, "trap")) {
+		r_core_cmdf (core, "wx 000020d4");
+	} else if (!strcmp (op, "jz")) {
+		eprintf ("ARM jz hack not supported\n");
+		return false;
+	} else if (!strcmp (op, "jnz")) {
+		eprintf ("ARM jnz hack not supported\n");
+		return false;
+	} else if (!strcmp (op, "nocj")) {
+		eprintf ("ARM jnz hack not supported\n");
+		return false;
+	} else if (!strcmp (op, "recj")) {
+		eprintf ("TODO: use jnz or jz\n");
+		return false;
+	} else if (!strcmp (op, "ret1")) {
+		r_core_cmdf (core, "wa mov x0, 1,,ret");
+	} else if (!strcmp (op, "ret0")) {
+		r_core_cmdf (core, "wa mov x0, 0,,ret");
+	} else if (!strcmp (op, "retn")) {
+		r_core_cmdf (core, "wa mov x0, -1,,ret");
+	} else {
+		eprintf ("Invalid operation '%s'\n", op);
+		return false;
+	}
+	return true;
+}
 R_API bool r_core_hack_arm(RCore *core, const char *op, const RAnalOp *analop) {
 	const int bits = core->assembler->bits;
 	const ut8 *b = core->block;
@@ -149,10 +180,10 @@ R_API bool r_core_hack_x86(RCore *core, const char *op, const RAnalOp *analop) {
 			return false;
 		}
 		for (i = 0; i < size; i++)
-			memcpy(str + (i * 2), "90", 2);
+			memcpy (str + (i * 2), "90", 2);
 		str[size*2] = '\0';
-		r_core_cmdf(core, "wx %s\n", str);
-		free(str);
+		r_core_cmdf (core, "wx %s\n", str);
+		free (str);
 	} else if (!strcmp (op, "trap")) {
 		r_core_cmd0 (core, "wx cc\n");
 	} else if (!strcmp (op, "jz")) {
@@ -179,15 +210,13 @@ R_API bool r_core_hack_x86(RCore *core, const char *op, const RAnalOp *analop) {
 			return false;
 		}
 	} else if (!strcmp (op, "recj")) {
-		int of = *b == 0xf;
-		if (b[of] < 0x80 && b[of] >= 0x70) { // jo, jno, jb, jae, je, jne, jbe, ja, js, jns
-			if (of) {
-				r_core_cmdf (core, "wx 0f%x\n", (b[1]%2)? b[1] - 1: b[1] + 1);
-			} else {
+		int is_near = (*b == 0xf);
+		if (b[0] < 0x80 && b[0] >= 0x70) { // short jmps: jo, jno, jb, jae, je, jne, jbe, ja, js, jns
 				r_core_cmdf (core, "wx %x\n", (b[0]%2)? b[0] - 1: b[0] + 1);
-			}
+		} else if (is_near && b[1] < 0x90 && b[1] >= 0x80) { // near jmps: jo, jno, jb, jae, je, jne, jbe, ja, js, jns
+				r_core_cmdf (core, "wx 0f%x\n", (b[1]%2)? b[1] - 1: b[1] + 1);
 		} else {
-			eprintf ("Invalid opcode\n");
+			eprintf ("Invalid conditional jump opcode\n");
 			return false;
 		}
 	} else if (!strcmp (op, "ret1")) {
@@ -197,7 +226,7 @@ R_API bool r_core_hack_x86(RCore *core, const char *op, const RAnalOp *analop) {
 	} else if (!strcmp (op, "retn")) {
 		r_core_cmd0 (core, "wx c2ffff\n");
 	} else {
-		eprintf ("Invalid operation\n");
+		eprintf ("Invalid operation '%s'\n", op);
 		return false;
 	}
 	return true;
@@ -206,7 +235,7 @@ R_API bool r_core_hack_x86(RCore *core, const char *op, const RAnalOp *analop) {
 R_API int r_core_hack(RCore *core, const char *op) {
 	bool (*hack)(RCore *core, const char *op, const RAnalOp *analop) = NULL;
 	const char *asmarch = r_config_get (core->config, "asm.arch");
-	RAnalOp analop;
+	const int asmbits = core->assembler->bits;
 
 	if (!asmarch) {
 		return false;
@@ -214,12 +243,17 @@ R_API int r_core_hack(RCore *core, const char *op) {
 	if (strstr (asmarch, "x86")) {
 		hack = r_core_hack_x86;
 	} else if (strstr (asmarch, "arm")) {
-		hack = r_core_hack_arm;
+		if (asmbits == 64) {
+			hack = r_core_hack_arm64;
+		} else {
+			hack = r_core_hack_arm;
+		}
 	} else {
 		eprintf ("TODO: write hacks are only for x86\n");
 	}
 	if (hack) {
-		if (!r_anal_op (core->anal, &analop, core->offset, core->block, core->blocksize)) {
+		RAnalOp analop;
+		if (!r_anal_op (core->anal, &analop, core->offset, core->block, core->blocksize, R_ANAL_OP_MASK_BASIC)) {
 			eprintf ("anal op fail\n");
 			return false;
 		}
