@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2010-2017 - pancake, maijin */
+/* radare - LGPL - Copyright 2010-2018 - pancake, maijin */
 
 #include <r_types.h>
 #include <r_list.h>
@@ -15,7 +15,7 @@ static void __section_list_for_projects (RIO *io, RPrint *print) {
 	if (!io || !io->sections || !print || !print->cb_printf) {
 		return;
 	}
-	ls_foreach (io->sections, iter, s) {	
+	ls_foreach (io->sections, iter, s) {
 		print->cb_printf ("[%02d] 0x%08"PFMT64x" %s va=0x%08"PFMT64x
 			" sz=0x%04"PFMT64x" vsz=0x%04"PFMT64x" %s",
 			i, s->paddr, r_str_rwx_i (s->flags), s->vaddr,
@@ -94,7 +94,7 @@ static int projectInit(RCore *core) {
 	return ret;
 }
 
-static bool r_core_is_project(RCore *core, const char *name) {
+R_API bool r_core_is_project(RCore *core, const char *name) {
 	bool ret = false;
 	if (name && *name && *name != '.') {
 		char *path = projectScriptPath (core, name);
@@ -164,27 +164,44 @@ R_API int r_core_project_list(RCore *core, int mode) {
 }
 
 R_API int r_core_project_delete(RCore *core, const char *prjfile) {
-	char *path;
 	if (r_sandbox_enable (0)) {
 		eprintf ("Cannot delete project in sandbox mode\n");
 		return 0;
 	}
-	path = projectScriptPath (core, prjfile);
+	char *path = projectScriptPath (core, prjfile);
 	if (!path) {
 		eprintf ("Invalid project name '%s'\n", prjfile);
 		return false;
 	}
 	if (r_core_is_project (core, prjfile)) {
+		char *prjDir = r_file_dirname (path);
+		if (!prjDir) {
+			eprintf ("Cannot resolve directory\n");
+			free (path);
+			return false;
+		}
 		// rm project file
-		r_file_rm (path);
-		eprintf ("rm %s\n", path);
-		path = r_str_append (path, ".d");
-		if (r_file_is_directory (path)) {
+		if (r_file_exists (path)) {
+			r_file_rm (path);
+			eprintf ("rm %s\n", path);
+		}
+
+		//rm notes.txt file
+		char *notes_txt = r_str_newf ("%s%s%s", prjDir, R_SYS_DIR, "notes.txt");
+		if (r_file_exists (notes_txt)) {
+			r_file_rm (notes_txt);
+			eprintf ("rm %s\n", notes_txt);
+		}
+		free(notes_txt);
+
+		char *rop_d = r_str_newf ("%s%s%s", prjDir, R_SYS_DIR, "rop.d");
+
+		if (r_file_is_directory (rop_d)) {
 			char *f;
 			RListIter *iter;
-			RList *files = r_sys_dir (path);
+			RList *files = r_sys_dir (rop_d);
 			r_list_foreach (files, iter, f) {
-				char *filepath = r_str_append (strdup (path), R_SYS_DIR);
+				char *filepath = r_str_append (strdup (rop_d), R_SYS_DIR);
 				filepath = r_str_append (filepath, f);
 				if (!r_file_is_directory (filepath)) {
 					eprintf ("rm %s\n", filepath);
@@ -192,12 +209,14 @@ R_API int r_core_project_delete(RCore *core, const char *prjfile) {
 				}
 				free (filepath);
 			}
-			r_file_rm (path);
-			eprintf ("rm %s\n", path);
+			r_file_rm (rop_d);
+			eprintf ("rm %s\n", rop_d);
 			r_list_free (files);
 		}
-		// TODO: remove .d directory (BEWARE OF ROOT RIMRAFS!)
-		// TODO: r_file_rmrf (path);
+		free (rop_d);
+		// remove directory only if it's empty
+		r_file_rm (prjDir);
+		free (prjDir);
 	}
 	free (path);
 	return 0;
@@ -783,8 +802,6 @@ R_API bool r_core_project_save(RCore *core, const char *prjName) {
 	}
 	projectInit (core);
 
-	r_anal_project_save (core->anal, prjDir);
-
 	Sdb *rop_db = sdb_ns (core->sdb, "rop", false);
 	if (rop_db) {
 		/* set filepath for all the rop sub-dbs */
@@ -802,12 +819,12 @@ R_API bool r_core_project_save(RCore *core, const char *prjName) {
 	}
 	r_config_set (core->config, "prj.name", prjName);
 	if (r_config_get_i (core->config, "prj.simple")) {
-		if (!simpleProjectSaveScript (core, scriptPath, R_CORE_PRJ_ALL ^ R_CORE_PRJ_XREFS)) {
+		if (!simpleProjectSaveScript (core, scriptPath, R_CORE_PRJ_ALL)) {
 			eprintf ("Cannot open '%s' for writing\n", prjName);
 			ret = false;
 		}
 	} else {
-		if (!projectSaveScript (core, scriptPath, R_CORE_PRJ_ALL ^ R_CORE_PRJ_XREFS)) {
+		if (!projectSaveScript (core, scriptPath, R_CORE_PRJ_ALL)) {
 			eprintf ("Cannot open '%s' for writing\n", prjName);
 			ret = false;
 		}
@@ -864,7 +881,7 @@ R_API bool r_core_project_save(RCore *core, const char *prjName) {
 	if (scr_null) {
 		r_config_set_i (core->config, "scr.null", true);
 	}
-	if (!ret) {
+	if (!ret && oldPrjName) {
 		// reset prj.name on fail
 		r_config_set (core->config, "prj.name", oldPrjName);
 	}
@@ -882,62 +899,11 @@ R_API char *r_core_project_notes_file(RCore *core, const char *prjName) {
 	return notes_txt;
 }
 
-#define DB core->anal->sdb_xrefs
-
-static bool projectLoadXrefs(RCore *core, const char *prjName) {
-	char *path, *db;
-
-	if (!prjName || !*prjName) {
-		return false;
-	}
-	const char *prjdir = r_config_get (core->config, "dir.projects");
-
-	if (prjName[0] == R_SYS_DIR[0]) {
-		db = r_str_newf ("%s", prjName);
-		if (!db) {
-			return false;
-		}
-		path = strdup (db);
-	} else {
-		db = r_str_newf ("%s" R_SYS_DIR "%s", prjdir, prjName);
-		if (!db) {
-			return false;
-		}
-		path = r_file_abspath (db);
-	}
-
-	if (!path) {
-		free (db);
-		return false;
-	}
-	if (!r_file_is_directory (db)) {
-		db = r_str_append (db, ".d");
-	}
-
-	if (!sdb_ns_unset (core->anal->sdb, NULL, DB)) {
-		sdb_free (DB);
-	}
-	const char *xrefs_path = r_file_fexists ("%s" R_SYS_DIR "xrefs.sdb", path)
-	                         ? "xrefs.sdb": "xrefs";
-	DB = sdb_new (path, xrefs_path, 0);
-	if (!DB) {
-		free (db);
-		free (path);
-		return false;
-	}
-	sdb_ns_set (core->anal->sdb, "xrefs", DB);
-	free (path);
-
-	free (db);
-	return true;
-}
-
 R_API bool r_core_project_load(RCore *core, const char *prjName, const char *rcpath) {
 	const bool cfg_fortunes = r_config_get_i (core->config, "cfg.fortunes");
 	const bool scr_interactive = r_config_get_i (core->config, "scr.interactive");
 	const bool scr_prompt = r_config_get_i (core->config, "scr.prompt");
 	(void) projectLoadRop (core, prjName);
-	(void) projectLoadXrefs (core, prjName);
 	bool ret = r_core_cmd_file (core, rcpath);
 	r_config_set_i (core->config, "cfg.fortunes", cfg_fortunes);
 	r_config_set_i (core->config, "scr.interactive", scr_interactive);
@@ -945,4 +911,3 @@ R_API bool r_core_project_load(RCore *core, const char *prjName, const char *rcp
 	r_config_bump (core->config, "asm.arch");
 	return ret;
 }
-
