@@ -28,7 +28,7 @@ typedef struct {
 static char *prompt(const char *str, const char *txt) {
 	char cmd[1024];
 	char *res = NULL;
-	char *oprompt = strdup (r_cons_singleton()->line->prompt);
+	char *oprompt = strdup (r_cons_singleton ()->line->prompt);
 	r_cons_show_cursor (true);
 	if (txt && *txt) {
 		free (r_cons_singleton ()->line->contents);
@@ -55,15 +55,16 @@ static char *prompt(const char *str, const char *txt) {
 
 static inline char *getformat (RCoreVisualTypes *vt, const char *k) {
 	return sdb_get (vt->core->anal->sdb_types,
-		sdb_fmt (0, "type.%s", k), 0);
+		sdb_fmt ("type.%s", k), 0);
 }
 
-static char *colorize_asm_string(RCore *core, const char *buf_asm, int optype) {
+static char *colorize_asm_string(RCore *core, const char *buf_asm, int optype, ut64 addr) {
 	char *tmp, *spacer = NULL;
 	char *source = (char*)buf_asm;
 	bool use_color = core->print->flags & R_PRINT_FLAGS_COLOR;
 	const char *color_num = core->cons->pal.num;
 	const char *color_reg = core->cons->pal.reg;
+	RAnalFunction* fcn = r_anal_get_fcn_in (core->anal, addr, R_ANAL_FCN_TYPE_NULL);
 
 	if (!use_color) {
 		return strdup (source);
@@ -71,27 +72,20 @@ static char *colorize_asm_string(RCore *core, const char *buf_asm, int optype) {
 	// workaround dummy colorizer in case of paired commands (tms320 & friends)
 	spacer = strstr (source, "||");
 	if (spacer) {
-		char *scol1, *s1 = r_str_ndup (source, spacer - source);
-		char *scol2, *s2 = strdup (spacer + 2);
-		scol1 = r_print_colorize_opcode (core->print, s1, color_reg, color_num, false);
-		free (s1);
-		scol2 = r_print_colorize_opcode (core->print, s2, color_reg, color_num, false);
-		free (s2);
-		if (!scol1) {
-			scol1 = strdup ("");
-		}
-		if (!scol2) {
-			scol2 = strdup ("");
-		}
-		source = malloc (strlen(scol1) + strlen(scol2) + 2 + 1); // reuse source variable
-		sprintf (source, "%s||%s", scol1, scol2);
+		char *s1 = r_str_ndup (source, spacer - source);
+		char *s2 = strdup (spacer + 2);
+		char *scol1 = r_print_colorize_opcode (core->print, s1, color_reg, color_num, false, fcn ? fcn->addr : 0);
+		char *scol2 = r_print_colorize_opcode (core->print, s2, color_reg, color_num, false, fcn ? fcn->addr : 0);
+		char *source = r_str_newf ("%s||%s", r_str_get2 (scol1), r_str_get2 (scol2));
 		free (scol1);
 		free (scol2);
+		free (s1);
+		free (s2);
 		return source;
 	}
-	char *res = strdup("");
+	char *res = strdup ("");
 	res = r_str_append (res, r_print_color_op_type (core->print, optype));
-	tmp = r_print_colorize_opcode (core->print, source, color_reg, color_num, false);
+	tmp = r_print_colorize_opcode (core->print, source, color_reg, color_num, false, fcn ? fcn->addr : 0);
 	res = r_str_append (res, tmp);
 	free (tmp);
 	return res;
@@ -139,12 +133,13 @@ R_API bool r_core_visual_esil(RCore *core) {
 	RAsmOp asmop;
 	RAnalOp analop;
 	ut8 buf[sizeof (ut64)];
+	unsigned int addrsize = r_config_get_i (core->config, "esil.addr.size");
 
 	if (core->blocksize < sizeof (ut64)) {
 		return false;
 	}
 	memcpy (buf, core->block, sizeof (ut64));
-	esil = r_anal_esil_new (20, 0);
+	esil = r_anal_esil_new (20, 0, addrsize);
 	esil->anal = core->anal;
 	r_anal_esil_set_pc (esil, core->offset);
 	for (;;) {
@@ -152,7 +147,7 @@ R_API bool r_core_visual_esil(RCore *core) {
 		// bool use_color = core->print->flags & R_PRINT_FLAGS_COLOR;
 		(void) r_asm_disassemble (core->assembler, &asmop, buf, sizeof (ut64));
 		analop.type = -1;
-		(void)r_anal_op (core->anal, &analop, core->offset, buf, sizeof (ut64));
+		(void)r_anal_op (core->anal, &analop, core->offset, buf, sizeof (ut64), R_ANAL_OP_MASK_ESIL);
 		analopType = analop.type & R_ANAL_OP_TYPE_MASK;
 		r_cons_printf ("r2's esil debugger:\n\n");
 		r_cons_printf ("pos: %d\n", x);
@@ -162,7 +157,7 @@ R_API bool r_core_visual_esil(RCore *core) {
 			free (res);
 		}
 		{
-			char *op = colorize_asm_string (core, asmop.buf_asm, analopType);
+			char *op = colorize_asm_string (core, asmop.buf_asm, analopType, core->offset);
 			r_cons_printf (Color_RESET"asm: %s\n"Color_RESET, op);
 			free (op);
 		}
@@ -256,7 +251,7 @@ R_API bool r_core_visual_esil(RCore *core) {
 			if (r_cons_fgets (cmd, sizeof (cmd)-1, 0, NULL) < 0) {
 				cmd[0] = '\0';
 			}
-			r_core_cmd (core, cmd, 1);
+			r_core_cmd_task_sync (core, cmd, 1);
 			r_cons_set_raw (1);
 			r_cons_show_cursor (false);
 			if (cmd[0]) {
@@ -291,7 +286,7 @@ static bool edit_bits (RCore *core) {
 		bool use_color = core->print->flags & R_PRINT_FLAGS_COLOR;
 		(void) r_asm_disassemble (core->assembler, &asmop, buf, sizeof (ut64));
 		analop.type = -1;
-		(void)r_anal_op (core->anal, &analop, core->offset, buf, sizeof (ut64));
+		(void)r_anal_op (core->anal, &analop, core->offset, buf, sizeof (ut64), R_ANAL_OP_MASK_ESIL);
 		analopType = analop.type & R_ANAL_OP_TYPE_MASK;
 		r_cons_printf ("r2's bit editor:\n\n");
 		{
@@ -305,7 +300,7 @@ static bool edit_bits (RCore *core) {
 			r_cons_printf ("shift: >> %d << %d\n", word, (asmop.size * 8) - word - 1);
 		}
 		{
-			char *op = colorize_asm_string (core, asmop.buf_asm, analopType);
+			char *op = colorize_asm_string (core, asmop.buf_asm, analopType, core->offset);
 			r_cons_printf (Color_RESET"asm: %s\n"Color_RESET, op);
 			free (op);
 		}
@@ -486,6 +481,7 @@ static int sdbforcb (void *p, const char *k, const char *v) {
 	const char *pre = " ";
 	RCoreVisualTypes *vt = (RCoreVisualTypes*)p;
 	bool use_color = vt->core->print->flags & R_PRINT_FLAGS_COLOR;
+	char *color_sel = vt->core->cons->pal.prompt;
 	if (vt->optword) {
 		if (!strcmp (vt->type, "struct")) {
 			char *s = r_str_newf ("struct.%s.", vt->optword);
@@ -499,7 +495,7 @@ static int sdbforcb (void *p, const char *k, const char *v) {
 					pre = ">";
 				}
 				if (use_color && *pre=='>')
-					r_cons_printf (Color_YELLOW" %s %s  %s\n"
+					r_cons_printf ("%s %s %s  %s\n", color_sel,
 						Color_RESET, pre, k+strlen (s), v);
 				else
 					r_cons_printf (" %s %s  %s\n",
@@ -520,7 +516,7 @@ static int sdbforcb (void *p, const char *k, const char *v) {
 						pre = ">";
 					}
 					if (use_color && *pre=='>') {
-						r_cons_printf (Color_YELLOW" %s %s  %s\n"
+						r_cons_printf ("%s %s %s  %s\n", color_sel,
 							Color_RESET, pre, k, v);
 					} else {
 						r_cons_printf (" %s %s  %s\n",
@@ -542,8 +538,8 @@ static int sdbforcb (void *p, const char *k, const char *v) {
 				pre = ">";
 			}
 			if (use_color && *pre=='>') {
-				r_cons_printf (Color_YELLOW" %s pf %3s   %s\n"
-					Color_RESET,pre, fmt, k);
+				r_cons_printf ("%s %s pf %3s   %s\n"Color_RESET,
+					color_sel, pre, fmt, k);
 			} else {
 				r_cons_printf (" %s pf %3s   %s\n",
 					pre, fmt, k);
@@ -558,7 +554,7 @@ static int sdbforcb (void *p, const char *k, const char *v) {
 				pre = ">";
 			}
 			if (use_color && *pre == '>') {
-				r_cons_printf (Color_YELLOW" %s %s\n"Color_RESET,
+				r_cons_printf ("%s %s %s\n"Color_RESET, color_sel,
 					(vt->t_idx == vt->t_ctr)?
 					">":" ", k);
 			} else {
@@ -602,14 +598,17 @@ R_API int r_core_visual_types(RCore *core) {
 	for (;;) {
 		r_cons_clear00 ();
 		for (i = 0; opts[i]; i++) {
-			const char *fmt = use_color
-				? (h_opt == i)
-					? Color_BGREEN"[%s] "Color_RESET
-					: Color_GREEN" %s  "Color_RESET
-				: (h_opt == i)
-					? "[%s] "
-					: " %s  ";
-			r_cons_printf (fmt, opts[i]);
+			if (use_color) {
+				if (h_opt == i) {
+					r_cons_printf ("%s[%s]%s ", core->cons->pal.call, 
+						opts[i], Color_RESET);
+				} else {
+					r_cons_printf ("%s%s%s  ", core->cons->pal.other, 
+						opts[i], Color_RESET);
+				}
+			} else {
+				r_cons_printf (h_opt == i ? "[%s] " : " %s  ", opts[i]);
+			}
 		}
 		r_cons_newline ();
 		if (optword) {
@@ -1198,6 +1197,7 @@ R_API int r_core_visual_trackflags(RCore *core) {
 		menu = 1;
 	}
 	for (;;) {
+		bool hasColor = r_config_get_i (core->config, "scr.color");
 		r_cons_clear00 ();
 
 		if (menu) {
@@ -1216,9 +1216,15 @@ R_API int r_core_visual_trackflags(RCore *core) {
 					hit = 1;
 				}
 				if ((i>=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
+					bool cur = option == i;
+					if (cur && hasColor) {
+						r_cons_printf (Color_INVERT);
+					}
 					r_cons_printf (" %c  %03d 0x%08"PFMT64x" %4"PFMT64d" %s\n",
-						(option==i)?'>':' ',
-						i, flag->offset, flag->size, flag->name);
+							cur?'>':' ', i, flag->offset, flag->size, flag->name);
+					if (cur && hasColor) {
+						r_cons_printf (Color_RESET);
+					}
 					j++;
 				}
 				i++;
@@ -1255,7 +1261,7 @@ R_API int r_core_visual_trackflags(RCore *core) {
 					}
 					if ((i >= option - delta) && ((i < option + delta)|| \
 							((option < delta) && (i < (delta << 1))))) {
-						r_cons_printf(" %c %02d %c %s\n",
+						r_cons_printf (" %c %02d %c %s\n",
 							(option==i)?'>':' ', j,
 							(i==core->flags->space_idx)?'*':' ',
 							core->flags->spaces[i]);
@@ -1303,10 +1309,13 @@ R_API int r_core_visual_trackflags(RCore *core) {
 		case 'b': // back
 		case 'Q':
 		case 'q':
-			if (menu <= 0) return true;
+			if (menu <= 0) {
+				return true;
+			}
 			menu--;
 			option = _option;
 			if (menu == 0) {
+				r_flag_space_set (core->flags, NULL);
 				// if no flagspaces, just quit
 				for (j=i=0;i<R_FLAG_SPACES_MAX;i++) {
 					if (core->flags->spaces[i]) {
@@ -1443,16 +1452,18 @@ R_API int r_core_visual_trackflags(RCore *core) {
 		case ':':
 			r_cons_show_cursor (true);
 			r_cons_set_raw (0);
-			cmd[0]='\0';
+			*cmd = 0;
 			r_line_set_prompt (":> ");
-			if (r_cons_fgets (cmd, sizeof (cmd)-1, 0, NULL) <0)
-				cmd[0]='\0';
-			//line[strlen(line)-1]='\0';
-			r_core_cmd (core, cmd, 1);
+			if (r_cons_fgets (cmd, sizeof (cmd) - 1, 0, NULL) <0) {
+				*cmd = 0;
+			}
+			cmd[sizeof (cmd) - 1] = 0;
+			r_core_cmd_task_sync (core, cmd, 1);
 			r_cons_set_raw (1);
 			r_cons_show_cursor (false);
-			if (cmd[0])
+			if (*cmd) {
 				r_cons_any_key (NULL);
+			}
 			//cons_gotoxy(0,0);
 			r_cons_clear ();
 			continue;
@@ -1460,57 +1471,72 @@ R_API int r_core_visual_trackflags(RCore *core) {
 	}
 	return true;
 }
+static bool meta_deserialize(RAnalMetaItem *it, const char *k, const char *v) {
+	if (strlen (k) < 8) {
+		return false;
+	}
+	if (memcmp (k + 6, ".0x", 3)) {
+		return false;
+	}
+	return r_meta_deserialize_val (it, k[5], sdb_atoi (k + 7), v);
+}
+
+static int meta_enumerate_cb(void *user, const char *k, const char *v) {
+	RAnalMetaUserItem *ui = user;
+	RList *list = ui->user;
+	RAnalMetaItem *it = R_NEW0 (RAnalMetaItem);
+	if (!it) {
+		return 0;
+	}
+	if (!meta_deserialize (it, k, v)) {
+		free (it);
+		goto beach;
+	}
+	if (!it->str) {
+		free (it);
+		goto beach;
+	}
+	r_list_append (list, it);
+beach:
+	return 1;
+}
 
 R_API int r_core_visual_comments (RCore *core) {
-#undef DB
-#define DB core->anal->sdb_meta
-	const char *val, *comma = NULL;
-	char *list = sdb_get (DB, "meta.C", 0);
-	char *str, *next, *cur = list;
-	char key[128], cmd[512], *p = NULL;
-	int i, ch, option = 0, delta = 7;
+	char *str;
+	char cmd[512], *p = NULL;
+	int ch, option = 0;
 	int format = 0, found = 0;
 	ut64 addr, from = 0, size = 0;
 
 	for (;;) {
 		r_cons_clear00 ();
 		r_cons_strcat ("Comments:\n");
-		i = 0;
 		found = 0;
-		if (list) {
-			for (i=0; ;i++) {
-				cur = sdb_anext (cur, &next);
-				addr = sdb_atoi (cur);
-				snprintf (key, sizeof (key)-1, "meta.C.0x%08"PFMT64x, addr);
-				val = sdb_const_get (DB, key, 0);
-				if (val)
-					comma = strchr (val, ',');
-				if (comma) {
-					str = (char *)sdb_decode (comma+1, 0);
-					if ((i>=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
-						r_str_sanitize (str);
-						if (option==i) {
-							found = 1;
-							from = addr;
-							size = 1; // XXX: remove this thing size for comments is useless d->size;
-							free (p);
-							p = str;
-							r_cons_printf ("  >  %s\n", str);
-						} else {
-							r_cons_printf ("     %s\n", str);
-							free (str);
-						}
-					} else free (str);
-				}
-				if (!next) {
-					break;
-				}
-				cur = next;
+		RList *items = r_list_newf (free);
+		RAnalMetaItem *item;
+		RListIter *iter;
+		r_meta_list_cb (core->anal, R_META_TYPE_COMMENT, 0, meta_enumerate_cb, items, UT64_MAX);
+		int i = 0;
+		r_list_foreach (items, iter, item) {
+			str = item->str;
+			addr = item->from;
+			if (option==i) {
+				found = 1;
+				from = addr;
+				size = 1; // XXX: remove this thing size for comments is useless d->size;
+				free (p);
+				p = str;
+				r_cons_printf ("  >  %s\n", str);
+			} else {
+				r_cons_printf ("     %s\n", str);
+				free (str);
 			}
+			i ++;
+			found = true;
 		}
-
 		if (!found) {
 			if (--option < 0) {
+				r_cons_any_key ("No comments");
 				break;
 			}
 			continue;
@@ -1674,7 +1700,7 @@ R_API void r_core_visual_config(RCore *core) {
 				if (show) {
 					if (option == i) hit = 1;
 					if ( (i >=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
-						r_cons_printf(" %c  %s\n", (option==i)?'>':' ', old);
+						r_cons_printf (" %c  %s\n", (option == i)?'>':' ', old);
 						j++;
 					}
 					i++;
@@ -1788,7 +1814,7 @@ R_API void r_core_visual_config(RCore *core) {
 			break;
 		case ':':
 			r_cons_show_cursor (true);
-			r_cons_set_raw(0);
+			r_cons_set_raw (0);
 			 {
 				char *cmd = prompt (":> ", NULL);
 				r_core_cmd (core, cmd, 1);
@@ -1845,7 +1871,7 @@ R_API void r_core_visual_mounts(RCore *core) {
 			} else r_cons_printf ("Cannot read partition\n");
 		} else if (mode == 1) {
 			r_cons_printf ("Types:\n\n");
-			for(i=0;;i++) {
+			for (i=0;;i++) {
 				n = r_fs_partition_type_get (i);
 				if (!n) break;
 				r_cons_printf ("%s%s\n", (i==partition)?" > ":"   ", n);
@@ -2172,11 +2198,7 @@ static void variable_rename (RCore *core, ut64 addr, int vindex, const char *nam
 	ut64 a_tmp = core->offset;
 	int i = 0;
 	RListIter *iter;
-	// arguments.
-	RList* list2 = r_anal_var_list (core->anal, fcn, true);
-	// variables.
-	RList* list = r_anal_var_list (core->anal, fcn, false);
-	r_list_join (list, list2);
+	RList *list = r_anal_var_all_list (core->anal, fcn);
 	RAnalVar* var;
 
 	r_list_foreach (list, iter, var) {
@@ -2189,7 +2211,6 @@ static void variable_rename (RCore *core, ut64 addr, int vindex, const char *nam
 		++i;
 	}
 	r_list_free (list);
-	r_list_free (list2);
 }
 
 // In visual mode, display function list
@@ -2204,6 +2225,9 @@ static ut64 var_functions_show(RCore *core, int idx, int show) {
 	// Adjust the windows size automaticaly
 	(void)r_cons_get_size (&window);
 	window -= 8; // Size of printed things
+	bool color = r_config_get_i (core->config, "scr.color");
+	const char *color_addr = core->cons->pal.offset;
+	const char *color_fcn = core->cons->pal.fname;
 
 	r_list_foreach (core->anal->fcns, iter, fcn) {
 		if (i >= wdelta) {
@@ -2214,11 +2238,20 @@ static ut64 var_functions_show(RCore *core, int idx, int show) {
 			if (idx == i) {
 				addr = fcn->addr;
 			}
-			if (show)
-				r_cons_printf ("%c%c 0x%08"PFMT64x" %4d %s\n",
-					(seek == fcn->addr)?'>':' ',
-					(idx==i)?'*':' ',
-					fcn->addr, r_anal_fcn_realsize (fcn), fcn->name);
+			if (show) {
+				if (color) {
+					r_cons_printf ("%c%c %s0x%08"PFMT64x"" Color_RESET" %4d %s%s"Color_RESET"\n",
+							(seek == fcn->addr)?'>':' ',
+							(idx==i)?'*':' ',
+							color_addr, fcn->addr, r_anal_fcn_realsize (fcn),
+							color_fcn, fcn->name);
+				} else {
+					r_cons_printf ("%c%c 0x%08"PFMT64x" %4d %s\n",
+							(seek == fcn->addr)?'>':' ',
+							(idx==i)?'*':' ',
+							fcn->addr, r_anal_fcn_realsize (fcn), fcn->name);
+				}
+			}
 		}
 		i++;
 	}
@@ -2233,11 +2266,7 @@ static ut64 var_variables_show(RCore* core, int idx, int *vindex, int show) {
 	int window;
 	int wdelta = (idx > 5) ? idx - 5 : 0;
 	RListIter *iter;
-	// arguments.
-	RList* list2 = r_anal_var_list (core->anal, fcn, true);
-	// variables.
-	RList* list = r_anal_var_list (core->anal, fcn, false);
-	r_list_join (list, list2);
+	RList *list = r_anal_var_all_list (core->anal, fcn);
 	RAnalVar* var;
 	// Adjust the window size automatically.
 	(void)r_cons_get_size (&window);
@@ -2259,7 +2288,7 @@ static ut64 var_variables_show(RCore* core, int idx, int *vindex, int show) {
 			}
 			if (show) {
 				r_cons_printf ("%s%s %s %s @ %s%s0x%x\n",
-						i == *vindex ? "* ":"",
+						i == *vindex ? "* ":"  ",
 						var->kind=='v'?"var":"arg",
 						var->type, var->name,
 						core->anal->reg->name[R_REG_NAME_BP],
@@ -2270,16 +2299,20 @@ static ut64 var_variables_show(RCore* core, int idx, int *vindex, int show) {
 		++i;
 	}
 	r_list_free (list);
-	r_list_free (list2);
 	return addr;
 }
 
 static int level = 0;
 static ut64 addr = 0;
+static st64 delta = 0;
 static int option = 0;
 static int variable_option = 0;
 static int printMode = 0;
-#define lastPrintMode 5
+static bool selectPanel = false;
+#define lastPrintMode 6
+static const char *cmd, *printCmds[lastPrintMode] = {
+	"pdf", "pd $r", "afi", "pdsf", "pdc", "pdr"
+};
 
 static void r_core_visual_anal_refresh_column (RCore *core, int colpos) {
 	const ut64 addr = (level != 0 && level != 1)
@@ -2289,15 +2322,12 @@ static void r_core_visual_anal_refresh_column (RCore *core, int colpos) {
 	int h, w = r_cons_get_size (&h);
 	// int sz = (fcn)? R_MIN (r_anal_fcn_size (fcn), h * 15) : 16; // max instr is 15 bytes.
 
-	const char *cmd, *printCmds[lastPrintMode] = {
-		"pdf", "afi", "pds", "pdc", "pdr"
-	};
 	if (printMode > 0 && printMode < lastPrintMode) {
 		cmd = printCmds[printMode];
 	} else {
 		cmd = printCmds[printMode = 0];
 	}
-	char *cmdf = r_str_newf ("%s @ 0x%"PFMT64x, cmd, addr);
+	char *cmdf = r_str_newf ("%s @ 0x%"PFMT64x, cmd, addr + delta);
 	if (!cmdf) {
 		return;
 	}
@@ -2305,7 +2335,7 @@ static void r_core_visual_anal_refresh_column (RCore *core, int colpos) {
 	if (output) {
 		// 'h - 2' because we have two new lines in r_cons_printf
 		char *out = r_str_ansi_crop (output, 0, 0, w - colpos, h - 2);
-		r_cons_printf ("Visual code review (%s)\n%s\n", cmd, out);
+		r_cons_printf ("\n%s\n", out);
 		free (out);
 		R_FREE (output);
 	}
@@ -2313,12 +2343,14 @@ static void r_core_visual_anal_refresh_column (RCore *core, int colpos) {
 }
 
 static ut64 r_core_visual_anal_refresh (RCore *core) {
-	ut64 addr;
-	char old[1024];
-	int cols = r_cons_get_size (NULL);
+	eprintf("r_core_visual_anal_refresh\n");
 	if (!core) {
 		return 0LL;
 	}
+	ut64 addr;
+	char old[1024];
+	bool color = r_config_get_i (core->config, "scr.color");
+	int cols = r_cons_get_size (NULL);
 	old[0] = '\0';
 	addr = core->offset;
 	cols -= 50;
@@ -2327,7 +2359,6 @@ static ut64 r_core_visual_anal_refresh (RCore *core) {
 	}
 
 	r_cons_clear00 ();
-	r_cons_flush ();
 	r_core_visual_anal_refresh_column (core, cols);
 	if (cols > 30) {
 		r_cons_column (cols);
@@ -2335,10 +2366,21 @@ static ut64 r_core_visual_anal_refresh (RCore *core) {
 	switch (level) {
 	// Show functions list help in visual mode
 	case 0:
-		r_cons_printf ("-[ functions ]---------------- \n"
-			"(a) add     (x)xrefs     (q)quit \n"
-			"(r) rename  (c)calls     (g)go \n"
-			"(d) delete  (v)variables (?)help \n");
+		if (color) {
+			r_cons_strcat (core->cons->pal.prompt);
+		}
+		if (selectPanel) {
+			r_cons_printf ("-- functions -----------------[ %s ]-->>", printCmds[printMode]);
+		} else {
+			r_cons_printf ("-[ functions ]----------------- %s ---", printCmds[printMode]);
+		}
+		if (color) {
+			r_cons_strcat ("\n" Color_RESET);
+		}
+		r_cons_printf (
+			"(a) add     (x) xrefs  (q) quit  (jk) next/prev\n"
+			"(r) rename  (c) calls  (g) go    (tab) column\n"
+			"(d) delete  (v) vars   (?) help  (:)  enter cmd\n");
 		addr = var_functions_show (core, option, 1);
 		break;
 	case 1:
@@ -2354,7 +2396,7 @@ static ut64 r_core_visual_anal_refresh (RCore *core) {
 		r_cons_printf ("Press 'q' to quit call refs\n");
 		r_cons_printf ("-[ calls ]----------------------- 0x%08"PFMT64x" (TODO)\n", addr);
 		// TODO: filter only the callrefs. but we cant grep here
-		sprintf(old, "afi @ 0x%08"PFMT64x, addr);
+		sprintf (old, "afi @ 0x%08"PFMT64x, addr);
 		r_core_cmd0 (core, old);
 		break;
 	case 3:
@@ -2369,20 +2411,29 @@ static ut64 r_core_visual_anal_refresh (RCore *core) {
 	return addr;
 }
 
+static void r_core_visual_anal_refresh_oneshot (RCore *core) {
+	r_core_task_enqueue_oneshot (core, (RCoreTaskOneShot) r_core_visual_anal_refresh, core);
+}
+
 /* Like emenu but for real */
 R_API void r_core_visual_anal(RCore *core) {
 	char old[218];
-	int ch, _option = 0;
-	int nfcns = r_list_length (core->anal->fcns);
+	int nfcns, ch, _option = 0;
+
 	RConsEvent olde = core->cons->event_resize;
-	core->cons->event_resize = (RConsEvent) r_core_visual_anal_refresh;
-	core->cons->event_data = (void *) core;
+	void *olde_user = core->cons->event_data;
+
+	core->cons->event_resize = NULL; // avoid running old event with new data
+	core->cons->event_data = core;
+	core->cons->event_resize = (RConsEvent) r_core_visual_anal_refresh_oneshot;
+
 	level = 0;
 	addr = core->offset;
 
 	int asmbytes = r_config_get_i (core->config, "asm.bytes");
 	r_config_set_i (core->config, "asm.bytes", 0);
 	for (;;) {
+		nfcns = r_list_length (core->anal->fcns);
 		addr = r_core_visual_anal_refresh (core);
 		ch = r_cons_readchar ();
 		if (ch == 4 || ch == -1) {
@@ -2395,22 +2446,36 @@ R_API void r_core_visual_anal(RCore *core) {
 		ch = r_cons_arrow_to_hjkl (ch); // get ESC+char, return 'hjkl' char
 		switch (ch) {
 		case '?':
-			r_cons_clear ();
+			r_cons_clear00 ();
 			r_cons_printf (
-				"Usage: Vv [\n"
+				"Usage: vv\n"
 				"Actions supported:\n"
 				" functions: Add, Modify, Delete, Xrefs Calls Vars\n"
 				" variables: Add, Modify, Delete\n"
-				"Moving:\n"
-				" j,k     select next/prev item\n"
-				" J,K     scroll next/prev page\n"
+				"Keys:\n"
+				" j,k     select next/prev item or scroll if tab pressed\n"
+				" J,K     scroll next/prev page \"\"\n"
 				" h,q     go back, quit\n"
 				" p,P     switch next/prev print mode\n"
-				" l,ret   enter, function\n"
+				" v       view selected function arguments and variables\n"
+				" x       see xrefs to the selected function\n"
+				" tab     toggle disasm column selection (to scroll in code)\n"
+				" !       run 'afls' to sort all functions by address\n"
+				" .       seek to current function address\n"
+				" :       run r2 commands\n"
+				" _       hud mode. same as: s $(afl~...)\n"
+				" enter   enter function view (variables), xrefs\n"
 			);
 			r_cons_flush ();
 			r_cons_any_key (NULL);
 			break;
+		case 9:
+			selectPanel = !selectPanel;
+			if (!selectPanel) {
+				delta = 0;
+				printMode = 0;
+			}
+			break;
 		case ':':
 			r_core_visual_prompt (core);
 			r_cons_any_key (NULL);
@@ -2474,6 +2539,12 @@ R_API void r_core_visual_anal(RCore *core) {
 				r_cons_show_cursor (false);
 			}
 			break;
+		case '.':
+			delta = 0;
+			break;
+		case 'R':
+			r_core_cmd0 (core, "ecn");
+			break;
 		case 'p':
 			printMode ++;
 			break;
@@ -2487,43 +2558,58 @@ R_API void r_core_visual_anal(RCore *core) {
 		case 'd':
 			switch (level) {
 			case 0:
-				eprintf ("TODO\n");
-				//data_del(addr, DATA_FUN, 0);
-				// XXX correcly remove all the data contained inside the size of the function
-				//flag_remove_at(addr);
+				r_core_cmdf (core, "af-0x%"PFMT64x, addr);
 				break;
 			}
 			break;
 		case 'x': level = 3; break;
 		case 'c': level = 2; break;
 		case 'v': level = 1; variable_option = 0; break;
+		case '_':
+			r_core_cmd0 (core, "s $(afl~...)");
+			break;
 		case 'j':
-			{
+			if (selectPanel) {
+				printMode = 1;
+				delta += 16;
+			} else {
+				delta = 0;
 				switch (level) {
-					case 1:
-						variable_option++;
-						break;
-					default:
-						option++;
-						if (option >= nfcns) --option;
+				case 1:
+					variable_option++;
+					break;
+				default:
+					option++;
+					if (option >= nfcns) --option;
+					break;
 				}
 			}
 			break;
+		case '!':
+			r_core_cmd0 (core, "afls");
+			break;
 		case 'k':
-			{
+			if (selectPanel) {
+				printMode = 1;
+				delta -= 16;
+			} else {
+				delta = 0;
 				switch (level) {
-					case 1:
-						variable_option = (variable_option<=0)? 0: variable_option-1;
-						break;
-					default:
-						option = (option<=0)? 0: option-1;
-						break;
+				case 1:
+					variable_option = (variable_option<=0)? 0: variable_option-1;
+					break;
+				default:
+					option = (option<=0)? 0: option-1;
+					break;
 				}
 			}
 
 			break;
 		case 'J':
-			{
+			if (selectPanel) {
+				printMode = 1;
+				delta += 40;
+			} else {
 				int rows = 0;
 				r_cons_get_size (&rows);
 				option += (rows - 5);
@@ -2533,7 +2619,10 @@ R_API void r_core_visual_anal(RCore *core) {
 			}
 			break;
 		case 'K':
-			{
+			if (selectPanel) {
+				printMode = 1;
+				delta -= 40;
+			} else {
 				int rows = 0;
 				r_cons_get_size (&rows);
 				option -= (rows - 5);
@@ -2546,6 +2635,12 @@ R_API void r_core_visual_anal(RCore *core) {
 			r_core_seek (core, addr, SEEK_SET);
 			goto beach;
 		case ' ':
+		case '\r':
+		case '\n':
+			level = 0;
+			r_core_seek (core, addr, SEEK_SET);
+			goto beach;
+			break;
 		case 'l':
 			level = 1;
 			_option = option;
@@ -2565,6 +2660,8 @@ R_API void r_core_visual_anal(RCore *core) {
 		}
 	}
 beach:
+	core->cons->event_resize = NULL; // avoid running old event with new data
+	core->cons->event_data = olde_user;
 	core->cons->event_resize = olde;
 	level = 0;
 	r_config_set_i (core->config, "asm.bytes", asmbytes);
@@ -2575,21 +2672,19 @@ R_API void r_core_seek_next(RCore *core, const char *type) {
 	ut64 next = UT64_MAX;
 	if (strstr (type, "opc")) {
 		RAnalOp aop;
-		if (r_anal_op (core->anal, &aop, core->offset, core->block, core->blocksize)) {
+		if (r_anal_op (core->anal, &aop, core->offset, core->block, core->blocksize, R_ANAL_OP_MASK_BASIC)) {
 			next = core->offset + aop.size;
 		} else {
 			eprintf ("Invalid opcode\n");
 		}
-	} else
-	if (strstr (type, "fun")) {
+	} else if (strstr (type, "fun")) {
 		RAnalFunction *fcni;
 		r_list_foreach (core->anal->fcns, iter, fcni) {
 			if (fcni->addr < next && fcni->addr > core->offset) {
 				next = fcni->addr;
 			}
 		}
-	} else
-	if (strstr (type, "hit")) {
+	} else if (strstr (type, "hit")) {
 		const char *pfx = r_config_get (core->config, "search.prefix");
 		RFlagItem *flag;
 		r_list_foreach (core->flags->flags, iter, flag) {
@@ -2714,7 +2809,7 @@ R_API void r_core_visual_define (RCore *core, const char *args) {
 		h = 0;
 		r_cons_clear00 ();
 	} else {
-		r_cons_gotoxy (0, h);
+		r_cons_gotoxy (0, 3);
 	}
 	const char *lines[] = { ""
 		,"[Vd]- Define current block as:"
@@ -2739,6 +2834,7 @@ R_API void r_core_visual_define (RCore *core, const char *args) {
 		," s    set string"
 		," S    set strings in current block"
 		," u    undefine metadata here"
+		," v    rename variable at offset that matches some hex digits"
 		," x    find xrefs to current address (./r)"
 		," w    set as 32bit word"
 		," W    set as 64bit word"
@@ -2819,7 +2915,7 @@ repeat:
 		{
 			char *man = NULL;
 			/* check for manpage */
-			RAnalOp *op = r_core_anal_op (core, off);
+			RAnalOp *op = r_core_anal_op (core, off, R_ANAL_OP_MASK_BASIC);
 			if (op) {
 				if (op->jump != UT64_MAX) {
 					RFlagItem *item = r_flag_get_i (core->flags, op->jump);
@@ -2834,8 +2930,8 @@ repeat:
 			if (man) {
 				char *p = strstr (man, "INODE");
 				if (p) *p = 0;
-				r_cons_clear();
-				r_cons_flush();
+				r_cons_clear ();
+				r_cons_flush ();
 				r_sys_cmdf ("man %s", man);
 				free (man);
 			}
@@ -2852,27 +2948,19 @@ repeat:
 		}
 		// TODO: get the aligned instruction even if the cursor is in the middle of it.
 		r_anal_op (core->anal, &op, off,
-			core->block + off - core->offset, 32);
+			core->block + off - core->offset, 32, R_ANAL_OP_MASK_BASIC);
 
 		tgt_addr = op.jump != UT64_MAX ? op.jump : op.ptr;
 		if (op.var) {
 //			q = r_str_newf ("?i Rename variable %s to;afvn %s `?y`", op.var->name, op.var->name);
 			RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, off, 0);
 			if (fcn) {
-				RAnalVar *bar = r_anal_var_get_byname (core->anal, fcn, op.var->name);
-				if (!bar) {
-					bar = r_anal_var_get_byname (core->anal, fcn, op.var->name);
-					if (!bar) {
-						bar = r_anal_var_get_byname (core->anal, fcn, op.var->name);
-					}
-				}
+				RAnalVar *bar = r_anal_var_get_byname (core->anal, fcn->addr, op.var->name);
 				if (bar) {
-					char *newname = r_cons_input (sdb_fmt (0, "New variable name for '%s': ", bar->name));
-					if (newname) {
-						if (*newname) {
-							r_anal_var_rename (core->anal, fcn->addr, bar->scope,
-								bar->kind, bar->name, newname);
-						}
+					char *newname = r_cons_input (sdb_fmt ("New variable name for '%s': ", bar->name));
+					if (newname && *newname) {
+						r_anal_var_rename (core->anal, fcn->addr, bar->scope,
+								bar->kind, bar->name, newname, true);
 						free (newname);
 					}
 				} else {
@@ -2955,9 +3043,9 @@ repeat:
 			RAnalOp op;
 			ut64 size;
 			if (r_anal_op (core->anal, &op, off, core->block+delta,
-					core->blocksize-delta)) {
+					core->blocksize-delta, R_ANAL_OP_MASK_BASIC)) {
 				size = off - fcn->addr + op.size;
-				r_anal_fcn_resize (fcn, size);
+				r_anal_fcn_resize (core->anal, fcn, size);
 			}
 		}
 		}
@@ -3020,7 +3108,7 @@ repeat:
 						   (const char *)name + 4);
 			}
 			r_name_filter (name, n + 10);
-			r_flag_set (core->flags, name, off+ntotal, n);
+			r_flag_set (core->flags, name, off + ntotal, n);
 			free (name);
 			if (is_wide) {
 				ntotal += n * 2 - 1;
@@ -3090,7 +3178,7 @@ repeat:
 			int funsize = 0;
 			RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->offset, 0);
 			if (fcn) {
-				r_anal_fcn_resize (fcn, core->offset - fcn->addr);
+				r_anal_fcn_resize (core->anal, fcn, core->offset - fcn->addr);
 			}
 			//int depth = r_config_get_i (core->config, "anal.depth");
 			if (core->print->cur_enabled) {
@@ -3104,10 +3192,80 @@ repeat:
 			r_cons_break_pop ();
 			if (funsize) {
 				RAnalFunction *f = r_anal_get_fcn_in (core->anal, off, -1);
-				r_anal_fcn_set_size (f, funsize);
+				r_anal_fcn_set_size (core->anal, f, funsize);
 			}
 		}
 		break;
+        case 'v':
+        {
+		RAnalOp op;
+		ut64 N;
+		char *endptr = NULL;
+		char *end_off = r_cons_input ("Last hexadecimal digits of instruction: ");
+		if (end_off) {
+			N = strtoull (end_off, &endptr, 16);
+		}
+		if (!end_off || end_off == endptr) {
+			eprintf ("Invalid numeric input\n");
+			r_cons_any_key (NULL);
+			free (end_off);
+			break;
+		}
+		free (end_off);
+
+		ut64 incr = 0x10;
+		ut64 tmp_N = N >> 4;
+		while (tmp_N > 0) {
+			tmp_N = tmp_N >> 4;
+			incr = incr << 4;
+		}
+		ut64 mask = incr - 1;
+
+		ut64 start_off;
+		if ((off & mask) <= N) {
+			start_off = (off & ~mask) ^ N;
+		} else {
+			start_off = ((off & ~mask) ^ incr) ^ N;
+		}
+
+		ut64 try_off;
+		bool found = false;
+		for (try_off = start_off; try_off < start_off + incr*16; try_off += incr) {
+			r_anal_op (core->anal, &op, try_off,
+				core->block + try_off - core->offset, 32, R_ANAL_OP_MASK_ALL);
+			if (op.var) {
+				found = true;
+				break;
+			}
+		}
+
+		if (found) {
+			RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, off, 0);
+			if (fcn) {
+				RAnalVar *bar = r_anal_var_get_byname (core->anal, fcn->addr, op.var->name);
+				if (bar) {
+					char *newname = r_cons_input (sdb_fmt ("New variable name for '%s': ", bar->name));
+					if (newname && *newname) {
+						r_anal_var_rename (core->anal, fcn->addr, bar->scope,
+								bar->kind, bar->name, newname, true);
+						free (newname);
+					}
+				} else {
+					eprintf ("Cannot find variable\n");
+					r_cons_any_key (NULL);
+				}
+			} else {
+				eprintf ("Cannot find function\n");
+				r_cons_any_key (NULL);
+			}
+		} else {
+			eprintf ("Cannot find instruction with a variable\n");
+			r_cons_any_key (NULL);
+		}
+
+		r_anal_op_fini (&op);
+		break;
+        }
 	case 'Q':
 	case 'q':
 	default:
@@ -3124,62 +3282,75 @@ repeat:
 
 R_API void r_core_visual_colors(RCore *core) {
 	char color[32], cstr[32];
-	const char *k, *kol;
 	int ch, opt = 0, oopt = -1;
-	ut8 r, g, b;
+	const char *k;
+	RColor rcolor;
 
-	r = g = b = 0;
-	kol = r_cons_pal_get_color (opt);
-	r_cons_rgb_parse (kol, &r, &g, &b, NULL);
+	rcolor = r_cons_pal_get_i (opt);
 	for (;;) {
 		r_cons_clear ();
-		k = r_cons_pal_get_i (opt);
+		k = r_cons_pal_get_name (opt);
 		if (!k) {
 			opt = 0;
-			k = r_cons_pal_get_i (opt);
+			k = r_cons_pal_get_name (opt);
 		}
 		r_cons_gotoxy (0, 0);
-		r_cons_rgb_str (cstr, r, g, b, 0);
-		r&=0xf;
-		g&=0xf;
-		b&=0xf;
-		sprintf (color, "rgb:%x%x%x", r, g, b);
-//r_cons_printf ("COLOR%s(%sXXX)"Color_RESET"\n", kol, kol?kol+1:"");
-		r_cons_printf ("# Colorscheme %d - Use '.' and ':' to randomize palette\n"
-			"# Press 'rRgGbB', 'jk' or 'q'\nec %s %s   # %d (%s)\n",
+		r_cons_rgb_str (cstr, sizeof (cstr), &rcolor);
+		if (r_cons_singleton ()->color < COLOR_MODE_16M) {
+			rcolor.r &= 0xf;
+			rcolor.g &= 0xf;
+			rcolor.b &= 0xf;
+		}
+		sprintf (color, "rgb:%x%x%x", rcolor.r, rcolor.g, rcolor.b);
+		r_cons_printf ("# Colorscheme %d - Use '.' to randomize current color and ':' to randomize palette\n"
+			"# Press 'rRgGbB', 'jk' or 'q'\nec %s %s   # %d (\\x1b%s)\n",
 			opt, k, color, atoi (cstr+7), cstr+1);
 		r_core_cmdf (core, "ec %s %s", k, color);
 		char * res = r_core_cmd_str (core, "pd $r");
 		int h, w = r_cons_get_size (&h);
 		char *body = r_str_ansi_crop (res, 0, 0, w, h - 4);
-		r_cons_printf("%s", body);
+		if (body) {
+			r_cons_printf ("%s", body);
+		}
 		r_cons_flush ();
 		ch = r_cons_readchar ();
 		ch = r_cons_arrow_to_hjkl (ch);
 		switch (ch) {
 #define CASE_RGB(x,X,y) \
-	case x:y--;if(y>0x7f)y=0;break;\
-	case X:y++;if(y>15)y=15;break;
-		CASE_RGB ('R','r',r);
-		CASE_RGB ('G','g',g);
-		CASE_RGB ('B','b',b);
+	case x:if (y > 0x00) { y--; } break;\
+	case X:if (y < 0xff) { y++; } break;
+		CASE_RGB ('R','r',rcolor.r);
+		CASE_RGB ('G','g',rcolor.g);
+		CASE_RGB ('B','b',rcolor.b);
 		case 'Q':
-		case 'q': return;
-		case 'k': opt--; break;
-		case 'j': opt++; break;
-		case 'K': opt=0; break;
-		case 'J': opt=0; break; // XXX must go to end
-		case ':': r_cons_pal_random (); break;
+		case 'q':
+			free (body);
+			return;
+		case 'k':
+			opt--;
+			break;
+		case 'j':
+			opt++;
+			break;
+		case 'K':
+			opt = 0;
+			break;
+		case 'J':
+			opt = r_cons_pal_len () - 1;
+			break;
+		case ':':
+			r_cons_pal_random ();
+			break;
 		case '.':
-			r = r_num_rand (0xf);
-			g = r_num_rand (0xf);
-			b = r_num_rand (0xf);
+			rcolor.r = r_num_rand (0xff);
+			rcolor.g = r_num_rand (0xff);
+			rcolor.b = r_num_rand (0xff);
 			break;
 		}
 		if (opt != oopt) {
-			kol = r_cons_pal_get_color (opt);
-			r_cons_rgb_parse (kol, &r, &g, &b, NULL);
+			rcolor = r_cons_pal_get_i (opt);
 			oopt = opt;
 		}
+		free (body);
 	}
 }
