@@ -528,7 +528,7 @@ RDebugPlugin r_debug_plugin_bochs = {
 	.stop = &r_debug_bochs_stop,
 	.wait = &r_debug_bochs_wait,
 	.map_get = r_debug_bochs_map_get,
-	.breakpoint = (RBreakpointCallback)&r_debug_bochs_breakpoint,
+	.breakpoint = r_debug_bochs_breakpoint,
 	.reg_read = &r_debug_bochs_reg_read,
 	.reg_write = &r_debug_bochs_reg_write,
 	.reg_profile = (void *)r_debug_bochs_reg_profile,
