@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2017 - pancake */
+/* radare - LGPL - Copyright 2009-2018 - pancake */
 
 #include <r_userconf.h>
 #include <r_debug.h>
@@ -134,7 +134,7 @@ static int windows_step (RDebug *dbg) {
 	regs.EFlags |= 0x100;
 	r_debug_native_reg_write (dbg, R_REG_TYPE_GPR, (ut8 *)&regs, sizeof (regs));
 	r_debug_native_continue (dbg, dbg->pid, dbg->tid, dbg->reason.signum);
-	r_debug_handle_signals (dbg);
+	(void)r_debug_handle_signals (dbg);
 	return true;
 }
 #endif
@@ -249,8 +249,7 @@ static int r_debug_native_continue(RDebug *dbg, int pid, int tid, int sig) {
 	}
 	return tid;
 #elif __APPLE__
-	bool ret;
-	ret = xnu_continue (dbg, pid, tid, sig);
+	bool ret = xnu_continue (dbg, pid, tid, sig);
 	if (!ret) {
 		return -1;
 	}
@@ -316,8 +315,11 @@ static bool tracelib(RDebug *dbg, const char *mode, PLIB_ITEM item) {
 		case 'u': needle = dbg->glob_unlibs; break;
 		}
 	}
-	eprintf ("(%d) %sing library at %p (%s) %s\n", item->pid, mode,
+	//eprintf ("(%d) %sing library at %p (%s) %s\n", item->pid, mode,
+		//item->BaseOfDll, item->Path, item->Name);
+	r_cons_printf ("(%d) %sing library at %p (%s) %s\n", item->pid, mode,
 		item->BaseOfDll, item->Path, item->Name);
+	r_cons_flush ();
 	if (needle && strlen (needle)) {
 		tmp = r_str_glob (item->Name, needle);
 	}
@@ -365,7 +367,9 @@ static RDebugReasonType r_debug_native_wait (RDebug *dbg, int pid) {
 				dbg->corebind.cmdf (core, "o-%d", fd);
 			}
 		} else {
-			eprintf ("Loading unknown library.\n");
+			//eprintf ("Loading unknown library.\n");
+			r_cons_printf ("Loading unknown library.\n");
+			r_cons_flush ();
 		}
 	} else if (reason == R_DEBUG_REASON_EXIT_LIB) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
@@ -375,13 +379,19 @@ static RDebugReasonType r_debug_native_wait (RDebug *dbg, int pid) {
 			}
 			r_debug_info_free (r);
 		} else {
-			eprintf ("Unloading unknown library.\n");
+			//eprintf ("Unloading unknown library.\n");
+			r_cons_printf ("Unloading unknown library.\n");
+			r_cons_flush ();
+
 		}
 	} else if (reason == R_DEBUG_REASON_NEW_TID) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
 		if (r && r->thread) {
 			PTHREAD_ITEM item = r->thread;
-			eprintf ("(%d) Created thread %d (start @ %p)\n", item->pid, item->tid, item->lpStartAddress);
+			//eprintf ("(%d) Created thread %d (start @ %p)\n", item->pid, item->tid, item->lpStartAddress);
+			r_cons_printf ("(%d) Created thread %d (start @ %p)\n", item->pid, item->tid, item->lpStartAddress);
+			r_cons_flush ();
+
 			r_debug_info_free (r);
 		}
 
@@ -389,7 +399,10 @@ static RDebugReasonType r_debug_native_wait (RDebug *dbg, int pid) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
 		if (r && r->thread) {
 			PTHREAD_ITEM item = r->thread;
-			eprintf ("(%d) Finished thread %d Exit code %d\n", (ut32)item->pid, (ut32)item->tid, (ut32)item->dwExitCode);
+			//eprintf ("(%d) Finished thread %d Exit code %d\n", (ut32)item->pid, (ut32)item->tid, (ut32)item->dwExitCode);
+			r_cons_printf ("(%d) Finished thread %d Exit code %d\n", (ut32)item->pid, (ut32)item->tid, (ut32)item->dwExitCode);
+			r_cons_flush ();
+
 			r_debug_info_free (r);
 		}
 	}
@@ -440,7 +453,10 @@ static RDebugReasonType r_debug_native_wait (RDebug *dbg, int pid) {
 #ifdef WAIT_ON_ALL_CHILDREN
 	int ret = waitpid (-1, &status, WAITPID_FLAGS);
 #else
-	int ret = waitpid (pid, &status, WAITPID_FLAGS);
+	int ret = waitpid (-1, &status, 0);
+	if (ret != -1) {
+		reason = R_DEBUG_REASON_TRAP;
+	}
 #endif // WAIT_ON_ALL_CHILDREN
 	if (ret == -1) {
 		r_sys_perror ("waitpid");
@@ -453,10 +469,9 @@ static RDebugReasonType r_debug_native_wait (RDebug *dbg, int pid) {
 	if (ret != pid) {
 		reason = R_DEBUG_REASON_NEW_PID;
 		eprintf ("switching to pid %d\n", ret);
-		r_debug_select(dbg, ret, ret);
+		r_debug_select (dbg, ret, ret);
 	}
 #endif // WAIT_ON_ALL_CHILDREN
-
 	// TODO: switch status and handle reasons here
 #if __linux__ && defined(PT_GETEVENTMSG)
 	reason = linux_ptrace_event (dbg, pid, status);
@@ -556,92 +571,76 @@ static RList *r_debug_native_pids (RDebug *dbg, int pid) {
 		}
 	}
 #elif __linux__
-	int i;
-	char *ptr, buf[1024];
-
 	list->free = (RListFree)&r_debug_pid_free;
+	DIR *dh;
+	struct dirent *de;
+	char *ptr, st, buf[1024];
+	int i, uid;
 	if (pid) {
-		DIR *dh;
-		struct dirent *de;
-
 		/* add the requested pid. should we do this? we don't even know if it's valid still.. */
 		r_list_append (list, r_debug_pid_new ("(current)", pid, 0, 's', 0));
-
-		/* list parents */
-		dh = opendir ("/proc");
-		if (!dh) {
-			r_sys_perror ("opendir /proc");
-			r_list_free (list);
-			return NULL;
-		}
-		while ((de = readdir (dh))) {
-			int uid = 0;
-			int gid = 0; // unused
-			/* for each existing pid file... */
-			i = atoi (de->d_name);
-			if (i <= 0) {
-				continue;
-			}
-
-			/* try to read the status */
-			buf[0] = 0;
-			if (procfs_pid_slurp (i, "status", buf, sizeof (buf)) == -1) {
-				continue;
-			}
-			buf[sizeof (buf) - 1] = 0;
-
-			ptr = strstr (buf, "Uid:");
-			if (ptr) {
-				uid = atoi (ptr + 4);
-			}
-
-			ptr = strstr (buf, "Gid:");
-			if (ptr) {
-				gid = atoi (ptr + 4);
-			}
-
-			/* look for the parent process id */
-			ptr = strstr (buf, "PPid:");
-			if (ptr) {
-				int ppid = atoi (ptr + 5);
-
-				/* if this is the requested process... */
-				if (i == pid) {
-					// eprintf ("PPid: %d\n", ppid);
-					// append it to the list with parent
-					r_list_append (list, r_debug_pid_new (
-						"(ppid)", ppid, uid, 's', 0));
-				}
-
-				/* ignore it if it is not one of our children */
-				if (ppid != pid) {
-					continue;
-				}
-
-				/* it's a child of the requested pid, read it's command line and add it */
-				if (procfs_pid_slurp (ppid, "cmdline", buf, sizeof (buf)) == -1) {
-					continue;
-				}
-				// TODO: add support for gid in RDebugPid.new()
-				r_list_append (list, r_debug_pid_new (buf, i, uid, 's', 0));
-			}
-		}
-		closedir (dh);
-	} else {
-		/* try to bruteforce the processes
-		 * XXX(jjd): wouldn't listing the processes like before work better?
-		 */
-		for (i = 2; i < MAXPID; i++) {
-			/* try to send signal 0, if it fails it must not be valid */
-			if (r_sandbox_kill (i, 0) == -1) {
-				continue;
-			}
-			if (procfs_pid_slurp (i, "cmdline", buf, sizeof (buf)) == -1) {
-				continue;
-			}
-			r_list_append (list, r_debug_pid_new (buf, i, 0, 's', 0));
-		}
 	}
+	dh = opendir ("/proc");
+	if (!dh) {
+		r_sys_perror ("opendir /proc");
+		r_list_free (list);
+		list = NULL;
+	}
+	while ((de = readdir (dh))) {
+		uid = 0;
+		st = ' ';
+		/* for each existing pid file... */
+		i = atoi (de->d_name);
+		if (i <= 0) {
+			continue;
+		}
+
+		/* try to read the status */
+		buf[0] = 0;
+		if (procfs_pid_slurp (i, "status", buf, sizeof (buf)) == -1) {
+			continue;
+		}
+		buf[sizeof (buf) - 1] = 0;
+
+		// get process State
+		ptr = strstr (buf, "State:");
+		if (ptr) {
+			st = ptr[7];
+		}
+		/* look for the parent process id */
+		ptr = strstr (buf, "PPid:");
+		if (pid && ptr) {
+			int ppid = atoi (ptr + 5);
+
+			/* if this is the requested process... */
+			if (i == pid) {
+				// append it to the list with parent
+				r_list_append (list, r_debug_pid_new (
+					"(ppid)", ppid, uid, st, 0));
+			}
+
+			/* ignore it if it is not one of our children */
+			if (ppid != pid) {
+				continue;
+			}
+		}
+
+		// get process Uid
+		ptr = strstr (buf, "Uid:");
+		if (ptr) {
+			uid = atoi (ptr + 4);
+		}
+		// TODO: add support for gid in RDebugPid.new()
+		// ptr = strstr (buf, "Gid:");
+		// if (ptr) {
+		// 	gid = atoi (ptr + 4);
+		// }
+		if (procfs_pid_slurp (i, "cmdline", buf, sizeof (buf)) == -1) {
+			continue;
+		}
+		r_list_append (list, r_debug_pid_new (buf, i, uid, st, 0));
+	}
+	closedir (dh);
 #else /* rest is BSD */
 #ifdef __NetBSD__
 # define KVM_OPEN_FLAG KVM_NO_FILES
@@ -969,6 +968,9 @@ static RDebugMap* linux_map_alloc (RDebug *dbg, ut64 addr, int size) {
 		sc_name = "mmap";
 	}
 	num = r_syscall_get_num (dbg->anal->syscall, sc_name);
+#ifndef MAP_ANONYMOUS
+#define MAP_ANONYMOUS 0x20
+#endif
 	snprintf (code, sizeof (code),
 		"sc_mmap@syscall(%d);\n"
 		"main@naked(0) { .rarg0 = sc_mmap(0x%08"PFMT64x",%d,%d,%d,%d,%d);break;\n"
@@ -1173,7 +1175,7 @@ static RList *r_debug_native_map_get (RDebug *dbg) {
 #endif
 	fd = fopen (path, "r");
 	if (!fd) {
-		perror (sdb_fmt (0, "Cannot open '%s'", path));
+		perror (sdb_fmt ("Cannot open '%s'", path));
 		return NULL;
 	}
 
@@ -1521,8 +1523,7 @@ static bool arm64_hwbp_del (RDebug *dbg, RBreakpoint *bp, RBreakpointItem *b) {
  * we only handle the case for hardware breakpoints here. otherwise,
  * we let the caller handle the work.
  */
-static int r_debug_native_bp (void *bp_, RBreakpointItem *b, bool set) {
-	RBreakpoint *bp = (RBreakpoint *)bp_;
+static int r_debug_native_bp (RBreakpoint *bp, RBreakpointItem *b, bool set) {
 	RDebug *dbg = bp->user;
 	if (b && b->hw) {
 #if __i386__ || __x86_64__
