@@ -342,7 +342,7 @@ RDebugPlugin r_debug_plugin_qnx = {
 	.canstep = 1,
 	.wait = &r_debug_qnx_wait,
 	.map_get = r_debug_qnx_map_get,
-	.breakpoint = (RBreakpointCallback)&r_debug_qnx_breakpoint,
+	.breakpoint = r_debug_qnx_breakpoint,
 	.reg_read = &r_debug_qnx_reg_read,
 	.reg_write = &r_debug_qnx_reg_write,
 	.reg_profile = (void *)r_debug_qnx_reg_profile,
