@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2016 - pancake */
+/* radare - LGPL - Copyright 2009-2018 - pancake */
 
 #include <r_userconf.h>
 
@@ -75,7 +75,7 @@ int linux_handle_signals (RDebug *dbg) {
 		//ptrace (PTRACE_SETSIGINFO, dbg->pid, 0, &siginfo);
 		dbg->reason.type = R_DEBUG_REASON_SIGNAL;
 		dbg->reason.signum = siginfo.si_signo;
-		//dbg->stopaddr = siginfo.si_addr;
+		dbg->stopaddr = (ut64)siginfo.si_addr;
 		//dbg->errno = siginfo.si_errno;
 		// siginfo.si_code -> HWBKPT, USER, KERNEL or WHAT
 #warning DO MORE RDEBUGREASON HERE
@@ -105,7 +105,7 @@ int linux_handle_signals (RDebug *dbg) {
 				}
 				if (dbg->reason.type != R_DEBUG_REASON_NEW_LIB &&
 					dbg->reason.type != R_DEBUG_REASON_EXIT_LIB) {
-					dbg->reason.bp_addr = (ut64)siginfo.si_addr;
+					dbg->reason.bp_addr = (ut64)(size_t)siginfo.si_addr;
 					dbg->reason.type = R_DEBUG_REASON_BREAKPOINT;
 				}
 			}
@@ -125,7 +125,7 @@ int linux_handle_signals (RDebug *dbg) {
 			eprintf ("[+] SIGNAL %d errno=%d addr=0x%08"PFMT64x
 				" code=%d ret=%d\n",
 				siginfo.si_signo, siginfo.si_errno,
-				(ut64)siginfo.si_addr, siginfo.si_code, ret);
+				(ut64)(size_t)siginfo.si_addr, siginfo.si_code, ret);
 		}
 		return true;
 	}
@@ -295,28 +295,6 @@ void linux_attach_new_process (RDebug *dbg) {
 	r_debug_select (dbg, dbg->forked_pid, dbg->forked_pid);
 }
 
-static RDebugPid *find_rdebug_pid (RDebug *dbg, int pid) {
-	RList *list = dbg->threads;
-	if (list) {
-		RDebugPid *th;
-		RListIter *it;
-		r_list_foreach (list, it, th) {
-			if (th->pid == pid) {
-				return th;
-			}
-		}
-	}
-	return NULL;
-}
-
-static bool get_pid_signalled_status (RDebug *dbg, int pid) {
-	RDebugPid *th = find_rdebug_pid (dbg, pid);
-	if (th) {
-		return th->signalled;
-	}
-	return false;
-}
-
 RDebugReasonType linux_dbg_wait(RDebug *dbg, int my_pid) {
 	RDebugReasonType reason = R_DEBUG_REASON_UNKNOWN;
 	int pid = (dbg->continue_all_threads && dbg->n_threads) ? -1 : dbg->main_pid;
@@ -780,7 +758,7 @@ void print_fpu (void *f, int r){
 	}
 #endif
 #else
-#warning not implemented for this platform
+#warning print_fpu not implemented for this platform
 #endif
 }
 
@@ -875,7 +853,7 @@ int linux_reg_read (RDebug *dbg, int type, ut8 *buf, int size) {
 		}
 		}
 #else
-	#warning not implemented for this platform
+	#warning getfpregs not implemented for this platform
 #endif
 		break;
 	case R_REG_TYPE_SEG:
