@@ -1,9 +1,13 @@
-/* radare - LGPL - Copyright 2007-2017 pancake */
+/* radare - LGPL - Copyright 2007-2018 pancake */
 
 #include <r_hash.h>
-#ifdef _MSC_VER
-#define strcasecmp stricmp
+#include "r_util.h"
+#if USE_LIB_XXHASH
+#include <xxhash.h>
+#else
+#include "xxhash.h"
 #endif
+
 R_LIB_VERSION (r_hash);
 
 struct {
@@ -158,6 +162,10 @@ R_API ut8 r_hash_mod255(const ut8 *b, ut64 len) {
 	return c % 255;
 }
 
+R_API ut32 r_hash_xxhash(const ut8 *buf, ut64 len) {
+	return XXH32 (buf, (size_t)len, 0);
+}
+
 R_API ut8 r_hash_deviation(const ut8 *b, ut64 len) {
 	int i, c;
 	for (c = i = 0, len--; i < len; i++) {
@@ -286,14 +294,14 @@ R_API ut64 r_hash_name_to_bits(const char *name) {
 	do {
 		/* Eat everything up to the comma */
 		for (i = 0; *ptr && *ptr != ',' && i < sizeof (tmp) - 1; i++) {
-			tmp[i] = *ptr++;
+			tmp[i] = tolower (*ptr++);
 		}
 
 		/* Safety net */
 		tmp[i] = '\0';
 
 		for (i = 0; hash_name_bytes[i].name; i++) {
-			if (!strcasecmp (tmp, hash_name_bytes[i].name)) {
+			if (!strcmp (tmp, hash_name_bytes[i].name)) {
 				ret |= hash_name_bytes[i].bit;
 				break;
 			}
@@ -343,7 +351,10 @@ R_API char *r_hash_to_string(RHash *ctx, const char *name, const ut8 *data, int
 	r_hash_do_begin (ctx, algo);
 	digest_size = r_hash_calculate (ctx, algo, data, len);
 	r_hash_do_end (ctx, algo);
-	if (digest_size > 0) {
+	if (digest_size == 0) {
+		digest_hex = calloc (16, 1);
+		snprintf (digest_hex, 15, "%02.8f", ctx->entropy);
+	} else if (digest_size > 0) {
 		if (digest_size * 2 < digest_size) {
 			digest_hex = NULL;
 		} else {
