@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2008-2017 - condret, pancake, alvaro_fe */
+/* radare2 - LGPL - Copyright 2008-2018 - condret, pancake, alvaro_fe */
 
 #include <r_io.h>
 #include <sdb.h>
@@ -59,30 +59,30 @@ typedef int (*cbOnIterMap)(RIO *io, int fd, ut64 addr, ut8 *buf, int len, RIOMap
 // If prefix_mode is true, returns the number of bytes of operated prefix; returns < 0 on error.
 // If prefix_mode is false, operates in non-stop mode and returns true iff all IO operations on overlapped maps are complete.
 static st64 on_map_skyline(RIO *io, ut64 vaddr, ut8 *buf, int len, int match_flg, cbOnIterMap op, bool prefix_mode) {
-	const RVector *skyline = &io->map_skyline;
+	const RPVector *skyline = &io->map_skyline;
 	ut64 addr = vaddr;
-	int i;
+	size_t i;
 	bool ret = true, wrap = !prefix_mode && vaddr + len < vaddr;
 #define CMP(addr, part) (addr < r_itv_end (((RIOMapSkyline *)part)->itv) - 1 ? -1 : \
 			addr > r_itv_end (((RIOMapSkyline *)part)->itv) - 1 ? 1 : 0)
 	// Let i be the first skyline part whose right endpoint > addr
 	if (!len) {
-		i = skyline->len;
+		i = r_pvector_len (skyline);
 	} else {
-		r_vector_lower_bound (skyline, addr, i, CMP);
-		if (i == skyline->len && wrap) {
+		r_pvector_lower_bound (skyline, addr, i, CMP);
+		if (i == r_pvector_len (skyline) && wrap) {
 			wrap = false;
 			i = 0;
 			addr = 0;
 		}
 	}
 #undef CMP
-	while (i < skyline->len) {
-		const RIOMapSkyline *part = skyline->a[i];
+	while (i < r_pvector_len (skyline)) {
+		const RIOMapSkyline *part = r_pvector_at (skyline, i);
 		// Right endpoint <= addr
 		if (r_itv_end (part->itv) - 1 < addr) {
 			i++;
-			if (wrap && i == skyline->len) {
+			if (wrap && i == r_pvector_len (skyline)) {
 				wrap = false;
 				i = 0;
 				addr = 0;
@@ -120,6 +120,7 @@ static st64 on_map_skyline(RIO *io, ut64 vaddr, ut8 *buf, int len, int match_flg
 			break;
 		} else {
 			addr += len1;
+			ret = false;
 		}
 		// Reaches the end
 		if (addr == vaddr + len) {
@@ -207,6 +208,7 @@ R_API RIO* r_io_init(RIO* io) {
 	}
 	io->addrbytes = 1;
 	r_io_desc_init (io);
+	r_pvector_init (&io->map_skyline, free);
 	r_io_map_init (io);
 	r_io_section_init (io);
 	r_io_cache_init (io);
@@ -343,11 +345,22 @@ R_API bool r_io_reopen(RIO* io, int fd, int flags, int mode) {
 	}
 	//does this really work, or do we have to handler debuggers ugly
 	uri = old->referer? old->referer: old->uri;
+#if __WINDOWS__ //TODO: workaround, see https://github.com/radare/radare2/issues/8840
+	if (!r_io_desc_close (old)) {
+		return false; // TODO: this is an unrecoverable scenario
+	}
+	if (!(new = r_io_open_nomap (io, uri, flags, mode))) {
+		return false;
+	}
+	r_io_desc_exchange (io, old->fd, new->fd);
+	return true;
+#else
 	if (!(new = r_io_open_nomap (io, uri, flags, mode))) {
 		return false;
 	}
 	r_io_desc_exchange (io, old->fd, new->fd);
 	return r_io_desc_close (old); // magic
+#endif // __WINDOWS__
 }
 
 R_API int r_io_close_all(RIO* io) { // what about undo?
@@ -384,7 +397,8 @@ R_API int r_io_pwrite_at(RIO* io, ut64 paddr, const ut8* buf, int len) {
 	return r_io_desc_write_at (io->desc, paddr, buf, len);
 }
 
-R_API bool r_io_vread_at(RIO* io, ut64 vaddr, ut8* buf, int len) {
+// Returns true iff all reads on mapped regions are successful and complete.
+R_API bool r_io_vread_at_mapped(RIO* io, ut64 vaddr, ut8* buf, int len) {
 	if (!io || !buf || (len < 1)) {
 		return false;
 	}
@@ -398,7 +412,7 @@ R_API bool r_io_vread_at(RIO* io, ut64 vaddr, ut8* buf, int len) {
 	return onIterMap_wrap (io->maps->tail, io, vaddr, buf, len, R_IO_READ, fd_read_at_wrap, NULL);
 }
 
-R_API bool r_io_vwrite_at(RIO* io, ut64 vaddr, const ut8* buf, int len) {
+static bool r_io_vwrite_at(RIO* io, ut64 vaddr, const ut8* buf, int len) {
 	if (!io || !buf || (len < 1)) {
 		return false;
 	}
@@ -440,23 +454,77 @@ R_API RIOAccessLog *r_io_al_vwrite_at(RIO* io, ut64 vaddr, const ut8* buf, int l
 	return log;
 }
 
-R_API bool r_io_read_at(RIO* io, ut64 addr, ut8* buf, int len) {
+// Deprecated, use either r_io_read_at_mapped or r_io_nread_at instead.
+// For virtual mode, returns true if all reads on mapped regions are successful
+// and complete.
+// For physical mode, the interface is broken because the actual read bytes are
+// not available. This requires fixes in all call sites.
+R_API bool r_io_read_at(RIO *io, ut64 addr, ut8 *buf, int len) {
 	bool ret;
 	if (!io || !buf || len < 1) {
 		return false;
 	}
 	if (io->buffer_enabled) {
-		return !!r_io_buffer_read (io, addr, buf, len);
+		return !!r_io_buffer_read(io, addr, buf, len);
 	}
 	if (io->va) {
-		ret = r_io_vread_at (io, addr, buf, len);
-		io->ret = ret? len: -1;
+		ret = r_io_vread_at_mapped(io, addr, buf, len);
 	} else {
-		io->ret = r_io_pread_at (io, addr, buf, len);
-		ret = io->ret > 0;
+		ret = r_io_pread_at(io, addr, buf, len) > 0;
 	}
 	if (io->cached & R_IO_READ) {
-		(void)r_io_cache_read (io, addr, buf, len);
+		(void)r_io_cache_read(io, addr, buf, len);
+	}
+	return ret;
+}
+
+// Returns true iff all reads on mapped regions are successful and complete.
+// Unmapped regions are filled with io->Oxff in both physical and virtual modes.
+// Use this function if you want to ignore gaps or do not care about the number
+// of read bytes.
+R_API bool r_io_read_at_mapped(RIO *io, ut64 addr, ut8 *buf, int len) {
+	bool ret;
+	if (!io || !buf) {
+		return false;
+	}
+	if (io->buffer_enabled) {
+		return !!r_io_buffer_read(io, addr, buf, len);
+	}
+	if (io->ff) {
+		memset(buf, io->Oxff, len);
+	}
+	if (io->va) {
+		ret = on_map_skyline(io, addr, buf, len, R_IO_READ, fd_read_at_wrap, false);
+	} else {
+		ret = r_io_pread_at(io, addr, buf, len) > 0;
+	}
+	if (io->cached & R_IO_READ) {
+		(void)r_io_cache_read(io, addr, buf, len);
+	}
+	return ret;
+}
+
+// For both virtual and physical mode, returns the number of bytes of read
+// prefix.
+// Returns -1 on error.
+R_API int r_io_nread_at(RIO *io, ut64 addr, ut8 *buf, int len) {
+	int ret;
+	if (!io || !buf) {
+		return -1;
+	}
+	if (io->buffer_enabled) {
+		return r_io_buffer_read(io, addr, buf, len);
+	}
+	if (io->va) {
+		if (io->ff) {
+			memset(buf, io->Oxff, len);
+		}
+		ret = on_map_skyline(io, addr, buf, len, R_IO_READ, fd_read_at_wrap, true);
+	} else {
+		ret = r_io_pread_at(io, addr, buf, len);
+	}
+	if (ret > 0 && io->cached & R_IO_READ) {
+		(void)r_io_cache_read(io, addr, buf, len);
 	}
 	return ret;
 }
@@ -511,13 +579,10 @@ R_API bool r_io_write_at(RIO* io, ut64 addr, const ut8* buf, int len) {
 	}
 	if (io->cached & R_IO_WRITE) {
 		ret = r_io_cache_write (io, addr, mybuf, len);
-		io->ret = ret? len: -1;
 	} else if (io->va) {
 		ret = r_io_vwrite_at (io, addr, mybuf, len);
-		io->ret = ret? len: -1;
 	} else {
-		io->ret = r_io_pwrite_at (io, addr, mybuf, len);
-		ret = io->ret > 0;
+		ret = r_io_pwrite_at (io, addr, mybuf, len) > 0;
 	}
 	if (buf != mybuf) {
 		free (mybuf);
@@ -676,10 +741,13 @@ R_API int r_io_bind(RIO* io, RIOBind* bnd) {
 	bnd->fd_write_at = r_io_fd_write_at;
 	bnd->fd_is_dbg = r_io_fd_is_dbg;
 	bnd->fd_get_name = r_io_fd_get_name;
+	bnd->fd_get_map = r_io_map_get_for_fd;
+	bnd->fd_remap = r_io_map_remap_fd;
 	bnd->al_sort = r_io_accesslog_sort;
 	bnd->al_free = r_io_accesslog_free;
 	bnd->al_buf_byflags = r_io_accesslog_getf_buf_byflags;
 	bnd->is_valid_offset = r_io_is_valid_offset;
+	bnd->addr_is_mapped = r_io_addr_is_mapped;
 	bnd->sections_vget = r_io_sections_vget;
 	bnd->section_add = r_io_section_add;
 	bnd->sect_vget = r_io_section_vget;
