@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2017 - condret, MaskRay */
+/* radare2 - LGPL - Copyright 2017-2018 - condret, MaskRay */
 
 #include <r_io.h>
 #include <stdlib.h>
@@ -7,7 +7,7 @@
 #include "r_util.h"
 #include "r_vector.h"
 
-#define END_OF_MAP_IDS  0xffffffff
+#define END_OF_MAP_IDS UT32_MAX
 
 #define MAP_USE_HALF_CLOSED 0
 
@@ -34,7 +34,7 @@ static int _cmp_map_event_by_id(const void *a_, const void *b_) {
 }
 
 // Precondition: from == 0 && to == 0 (full address) or from < to
-static bool _map_skyline_push(RVector *map_skyline, ut64 from, ut64 to, RIOMap *map) {
+static bool _map_skyline_push(RPVector *map_skyline, ut64 from, ut64 to, RIOMap *map) {
 	RIOMapSkyline *part = R_NEW (RIOMapSkyline), *part1;
 	if (!part) {
 		return false;
@@ -49,11 +49,11 @@ static bool _map_skyline_push(RVector *map_skyline, ut64 from, ut64 to, RIOMap *
 		}
 		part1->map = map;
 		part1->itv = (RInterval){ UT64_MAX, 1 };
-		if (!r_vector_push (map_skyline, part1)) {
+		if (!r_pvector_push (map_skyline, part1)) {
 			free (part1);
 		}
 	}
-	if (!r_vector_push (map_skyline, part)) {
+	if (!r_pvector_push (map_skyline, part)) {
 		free (part);
 		return false;
 	}
@@ -64,12 +64,13 @@ static bool _map_skyline_push(RVector *map_skyline, ut64 from, ut64 to, RIOMap *
 R_API void r_io_map_calculate_skyline(RIO *io) {
 	SdbListIter *iter;
 	RIOMap *map;
-	RVector events = {0};
+	RPVector events;
 	RBinHeap heap;
 	struct map_event_t *ev;
 	bool *deleted = NULL;
-	r_vector_clear (&io->map_skyline, free);
-	if (!r_vector_reserve (&events, ls_length (io->maps) * 2) ||
+	r_pvector_clear (&io->map_skyline);
+	r_pvector_init (&events, free);
+	if (!r_pvector_reserve (&events, ls_length (io->maps) * 2) ||
 			!(deleted = calloc (ls_length (io->maps), 1))) {
 		goto out;
 	}
@@ -83,7 +84,7 @@ R_API void r_io_map_calculate_skyline(RIO *io) {
 		ev->addr = map->itv.addr;
 		ev->is_to = false;
 		ev->id = i;
-		r_vector_push (&events, ev);
+		r_pvector_push (&events, ev);
 		if (!(ev = R_NEW (struct map_event_t))) {
 			goto out;
 		}
@@ -91,16 +92,16 @@ R_API void r_io_map_calculate_skyline(RIO *io) {
 		ev->addr = r_itv_end (map->itv);
 		ev->is_to = true;
 		ev->id = i;
-		r_vector_push (&events, ev);
+		r_pvector_push (&events, ev);
 		i++;
 	}
-	r_vector_sort (&events, _cmp_map_event);
+	r_pvector_sort (&events, _cmp_map_event);
 
 	r_binheap_init (&heap, _cmp_map_event_by_id);
 	ut64 last;
 	RIOMap *last_map = NULL;
-	for (i = 0; i < events.len; i++) {
-		ev = events.a[i];
+	for (i = 0; i < r_pvector_len (&events); i++) {
+		ev = r_pvector_at (&events, i);
 		if (ev->is_to) {
 			deleted[ev->id] = true;
 		} else {
@@ -134,9 +135,9 @@ R_API void r_io_map_calculate_skyline(RIO *io) {
 		}
 	}
 
-	r_binheap_clear (&heap, NULL);
+	r_binheap_clear (&heap);
 out:
-	r_vector_clear (&events, free);
+	r_pvector_clear (&events);
 	free (deleted);
 }
 
@@ -152,6 +153,7 @@ R_API RIOMap* r_io_map_new(RIO* io, int fd, int flags, ut64 delta, ut64 addr, ut
 	map->fd = fd;
 	map->delta = delta;
 	if ((UT64_MAX - size + 1) < addr) {
+		/// XXX: this is leaking a map!!!
 		r_io_map_new (io, fd, flags, delta - addr, 0LL, size + addr, do_skyline);
 		size = -(st64)addr;
 	}
@@ -183,6 +185,21 @@ R_API bool r_io_map_remap (RIO *io, ut32 id, ut64 addr) {
 	return false;
 }
 
+R_API bool r_io_map_remap_fd (RIO *io, int fd, ut64 addr) {
+	RList *maps;
+	RIOMap *map;
+	bool retval = false;
+	maps = r_io_map_get_for_fd (io, fd);
+	if (maps) {
+		map = r_list_get_n (maps, 0);
+		if (map) {
+			retval = r_io_map_remap (io, map->id, addr);
+		}
+		r_list_free (maps);
+	}
+	return retval;
+}
+
 static void _map_free(void* p) {
 	RIOMap* map = (RIOMap*) p;
 	if (map) {
@@ -340,6 +357,23 @@ R_API bool r_io_map_priorize(RIO* io, ut32 id) {
 	return false;
 }
 
+R_API bool r_io_map_depriorize(RIO* io, ut32 id) {
+	if (io) {
+		RIOMap* map;
+		SdbListIter* iter;
+		ls_foreach (io->maps, iter, map) {
+			// search for iter with the correct map
+			if (map->id == id) {
+				ls_split_iter (io->maps, iter);
+				ls_prepend (io->maps, map);
+				r_io_map_calculate_skyline (io);
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
 R_API bool r_io_map_priorize_for_fd(RIO* io, int fd) {
 	SdbListIter* iter, * ator;
 	RIOMap *map;
@@ -406,7 +440,7 @@ R_API void r_io_map_fini(RIO* io) {
 	io->maps = NULL;
 	r_id_pool_free (io->map_ids);
 	io->map_ids = NULL;
-	r_vector_clear (&io->map_skyline, free);
+	r_pvector_clear (&io->map_skyline);
 }
 
 R_API void r_io_map_set_name(RIOMap* map, const char* name) {
@@ -448,6 +482,24 @@ R_API RIOMap* r_io_map_add_next_available(RIO* io, int fd, int flags, ut64 delta
 	return r_io_map_new (io, fd, flags, delta, next_addr, size, true);
 }
 
+R_API ut64 r_io_map_next_address(RIO* io, ut64 addr) {
+	RIOMap* map;
+	SdbListIter* iter;
+	ut64 lowest = UT64_MAX;
+
+	ls_foreach (io->maps, iter, map) {
+		ut64 from = r_itv_begin (map->itv);
+		if (from > addr && addr < lowest) {
+			lowest = from;
+		}
+		ut64 to = r_itv_end (map->itv);
+		if (to > addr && to < lowest) {
+			lowest = to;
+		}
+	}
+	return lowest;
+}
+
 R_API RList* r_io_map_get_for_fd(RIO* io, int fd) {
 	RList* map_list = r_list_newf (NULL);
 	SdbListIter* iter;
