@@ -226,10 +226,14 @@ static int r_io_def_mmap_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 			free (a_buf);
 			return count;
 		}
+		if (lseek (mmo->fd, io->off, SEEK_SET) < 0) {
+			return -1;
+		}
 		return read (mmo->fd, buf, count);
 	}
-	if (mmo->buf->length < io->off)
+	if (mmo->buf->length < io->off) {
 		io->off = mmo->buf->length;
+	}
 	return r_buf_read_at (mmo->buf, io->off, buf, count);
 }
 
@@ -337,14 +341,13 @@ static bool __plugin_open_default(RIO *io, const char *file, bool many) {
 
 // default open should permit opening
 static RIODesc *__open_default(RIO *io, const char *file, int flags, int mode) {
-	RIODesc *iod;
 	if (r_str_startswith (file, "file://")) {
 		file += strlen ("file://");
 	}
 	if (!r_io_def_mmap_check_default (file)) {
 		return NULL;
 	}
-	iod = r_io_def_mmap_open (io, file, flags, mode);
+	RIODesc *iod = r_io_def_mmap_open (io, file, flags, mode);
 	return iod;
 // NTOE: uncomment this line to support loading files in ro as fallback is rw fails
 //	return iod? iod: r_io_def_mmap_open (io, file, R_IO_READ, mode);
