@@ -227,23 +227,39 @@ static void print_help (RIO *io, char *cmd, int p_usage) {
 	int i = 0;
 	int cmd_len = cmd ? strlen (cmd) : 0;
 	const char* usage = "Usage:   \\[MprRw][lpP] [args...]";
-	const char* help_msg[] = {"\\M                      Print kernel memory map",
-				  "\\b      beid [pid]      Change r2k backend. pid is required when beid is 1. Possible beid 0: linear address; 1: process address; 2: physical address",
-				  "\\p      pid             Print process information",
-				  "\\rl     addr len        Read from linear address",
-				  "\\rp     pid addr len    Read from process address",
-				  "\\rP     addr len        Read physical address",
-				  "\\R[p]                   Print control registers. Use =!Rp for detailed description",
-				  "\\wl[x]  addr input      Write at linear address. Use =!wlx for input in hex",
-				  "\\wp[x]  pid addr input  Write at process address. Use =!wpx for input in hex",
-				  "\\wP[x]  addr input      Write at physical address. Use =!wPx for input in hex",
-				  "\\W      1|0             Honor arch write protect (1 enable WP, 0 disable WP)"};
+	const char* help_msg[] = {
+		"\\dm              Print kernel memory map (or process if r2k.io==1)",
+		"\\dr              Print control registers",
+		"\\dR              Print control registers in detailed mode",
+		"\\dp [pid]        Print current selected pid or change it",
+		"\\e r2k.io=[012]  Read/Write from 0: Linear, 1: Process, 2: Physical addresses"
+	};
+	const char *help_msg_old[] = {
+		"\\M                      Print kernel memory map",
+		"\\b      beid [pid]      Change r2k backend. pid is required when beid is 1.",
+		"                         0: linear address; 1: process address; 2: physical address",
+		"\\p      pid             Print process information",
+		"\\rl     addr len        Read from linear address",
+		"\\rp     pid addr len    Read from process address",
+		"\\rP     addr len        Read physical address",
+		"\\R[p]                   Print control registers. Use =!Rp for detailed description",
+		"\\wl[x]  addr input      Write at linear address. Use =!wlx for input in hex",
+		"\\wp[x]  pid addr input  Write at process address. Use =!wpx for input in hex",
+		"\\wP[x]  addr input      Write at physical address. Use =!wPx for input in hex",
+		"\\W      1|0             Honor arch write protect (1 enable WP, 0 disable WP)"
+	};
 	if (p_usage) {
 		io->cb_printf ("%s\n", usage);
 	}
 	for (i = 0; i < (sizeof (help_msg) / sizeof (char*)); i++) {
 		if (!cmd || !strncmp (cmd, help_msg[i]+1, cmd_len)) {
-			io->cb_printf ("%s\n",help_msg[i]);
+			io->cb_printf ("%s\n", help_msg[i]);
+		}
+	}
+	io->cb_printf ("\nOld Commands: (deprecated)\n");
+	for (i = 0; i < (sizeof (help_msg_old) / sizeof (char*)); i++) {
+		if (!cmd || !strncmp (cmd, help_msg_old[i]+1, cmd_len)) {
+			io->cb_printf ("%s\n", help_msg_old[i]);
 		}
 	}
 }
@@ -361,12 +377,10 @@ int WriteMemory (RIO *io, RIODesc *iodesc, int ioctl_n, size_t pid, ut64 address
 	return ret;
 }
 
-int run_ioctl_command(RIO *io, RIODesc *iodesc, const char *buf) {
+int run_old_command(RIO *io, RIODesc *iodesc, const char *buf) {
 	int ret, inphex, ioctl_n;
 	size_t pid, addr, len;
 	ut8 *databuf = NULL;
-	buf = r_str_ichr ((char *) buf, ' ');
-
 	switch (*buf) {
 	case 'W':
 		{
@@ -549,9 +563,10 @@ int run_ioctl_command(RIO *io, RIODesc *iodesc, const char *buf) {
 			ioctl_n = IOCTL_WRITE_PHYSICAL_ADDR;
 			break;
 		default:
-			print_help(io, "w", 0);
+			print_help (io, "w", 0);
 			goto end;
 		}
+		// coverity says this cant happen, but it doesnt hurts to add a check
 		if (!buf) {
 			break;
 		}
@@ -706,7 +721,7 @@ int run_ioctl_command(RIO *io, RIODesc *iodesc, const char *buf) {
 			buffsize = (ut64) (sizeof (proc_data.vmareastruct) / sizeof (proc_data.vmareastruct[0]));
 			if (fflag) {
 				int j = 0;
-				for (i = 0; i < buffsize;) {
+				for (i = 0; i + 1 < buffsize;) {
 					nextstart = 0;
 					if (i + 7 < buffsize) {
 						nextstart = i + 7 + (strlen ((const char *)&(proc_data.vmareastruct[i + 7])) - 1 + sizeof (size_t)) / sizeof (size_t);
@@ -721,8 +736,10 @@ int run_ioctl_command(RIO *io, RIODesc *iodesc, const char *buf) {
 					j += 1;
 					i = nextstart;
 				}
+				io->cb_printf ("f pid.%d.task_struct = 0x%08"PFMT64x"\n", proc_data.pid, proc_data.task);
 			} else {
 				io->cb_printf ("pid = %d\nprocess name = %s\n", proc_data.pid, proc_data.comm);
+				io->cb_printf ("task_struct = 0x%08"PFMT64x"\n", proc_data.task);
 				for (i = 0; i < buffsize;) {
 					nextstart = 0;
 					if (i + 7 < buffsize) {
@@ -732,7 +749,7 @@ int run_ioctl_command(RIO *io, RIODesc *iodesc, const char *buf) {
 					    nextstart > 0 && nextstart - 1 < buffsize) {
 						break;
 					}
-					io->cb_printf ("%08"PFMT64x"-%08"PFMT64x" %c%c%c%c %08"PFMT64x" %02x:%02x %-8"PFMT64u"",
+					io->cb_printf ("0x%08"PFMT64x" - 0x%08"PFMT64x" %c%c%c%c 0x%08"PFMT64x" %02x:%02x %-8"PFMT64u"",
 							(ut64) proc_data.vmareastruct[i], (ut64) proc_data.vmareastruct[i+1],
 							proc_data.vmareastruct[i + 2] & VM_READ ? 'r' : '-',
 							proc_data.vmareastruct[i + 2] & VM_WRITE ? 'w' : '-',
@@ -757,4 +774,83 @@ int run_ioctl_command(RIO *io, RIODesc *iodesc, const char *buf) {
 	return 0;
 }
 
+int run_new_command(RIO *io, RIODesc *iodesc, const char *buf) {
+	if (r_str_startswith (buf, "dm")) {
+		if (buf[2] == ' ') {
+			// use \p pid
+			char *cmd = r_str_newf ("p %d", atoi (buf + 2));
+			run_old_command (io, iodesc, cmd);
+			free (cmd);
+		} else if (r2k_struct.beid == 1) {
+			// use \p pid
+			char *cmd = r_str_newf ("p %d", r2k_struct.pid);
+			run_old_command (io, iodesc, cmd);
+			free (cmd);
+		} else {
+			// use \M
+			run_old_command (io, iodesc, "M");
+		}
+		return 1;
+	}
+	if (r_str_startswith (buf, "dr")) {
+		run_old_command (io, iodesc, "R");
+		return 1;
+	}
+	if (r_str_startswith (buf, "dR")) {
+		run_old_command (io, iodesc, "Rp");
+		return 1;
+	}
+	if (r_str_startswith (buf, "dp")) {
+		if (buf[2] == ' ') {
+			r2k_struct.pid = atoi (buf + 3);
+		} else {
+			io->cb_printf ("%d\n", r2k_struct.pid);
+		}
+		return 1;
+	}
+	if (r_str_startswith (buf, "e r2k.io")) {
+		if (strchr (buf, '?')) {
+			io->cb_printf ("0: Linear memory\n");
+			io->cb_printf ("1: Process memory\n");
+			io->cb_printf ("2: Physical memory\n");
+			return 1;
+		}
+		const char *eq = strchr (buf, '=');
+		if (eq) {
+			int v = atoi (eq + 1);
+			int p = r2k_struct.pid;
+			char *cmd = r_str_newf ("b %d %d", v, p);
+			run_old_command (io, iodesc, cmd);
+			free (cmd);
+		} else {
+			run_new_command (io, iodesc, "dp");
+		}
+		return 1;
+	}
+	if (r_str_startswith (buf, "e r2k.wp")) {
+		if (strchr (buf, '?')) {
+			io->cb_printf ("<bool> enable write protection (disabled by default)\n");
+			return 1;
+		}
+		const char *eq = strchr (buf, '=');
+		if (eq) {
+			int v = atoi (eq + 1);
+			r2k_struct.wp = (ut8)v;
+		} else {
+			io->cb_printf ("%s", r_str_bool (r2k_struct.wp));
+		}
+		return 1;
+	}
+	return 0;
+}
+
+int run_ioctl_command(RIO *io, RIODesc *iodesc, const char *buf) {
+	buf = r_str_ichr ((char *) buf, ' ');
+
+	if (!run_new_command (io, iodesc, buf)) {
+		return run_old_command (io, iodesc, buf);
+	}
+	return 0;
+}
+
 #endif
