@@ -109,8 +109,8 @@ R_API int r_io_plugin_list_json(RIO *io) {
 		str[2] = plugin->isdbg ? 'd' : '_';
 		str[3] = 0;
 
-		io->cb_printf ("{\"Permissions\":\"%s\",\"Name\":\"%s\",\"Description\":\"%s\",\"License\":\"%s\"",
-				str, plugin->name,
+		io->cb_printf ("%s{\"Permissions\":\"%s\",\"Name\":\"%s\",\"Description\":\"%s\",\"License\":\"%s\"",
+				n? "," : "", str, plugin->name,
 			plugin->desc, plugin->license);
 		if (plugin->version) {
 			io->cb_printf (",\"version\":\"%s\"", plugin->version);
@@ -124,3 +124,37 @@ R_API int r_io_plugin_list_json(RIO *io) {
 	io->cb_printf("]}");
 	return n;
 }
+
+R_API int r_io_plugin_read(RIODesc *desc, ut8 *buf, int len) {
+	if (!buf || !desc || !desc->plugin || len < 1 || !(desc->flags & R_IO_READ)) {
+		return 0;
+	}
+	if (!desc->plugin->read) {
+		return -1;
+	}
+	return desc->plugin->read (desc->io, desc, buf, len);
+}
+
+R_API int r_io_plugin_write(RIODesc *desc, const ut8 *buf, int len) {
+	if (!buf || !desc || !desc->plugin || len < 1 || !(desc->flags & R_IO_WRITE)) {
+		return 0;
+	}
+	if (!desc->plugin->write) {
+		return -1;
+	}
+	return desc->plugin->write (desc->io, desc, buf, len);
+}
+
+R_API int r_io_plugin_read_at(RIODesc *desc, ut64 addr, ut8 *buf, int len) {
+	if (r_io_desc_seek (desc, addr, R_IO_SEEK_SET)  == addr) {
+		return r_io_plugin_read (desc, buf, len);
+	}
+	return 0;
+}
+
+R_API int r_io_plugin_write_at(RIODesc *desc, ut64 addr, const ut8 *buf, int len) {
+	if (r_io_desc_seek (desc, addr, R_IO_SEEK_SET)  == addr) {
+		return r_io_plugin_write (desc, buf, len);
+	}
+	return 0;
+}
