@@ -47,6 +47,8 @@
 #endif
 #if defined(__APPLE__) || defined(__NetBSD__) || defined(__OpenBSD__)
 #include <util.h>
+#elif defined(__FreeBSD__)
+#include <libutil.h>
 #endif
 #endif
 #ifdef _MSC_VER
@@ -54,7 +56,7 @@
 #include <process.h>  // to compile execv in msvc windows
 #endif
 
-#define HAVE_PTY __UNIX__ && !__ANDROID__ && LIBC_HAVE_FORK && !defined(__sun)
+#define HAVE_PTY __UNIX__ && !__ANDROID__ && LIBC_HAVE_FORK && !__sun
 
 #if EMSCRIPTEN
 #undef HAVE_PTY
@@ -216,11 +218,12 @@ static char *getstr(const char *src) {
 	return ret;
 }
 
-static int parseBool (const char *e) {
+static int parseBool(const char *e) {
 	return (strcmp (e, "yes")?
+		(strcmp (e, "on")?
 		(strcmp (e, "true")?
 		(strcmp (e, "1")?
-		0: 1): 1): 1);
+		0: 1): 1): 1): 1);
 }
 
 #if __linux__
@@ -243,7 +246,7 @@ static void setASLR(RRunProfile *r, int enabled) {
 #if __ANDROID__
 		setRVA ("0\n");
 #else
-#ifdef ADDR_NO_RANDOMIZE
+#if HAVE_DECL_ADDR_NO_RANDOMIZE
 		if (personality (ADDR_NO_RANDOMIZE) == -1)
 #endif
 			setRVA ("0\n");
@@ -267,17 +270,19 @@ static void setASLR(RRunProfile *r, int enabled) {
 #endif
 }
 
-static void restore_saved_fd (int saved, bool resore, int fd) {
+#if HAVE_PTY
+static void restore_saved_fd(int saved, bool restore, int fd) {
 	if (saved == -1) {
 		return;
 	}
-	if (resore) {
+	if (restore) {
 		dup2 (saved, fd);
 	}
 	close (saved);
 }
+#endif
 
-static int handle_redirection_proc (const char *cmd, bool in, bool out, bool err) {
+static int handle_redirection_proc(const char *cmd, bool in, bool out, bool err) {
 #if HAVE_PTY
 	// use PTY to redirect I/O because pipes can be problematic in
 	// case of interactive programs.
@@ -291,28 +296,53 @@ static int handle_redirection_proc (const char *cmd, bool in, bool out, bool err
 		return -1;
 	}
 	int fdm, pid = forkpty (&fdm, NULL, NULL, NULL);
+	if (pid == -1) {
+		close (saved_stdin);
+		close (saved_stdout);
+		return -1;
+	}
+	const char *tn = ttyname (fdm);
+	if (!tn) {
+		close (saved_stdin);
+		close (saved_stdout);
+		return -1;
+	}
+	int fds = open (tn, O_RDWR);
+	if (fds == -1) {
+		close (saved_stdin);
+		close (saved_stdout);
+		return -1;
+	}
 	if (pid == 0) {
+		close (fdm);
 		// child process
 		if (in) {
-			dup2 (fdm, STDIN_FILENO);
+			dup2 (fds, STDIN_FILENO);
 		}
 		if (out) {
-			dup2 (fdm, STDOUT_FILENO);
+			dup2 (fds, STDOUT_FILENO);
 		}
 		// child - program to run
 
 		// necessary because otherwise you can read the same thing you
 		// wrote on fdm.
 		struct termios t;
-		tcgetattr (0, &t);
+		tcgetattr (fds, &t);
 		cfmakeraw (&t);
-		tcsetattr (0, TCSANOW, &t);
+		tcsetattr (fds, TCSANOW, &t);
 
 		int code = r_sys_cmd (cmd);
 		restore_saved_fd (saved_stdin, in, STDIN_FILENO);
 		restore_saved_fd (saved_stdout, out, STDOUT_FILENO);
 		exit (code);
 	} else {
+		close (fds);
+		if (in) {
+			dup2 (fdm, STDIN_FILENO);
+		}
+		if (out) {
+			dup2 (fdm, STDOUT_FILENO);
+		}
 		// parent process
 		int status;
 		waitpid (pid, &status, 0);
@@ -397,7 +427,7 @@ static int handle_redirection(const char *cmd, bool in, bool out, bool err) {
 	}
 }
 
-R_API int r_run_parsefile (RRunProfile *p, const char *b) {
+R_API int r_run_parsefile(RRunProfile *p, const char *b) {
 	char *s = r_file_slurp (b, NULL);
 	if (s) {
 		int ret = r_run_parse (p, s);
@@ -407,7 +437,7 @@ R_API int r_run_parsefile (RRunProfile *p, const char *b) {
 	return 0;
 }
 
-R_API bool r_run_parseline (RRunProfile *p, char *b) {
+R_API bool r_run_parseline(RRunProfile *p, char *b) {
 	int must_free = false;
 	char *e = strchr (b, '=');
 	if (!e || *b == '#') {
@@ -551,7 +581,7 @@ R_API const char *r_run_help() {
 	"# #stdio=blah.txt\n"
 	"# #stderr=foo.txt\n"
 	"# stdout=foo.txt\n"
-	"# stdin=input.txt # or !program to redirect input to another program\n"
+	"# stdin=input.txt # or !program to redirect input from another program\n"
 	"# input=input.txt\n"
 	"# chdir=/\n"
 	"# chroot=/mnt/chroot\n"
@@ -565,7 +595,7 @@ R_API const char *r_run_help() {
 	"# nice=5\n";
 }
 
-#if __UNIX__
+#if HAVE_PTY
 static int fd_forward(int in_fd, int out_fd, char **buff) {
 	int size = 0;
 
@@ -583,7 +613,10 @@ static int fd_forward(int in_fd, int out_fd, char **buff) {
 		return -1;
 	}
 	*buff = new_buff;
-	(void)read (in_fd, *buff, size);
+	if (read (in_fd, *buff, size) != size) {
+		perror ("read");
+		return -1;
+	}
 	if (write (out_fd, *buff, size) != size) {
 		perror ("write");
 		return -1;
