@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2006-2017 - pancake */
+/* radare - LGPL - Copyright 2006-2018 - pancake */
 
 /* must be included first because of winsock2.h and windows.h */
 #include <r_socket.h>
@@ -40,7 +40,7 @@ R_API int r_socket_unix_listen (RSocket *s, const char *file) {
 R_API bool r_socket_connect (RSocket *s, const char *host, const char *port, int proto, unsigned int timeout) {
 	return false;
 }
-R_API int r_socket_spawn (RSocket *s, const char *cmd, unsigned int timeout) {
+R_API bool r_socket_spawn (RSocket *s, const char *cmd, unsigned int timeout) {
 	return -1;
 }
 R_API int r_socket_close_fd (RSocket *s) {
@@ -211,13 +211,13 @@ R_API RSocket *r_socket_new (int is_ssl) {
 	return s;
 }
 
-R_API int r_socket_spawn (RSocket *s, const char *cmd, unsigned int timeout) {
+R_API bool r_socket_spawn (RSocket *s, const char *cmd, unsigned int timeout) {
 	// XXX TODO: dont use sockets, we can achieve the same with pipes
 	const int port = 2000 + r_num_rand (2000);
-	int childPid = r_sys_fork();
+	int childPid = r_sys_fork ();
 	if (childPid == 0) {
 		char *a = r_str_replace (strdup (cmd), "\\", "\\\\", true);
-		r_sys_cmdf ("rarun2 system=\"%s\" listen=%d", a, port);
+		int res = r_sys_cmdf ("rarun2 system=\"%s\" listen=%d", a, port);
 		free (a);
 #if 0
 		// TODO: use the api
@@ -229,15 +229,35 @@ R_API int r_socket_spawn (RSocket *s, const char *cmd, unsigned int timeout) {
 		r_run_free (rp);
 		free (profile);
 #endif
+		if (res != 0) {
+			eprintf ("r_socket_spawn: rarun2 failed\n");
+			exit (1);
+		}
 		eprintf ("r_socket_spawn: %s is dead\n", cmd);
 		exit (0);
 	}
 	r_sys_sleep (1);
 	r_sys_usleep (timeout);
+
 	char aport[32];
 	sprintf (aport, "%d", port);
 	// redirect stdin/stdout/stderr
-	return r_socket_connect (s, "127.0.0.1", aport, R_SOCKET_PROTO_TCP, 2000);
+	bool sock = r_socket_connect (s, "127.0.0.1", aport, R_SOCKET_PROTO_TCP, 2000);
+	if (!sock) {
+		return false;
+	}
+#if __UNIX__
+	r_sys_sleep (4);
+	r_sys_usleep (timeout);
+
+	int status = 0;
+	int ret = waitpid (childPid, &status, WNOHANG);
+	if (ret != 0) {
+		r_socket_close (s);
+		return false;
+	}
+#endif
+	return true;
 }
 
 R_API bool r_socket_connect (RSocket *s, const char *host, const char *port, int proto, unsigned int timeout) {
@@ -559,7 +579,10 @@ R_API RSocket *r_socket_accept(RSocket *s) {
 	//signal (SIGPIPE, SIG_DFL);
 	sock->fd = accept (s->fd, (struct sockaddr *)&s->sa, &salen);
 	if (sock->fd == -1) {
-		r_sys_perror ("accept");
+		if (errno != EWOULDBLOCK) {
+			// not just a timeout
+			r_sys_perror ("accept");
+		}
 		free (sock);
 		return NULL;
 	}
@@ -587,6 +610,30 @@ R_API RSocket *r_socket_accept(RSocket *s) {
 	return sock;
 }
 
+R_API RSocket *r_socket_accept_timeout(RSocket *s, unsigned int timeout) {
+	fd_set read_fds;
+	fd_set except_fds;
+
+	FD_ZERO (&read_fds);
+	FD_SET (s->fd, &read_fds);
+
+	FD_ZERO (&except_fds);
+	FD_SET (s->fd, &except_fds);
+
+	struct timeval t;
+	t.tv_sec = timeout;
+	t.tv_usec = 0;
+
+	int r = select (s->fd + 1, &read_fds, NULL, &except_fds, &t);
+	if(r < 0) {
+		perror ("select");
+	} else if (r > 0 && FD_ISSET (s->fd, &read_fds)) {
+		return r_socket_accept (s);
+	}
+
+	return NULL;
+}
+
 R_API int r_socket_block_time (RSocket *s, int block, int sec) {
 #if __UNIX__ || defined(__CYGWIN__)
 	int ret, flags;
