@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2017 - pancake */
+/* radare - LGPL - Copyright 2007-2018 - pancake */
 
 #include "r_types.h"
 #include "r_util.h"
@@ -214,10 +214,11 @@ R_API char *r_file_abspath(const char *file) {
 }
 
 R_API char *r_file_path(const char *bin) {
-	char file[1024];
 	char *path_env;
+	char *file = NULL;
 	char *path = NULL;
 	char *str, *ptr;
+	const char *extension = "";
 	if (!bin) {
 		return NULL;
 	}
@@ -228,17 +229,23 @@ R_API char *r_file_path(const char *bin) {
 		return NULL;
 	}
 	path_env = (char *)r_sys_getenv ("PATH");
+#if __WINDOWS__
+	if (!r_str_endswith (bin, ".exe")) {
+		extension = ".exe";
+	}
+#endif
 	if (path_env) {
 		str = path = strdup (path_env);
 		do {
-			ptr = strchr (str, ':');
+			ptr = strchr (str, R_SYS_ENVSEP[0]);
 			if (ptr) {
 				*ptr = '\0';
-				snprintf (file, sizeof (file), "%s"R_SYS_DIR"%s", str, bin);
+				file = r_str_newf (R_JOIN_2_PATHS ("%s", "%s%s"), str, bin, extension);
 				if (r_file_exists (file)) {
+					free (file);
 					free (path);
 					free (path_env);
-					return strdup (file);
+					return file;
 				}
 				str = ptr + 1;
 			}
@@ -246,6 +253,7 @@ R_API char *r_file_path(const char *bin) {
 	}
 	free (path_env);
 	free (path);
+	free (file);
 	return strdup (bin);
 }
 
@@ -444,16 +452,14 @@ R_API char *r_file_slurp_random_line_count(const char *file, int *line) {
 	/* Reservoir Sampling */
 	char *ptr = NULL, *str;
 	int sz, i, lines, selection = -1;
-	struct timeval tv;
 	int start = *line;
 	if ((str = r_file_slurp (file, &sz))) {
-		gettimeofday (&tv, NULL);
-		srand (getpid() + tv.tv_usec);
+		r_num_irand ();
 		for (i = 0; str[i]; i++) {
 			if (str[i] == '\n') {
 				//here rand doesn't have any security implication
 				// https://www.securecoding.cert.org/confluence/display/c/MSC30-C.+Do+not+use+the+rand()+function+for+generating+pseudorandom+numbers
-				if (!(rand() % (++(*line)))) {
+				if (!(r_num_rand ((++(*line))))) {
 					selection = (*line - 1);  /* The line we want. */
 				}
 			}
@@ -574,9 +580,13 @@ R_API bool r_file_hexdump(const char *file, const ut8 *buf, int len, int append)
 	return true;
 }
 
-R_API bool r_file_dump(const char *file, const ut8 *buf, int len, int append) {
+R_API bool r_file_touch(const char *file) {
+	return r_file_dump(file, NULL, 0, true);
+}
+
+R_API bool r_file_dump(const char *file, const ut8 *buf, int len, bool append) {
 	FILE *fd;
-	if (!file || !*file || !buf || len < 0) {
+	if (!file || !*file) {
 		eprintf ("r_file_dump file: %s buf: %p\n", file, buf);
 		return false;
 	}
@@ -590,13 +600,15 @@ R_API bool r_file_dump(const char *file, const ut8 *buf, int len, int append) {
 		eprintf ("Cannot open '%s' for writing\n", file);
 		return false;
 	}
-	if (len < 0) {
-		len = strlen ((const char *)buf);
-	}
-	if (fwrite (buf, len, 1, fd) != 1) {
-		r_sys_perror ("r_file_dump: fwrite: error\n");
-		fclose (fd);
-		return false;
+	if (buf) {
+		if (len < 0) {
+			len = strlen ((const char *)buf);
+		}
+		if (len > 0 && fwrite (buf, len, 1, fd) != 1) {
+			r_sys_perror ("r_file_dump: fwrite: error\n");
+			fclose (fd);
+			return false;
+		}
 	}
 	fclose (fd);
 	return true;
@@ -822,7 +834,9 @@ static RMmap *r_file_mmap_other (RMmap *m) {
 R_API RMmap *r_file_mmap (const char *file, bool rw, ut64 base) {
 	RMmap *m = NULL;
 	int fd = -1;
-	if (!rw && !r_file_exists (file)) return m;
+	if (!rw && !r_file_exists (file)) {
+		return m;
+	}
 	fd = r_sandbox_open (file, rw? O_RDWR: O_RDONLY, 0644);
 	if (fd == -1 && !rw) {
 		eprintf ("r_file_mmap: file does not exis.\n");
@@ -904,6 +918,9 @@ R_API char *r_file_temp (const char *prefix) {
 R_API int r_file_mkstemp(const char *prefix, char **oname) {
 	int h = -1;
 	char *path = r_file_tmpdir ();
+	if (!prefix) {
+		prefix = "r2";
+	}
 #if __WINDOWS__
 	LPTSTR name = NULL;
 	LPTSTR path_ = r_sys_conv_utf8_to_utf16 (path);
@@ -933,10 +950,24 @@ err_r_file_mkstemp:
 	free (prefix_);
 #else
 	char name[1024];
+	char pfxx[1024];
+	const char *suffix = strchr (prefix, '*');
 
-	snprintf (name, sizeof (name) - 1, "%s/r2.%s.XXXXXX", path, prefix);
+	if (suffix) {
+		suffix++;
+		r_str_ncpy (pfxx, prefix, (size_t)(suffix - prefix));
+		prefix = pfxx;
+	} else {
+		suffix = "";
+	}
+
+	snprintf (name, sizeof (name) - 1, "%s/r2.%s.XXXXXX%s", path, prefix, suffix);
 	mode_t mask = umask (S_IWGRP | S_IWOTH);
-	h = mkstemp (name);
+	if (suffix && *suffix) {
+		h = mkstemps (name, strlen (suffix));
+	} else {
+		h = mkstemp (name);
+	}
 	umask (mask);
 	if (oname) {
 		*oname = (h!=-1)? strdup (name): NULL;
@@ -1002,7 +1033,7 @@ R_API bool r_file_copy (const char *src, const char *dst) {
 	/* TODO: implement in C */
 	/* TODO: Use NO_CACHE for iOS dyldcache copying */
 #if HAVE_COPYFILE_H
-	return copyfile (src, dst, 0, 0) != -1;
+	return copyfile (src, dst, 0, COPYFILE_DATA | COPYFILE_XATTR) != -1;
 #elif __WINDOWS__
 	return r_sys_cmdf ("copy %s %s", src, dst);
 #else
@@ -1014,3 +1045,72 @@ R_API bool r_file_copy (const char *src, const char *dst) {
 	return rc == 0;
 #endif
 }
+
+static void recursive_search_glob (const char *path, const char *glob, RList* list, int depth) {
+	if (depth < 1) {
+		return;
+	}
+	char* file;
+	RListIter *iter;
+	RList *dir = r_sys_dir (path);
+	r_list_foreach (dir, iter, file) {
+		if (!strcmp (file, ".") || !strcmp (file, "..")) {
+			continue;
+		}
+		char *filename = malloc (strlen (path) + strlen (file) + 2);
+		strcpy (filename, path);
+		strcat (filename, file);
+		if (r_file_is_directory (filename)) {
+			strcat (filename, R_SYS_DIR);
+			recursive_search_glob (filename, glob, list, depth - 1);
+			free (filename);
+		} else if (r_str_glob (file, glob)) {
+			r_list_append (list, filename);
+		} else {
+			free (filename);
+		}
+	}
+	r_list_free (dir);
+}
+
+R_API RList* r_file_globsearch (const char *_globbed_path, int maxdepth) {
+	char *globbed_path = strdup (_globbed_path);
+	RList *files = r_list_newf (free);
+	char *glob = strchr (globbed_path, '*');
+	if (!glob) {
+		r_list_append (files, strdup (globbed_path));
+	} else {
+		*glob = '\0';
+		char *last_slash = (char *)r_str_last (globbed_path, R_SYS_DIR);
+		*glob = '*';
+		char *path, *glob_ptr;
+		if (last_slash) {
+			glob_ptr = last_slash + 1;
+			if (globbed_path[0] == '~') {
+				char *rpath = r_str_newlen (globbed_path + 2, last_slash - globbed_path - 1);
+				path = r_str_home (rpath ? rpath : "");
+				free (rpath);
+			} else {
+				path = r_str_newlen (globbed_path, last_slash - globbed_path + 1);
+			}
+		} else {
+			glob_ptr = globbed_path;
+			path = r_str_newf (".%s", R_SYS_DIR);
+		}
+
+		if (!path) {
+			r_list_free (files);
+			free (globbed_path);
+			return NULL;
+		}
+
+		if (*(glob + 1) == '*') {  // "**"
+			recursive_search_glob (path, glob_ptr, files, maxdepth);
+		} else {                   // "*"
+			recursive_search_glob (path, glob_ptr, files, 1);
+		}
+		free (path);
+	}
+	free (globbed_path);
+	return files;
+}
