@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2016 - pancake, alvarofe */
+/* radare - LGPL - Copyright 2007-2018 - pancake, alvarofe */
 // TODO: RRef - reference counting
 
 #include <stdio.h>
@@ -6,11 +6,11 @@
 #define _R_LIST_C_
 #include "r_util.h"
 
-inline RListIter *r_list_iter_new () {
+inline RListIter *r_list_iter_new() {
 	return calloc (1, sizeof (RListIter));
 }
 
-void r_list_iter_free (RListIter *list) {
+void r_list_iter_free(RListIter *list) {
 	/* do nothing? */
 }
 
@@ -556,7 +556,27 @@ R_API void r_list_sort(RList *list, RListComparator cmp) {
 	}
 }
 
-
+R_API RList *r_list_uniq(const RList *list, RListComparator cmp) {
+	if (!list || !cmp) {
+		return NULL;
+	}
+	RList *nl = r_list_newf (NULL);
+	RListIter *iter, *iter2;
+	void *item, *item2;
+	r_list_foreach (list, iter, item) {
+		bool found = false;
+		r_list_foreach (nl, iter2, item2) {
+			if (cmp (item, item2)) {
+				found = true;
+				break;
+			}
+		}
+		if (!found) {
+			r_list_append (nl, item);
+		}
+	}
+	return nl;
+}
 
 #if TEST
 
@@ -574,11 +594,11 @@ int main () {
 
 	{
 		char *str;
-		r_list_foreach(l, iter, str) {
+		r_list_foreach (l, iter, str) {
 			printf("-- %s\n", str);
 		}
 		printf("--**--\n");
-		r_list_foreach_prev(l, iter, str) {
+		r_list_foreach_prev (l, iter, str) {
 			printf("-- %s\n", str);
 		}
 	}
@@ -663,7 +683,6 @@ int main () {
 	}
 
 	r_list_free (l);
-	//r_list_free (l);
 
 	return 0;
 }
