@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2017 - pancake */
+/* radare - LGPL - Copyright 2007-2018 - pancake */
 
 #include "r_anal.h"
 #include "r_cons.h"
@@ -7,6 +7,8 @@
 
 #define DFLT_ROWS 16
 
+#define IS_ALPHA(C) (((C) >= 'a' && (C) <= 'z') || ((C) >= 'A' && (C) <= 'Z'))
+
 static void nullprinter(const char *a, ...) { }
 static void libc_printf(const char *format, ...) {
 	va_list ap;
@@ -14,7 +16,8 @@ static void libc_printf(const char *format, ...) {
 	vprintf (format, ap);
 	va_end (ap);
 }
-static bool isInterrupted = false;
+
+static RPrintIsInterruptedCallback is_interrupted_cb = NULL;
 
 R_API void r_print_portionbar(RPrint *p, const ut64 *portions, int n_portions) {
 	const int use_color = p->flags & R_PRINT_FLAGS_COLOR;
@@ -108,12 +111,15 @@ R_API int r_util_lines_getline(ut64 *lines_cache, int lines_cache_sz, ut64 off)
 	return imin;
 }
 
-R_API int r_print_is_interrupted() {
-	return isInterrupted;
+R_API bool r_print_is_interrupted() {
+	if (is_interrupted_cb) {
+		return is_interrupted_cb ();
+	}
+	return false;
 }
 
-R_API void r_print_set_interrupted(int i) {
-	isInterrupted = i;
+R_API void r_print_set_is_interrupted_cb(RPrintIsInterruptedCallback cb) {
+	is_interrupted_cb = cb;
 }
 
 R_API bool r_print_mute(RPrint *p, int x) {
@@ -272,6 +278,7 @@ R_API RPrint* r_print_new() {
 	strcpy (p->datefmt, "%Y-%m-%d %H:%M:%S %z");
 	r_io_bind_init (p->iob);
 	p->pairs = true;
+	p->resetbg = true;
 	p->cb_printf = libc_printf;
 	p->oprintf = nullprinter;
 	p->bits = 32;
@@ -292,16 +299,19 @@ R_API RPrint* r_print_new() {
 		R_PRINT_FLAGS_OFFSET |
 		R_PRINT_FLAGS_HEADER |
 		R_PRINT_FLAGS_ADDRMOD;
+	p->seggrn = 4;
 	p->zoom = R_NEW0 (RPrintZoom);
 	p->reg = NULL;
 	p->get_register = NULL;
 	p->get_register_value = NULL;
 	p->lines_cache = NULL;
+	p->calc_row_offsets = true;
 	p->row_offsets_sz = 0;
 	p->row_offsets = NULL;
 	p->vflush = true;
 	p->screen_bounds = 0;
 	p->esc_bslash = false;
+	p->strconv_mode = NULL;
 	memset (&p->consbind, 0, sizeof (p->consbind));
 	return p;
 }
@@ -312,6 +322,7 @@ R_API RPrint* r_print_free(RPrint *p) {
 	}
 	sdb_free (p->formats);
 	p->formats = NULL;
+	R_FREE (p->strconv_mode);
 	if (p->zoom) {
 		free (p->zoom->buf);
 		free (p->zoom);
@@ -378,7 +389,7 @@ R_API void r_print_addr(RPrint *p, ut64 addr) {
 	if (use_segoff) {
 		ut32 s, a;
 		a = addr & 0xffff;
-		s = (addr - a) >> 4;
+		s = (addr - a) >> p->seggrn;
 		if (dec) {
 			snprintf (space, sizeof (space), "%d:%d", s & 0xffff, a & 0xffff);
 			white = r_str_pad (' ', 9 - strlen (space));
@@ -411,7 +422,7 @@ R_API void r_print_addr(RPrint *p, ut64 addr) {
 				// pre = r_cons_rgb_str_off (rgbstr, addr);
 				if (p && p->cons && p->cons->rgbstr) {
 					char rgbstr[32];
-					pre = p->cons->rgbstr (rgbstr, addr);
+					pre = p->cons->rgbstr (rgbstr, sizeof (rgbstr), addr);
 				}
 			}
 			if (dec) {
@@ -471,7 +482,9 @@ R_API char* r_print_hexpair(RPrint *p, const char *str, int n) {
 			if (i == ocur - n) {
 				memcat (d, Color_RESET);
 			}
-			memcat (d, lastcol);
+			if (colors) {
+				memcat (d, lastcol);
+			}
 			if (i >= cur - n && i < ocur - n) {
 				memcat (d, Color_INVERT);
 			}
@@ -507,7 +520,11 @@ R_API char* r_print_hexpair(RPrint *p, const char *str, int n) {
 		}
 	}
 	if (colors || p->cur_enabled) {
-		memcat (d, Color_RESET);
+		if (p->resetbg) {
+			memcat (d, Color_RESET);
+		} else {
+			memcat (d, Color_RESET_NOBG);
+		}
 	}
 	*d = '\0';
 	return dst;
@@ -562,14 +579,10 @@ R_API int r_print_string(RPrint *p, ut64 seek, const ut8 *buf, int len, int opti
 	bool zeroend = (options & R_PRINT_STRING_ZEROEND);
 	bool wrap = (options & R_PRINT_STRING_WRAP);
 	bool urlencode = (options & R_PRINT_STRING_URLENCODE);
+	bool esc_nl = (options & R_PRINT_STRING_ESC_NL);
 	p->interrupt = 0;
 	int col = 0;
 	i = 0;
-	if (!urlencode && !wrap) {
-		while (buf[i] == '\0' && i < 3 && i < len) {
-			i++;
-		}
-	}
 	for (; !p->interrupt && i < len; i++) {
 		if (wide32) {
 			int j = i;
@@ -591,7 +604,7 @@ R_API int r_print_string(RPrint *p, ut64 seek, const ut8 *buf, int len, int opti
 			// TODO: some ascii can be bypassed here
 			p->cb_printf ("%%%02x", b);
 		} else {
-			if (b == '\n' || IS_PRINTABLE (b)) {
+			if ((b == '\n' && !esc_nl) || IS_PRINTABLE (b)) {
 				p->cb_printf ("%c", b);
 			} else {
 				p->cb_printf ("\\x%02x", b);
@@ -805,7 +818,7 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 				if (use_segoff) {
 					ut32 s, a;
 					a = addr & 0xffff;
-					s = ((addr - a) >> 4) & 0xffff;
+					s = ((addr - a) >> p->seggrn) & 0xffff;
 					snprintf (soff, sizeof (soff), "%04x:%04x ", s, a);
 					printfmt ("- offset -");
 				} else {
@@ -867,9 +880,10 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 	if (p) {
 		p->interrupt = 0;
 	}
+	// is this necessary?
+	r_print_set_screenbounds (p, addr);
 	for (i = j = 0; i < len; i += (stride? stride: inc), j += (stride? stride: 0)) {
-		r_print_set_screenbounds (p, addr + i);
-		if (p && p->cons && p->cons->breaked) {
+		if (p && p->cons && p->cons->context && p->cons->context->breaked) {
 			break;
 		}
 		if (use_sparse) {
@@ -935,7 +949,11 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 					r_print_cursor (p, j, 1);
 					// stub for colors
 					if (p && p->colorfor) {
-						a = p->colorfor (p->user, n, true);
+						if (!p->iob.addr_is_mapped (p->iob.io, addr + j)) {
+							a = p->cons->pal.ai_unmap;
+						} else {
+							a = p->colorfor (p->user, n, true);
+						}
 						if (a && *a) {
 							b = Color_RESET;
 						} else {
@@ -1613,9 +1631,26 @@ static bool issymbol(char c) {
 	}
 }
 
-R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, const char *num, bool partial_reset) {
+static bool check_arg_name (RPrint *print, char *p, ut64 func_addr) {
+	if (func_addr && print->exists_var) {
+		int z;
+		for (z = 0; p[z] && (IS_ALPHA (p[z]) || IS_DIGIT (p[z]) || p[z] == '_'); z++);
+		char tmp = p[z];
+		p[z] = '\0';
+		bool ret = print->exists_var (print, func_addr, p);
+		p[z] = tmp;
+		return ret;
+	}
+	return false;
+}
+
+static bool ishexprefix(char *p) {
+	return (p[0] == '0' && p[1] == 'x');
+}
+
+R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, const char *num, bool partial_reset, ut64 func_addr) {
 	int i, j, k, is_mod, is_float = 0, is_arg = 0;
-	char *reset = partial_reset ? Color_NOBGRESET:Color_RESET;
+	char *reset = partial_reset ? Color_RESET_NOBG : Color_RESET;
 	ut32 c_reset = strlen (reset);
 	int is_jmp = p && (*p == 'j' || ((*p == 'c') && (p[1] == 'a')))? 1: 0;
 	ut32 opcode_sz = p && *p? strlen (p) * 10 + 1: 0;
@@ -1635,7 +1670,7 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 	memset (o, 0, COLORIZE_BUFSIZE);
 	for (i = j = 0; p[i]; i++, j++) {
 		/* colorize numbers */
-		if ((p[i] == '0' && p[i+1] == 'x') || (isdigit (p[i]) && issymbol (previous))) {
+		if ((ishexprefix (&p[i]) && previous != ':') || (isdigit (p[i]) && issymbol (previous))) {
 			int nlen = strlen (num);
 			if (nlen + j >= sizeof (o)) {
 				eprintf ("Colorize buffer is too small\n");
@@ -1689,17 +1724,20 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 					eprintf ("r_print_colorize_opcode(): buffer overflow!\n");
 					return strdup (p);
 				}
+
+				bool found_var = check_arg_name (print, p + i + 1, func_addr);
 				strcpy (o + j, reset);
 				j += strlen (reset);
 				o[j] = p[i];
 				if (!(p[i+1] == '$' || ((p[i+1] > '0') && (p[i+1] < '9')))) {
-					ut32 reg_len = strlen (reg);
-					if (reg_len + j + 10 >= COLORIZE_BUFSIZE) {
+					const char *color = found_var ? print->cons->pal.func_var_type : reg;
+					ut32 color_len = strlen (color);
+					if (color_len + j + 10 >= COLORIZE_BUFSIZE) {
 						eprintf ("r_print_colorize_opcode(): buffer overflow!\n");
 						return strdup (p);
 					}
-					strcpy (o + j + 1, reg);
-					j += strlen (reg);
+					strcpy (o + j + 1, color);
+					j += strlen (color);
 				}
 				continue;
 			}
@@ -1783,12 +1821,17 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 
 // reset the status of row_offsets
 R_API void r_print_init_rowoffsets(RPrint *p) {
-	R_FREE (p->row_offsets);
-	p->row_offsets_sz = 0;
+	if (p->calc_row_offsets) {
+		R_FREE(p->row_offsets);
+		p->row_offsets_sz = 0;
+	}
 }
 
 // set the offset, from the start of the printing, of the i-th row
-R_API void r_print_set_rowoff(RPrint *p, int i, ut32 offset) {
+R_API void r_print_set_rowoff(RPrint *p, int i, ut32 offset, bool overwrite) {
+	if (!overwrite) {
+		return;
+	}
 	if (i < 0) {
 		return;
 	}
@@ -1849,23 +1892,27 @@ R_API int r_print_jsondump(RPrint *p, const ut8 *buf, int len, int wordsize) {
 	int i, words = (len / bytesize);
 	p->cb_printf ("[");
 	for (i = 0; i < words; i++) {
-		ut16 w16 = r_read_ble16 (&buf16[i], p->big_endian);
-		ut32 w32 = r_read_ble32 (&buf32[i], p->big_endian);
-		ut64 w64 = r_read_ble64 (&buf64[i], p->big_endian);
 		switch (wordsize) {
-		case 8:
-			p->cb_printf ("%s%d", i?",":"", buf[i]);
+		case 8: {
+			p->cb_printf ("%s%d", i ? "," : "", buf[i]);
 			break;
-		case 16:
-			p->cb_printf ("%s%hd", i?",":"", w16);
+		}
+		case 16: {
+			ut16 w16 = r_read_ble16 (&buf16[i], p->big_endian);
+			p->cb_printf ("%s%hd", i ? "," : "", w16);
 			break;
-		case 32:
-			p->cb_printf ("%s%d", i?",":"", w32);
+		}
+		case 32: {
+			ut32 w32 = r_read_ble32 (&buf32[i], p->big_endian);
+			p->cb_printf ("%s%d", i ? "," : "", w32);
 			break;
-		case 64:
-			p->cb_printf ("%s%"PFMT64d, i?",":"", w64);
+		}
+		case 64: {
+			ut64 w64 = r_read_ble64 (&buf64[i], p->big_endian);
+			p->cb_printf ("%s%"PFMT64d, i ? "," : "", w64);
 			break;
 		}
+		}
 	}
 	p->cb_printf ("]\n");
 	return words;
