@@ -7,12 +7,16 @@
 #include <direct.h>  // to compile chdir under msvc windows
 #endif
 
+#if HAVE_CAPSICUM
+#include <sys/capsicum.h>
+#endif
+
 static bool enabled = false;
 static bool disabled = false;
 
 static bool inHomeWww(const char *path) {
 	bool ret = false;
-	char *homeWww = r_str_home (".config/radare2/www/");
+	char *homeWww = r_str_home (R2_HOME_WWWROOT R_SYS_DIR);
 	if (homeWww) {
 		if (!strncmp (path, homeWww, strlen (homeWww))) {
 			ret = true;
@@ -90,6 +94,12 @@ R_API bool r_sandbox_disable (bool e) {
 			eprintf ("sandbox mode couldn't be disabled when pledged\n");
 			return enabled;
 		}
+#endif
+#if HAVE_CAPSICUM
+		if (enabled) {
+			eprintf ("sandbox mode couldn't be disabled in capability mode\n");
+			return enabled;
+		}
 #endif
 		disabled = enabled;
 		enabled = false;
@@ -112,6 +122,12 @@ R_API bool r_sandbox_enable (bool e) {
 		eprintf ("sandbox: pledge call failed\n");
 		return false;
 	}
+#endif
+#if HAVE_CAPSICUM
+	if (enabled && cap_enter () != 0) {
+		eprintf ("sandbox: call_enter failed\n");
+		return false;
+	}
 #endif
 	return enabled;
 }
