@@ -9,43 +9,75 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdarg.h>
-#ifdef _MSC_VER
-#define strncasecmp _strnicmp
-#endif
 
 /* stable code */
 static const char *nullstr = "";
 static const char *nullstr_c = "(null)";
 static const char *rwxstr[] = {
-	[0] = "----",
-	[1] = "---x",
-	[2] = "--w-",
-	[3] = "--wx",
-	[4] = "-r--",
-	[5] = "-r-x",
-	[6] = "-rw-",
-	[7] = "-rwx",
+	[0] = "---",
+	[1] = "--x",
+	[2] = "-w-",
+	[3] = "-wx",
+	[4] = "r--",
+	[5] = "r-x",
+	[6] = "rw-",
+	[7] = "rwx",
 
-	[8] = "----",
-	[9] = "---x",
-	[10] = "--w-",
-	[11] = "--wx",
-	[12] = "-r--",
-	[13] = "-r-x",
-	[14] = "-rw-",
-	[15] = "-rwx",
-
-	[16] = "m---",
-	[17] = "m--x",
-	[18] = "m-w-",
-	[19] = "m-wx",
-	[20] = "mr--",
-	[21] = "mr-x",
-	[22] = "mrw-",
-	[23] = "mrwx",
+	[8] = "---",
+	[9] = "--x",
+	[10] = "-w-",
+	[11] = "-wx",
+	[12] = "r--",
+	[13] = "r-x",
+	[14] = "rw-",
+	[15] = "rwx",
 };
 
+R_API int r_str_casecmp(const char *s1, const char *s2) {
+#ifdef _MSC_VER
+	return stricmp (s1, s2);
+#else
+	return strcasecmp (s1, s2);
+#endif
+}
+
+R_API int r_str_ncasecmp(const char *s1, const char *s2, size_t n) {
+#ifdef _MSC_VER
+	return _strnicmp (s1, s2, n);
+#else
+	return strncasecmp (s1, s2, n);
+#endif
+}
+
+// GOOD
 // In-place replace the first instance of the character a, with the character b.
+R_API int r_str_replace_ch(char *s, char a, char b, bool global) {
+	int ret = 0;
+	char *o = s;
+	if (!s || a == b) {
+		return 0;
+	}
+	for (; *o; s++, o++) {
+		if (*o == a) {
+			ret++;
+			if (b) {
+				*s = b;
+			} else {
+				/* remove char */
+				s--;
+			}
+			if (!global) {
+				return 1;
+			}
+		} else {
+			*s = *o;
+		}
+	}
+	*s = 0;
+	return ret;
+}
+
+// DEPRECATED
 R_API int r_str_replace_char_once(char *s, int a, int b) {
 	int ret = 0;
 	char *o = s;
@@ -53,7 +85,7 @@ R_API int r_str_replace_char_once(char *s, int a, int b) {
 		return 0;
 	}
 	for (; *o; s++, o++) {
-		if (*o==a) {
+		if (*o == a) {
 			if (b) {
 				*s = b;
 				return ++ret;
@@ -66,8 +98,7 @@ R_API int r_str_replace_char_once(char *s, int a, int b) {
 	return ret;
 }
 
-// Spagetti.. must unify and support 'g', 'i' ...
-// In-place replace all instances of character a with character b.
+// DEPRECATED
 R_API int r_str_replace_char(char *s, int a, int b) {
 	int ret = 0;
 	char *o = s;
@@ -142,7 +173,7 @@ static void trimbits(char *b) {
 R_API int r_str_bits64(char* strout, ut64 in) {
 	int i, bit, count = 0;
 	count = 0;
-	for (i = (sizeof (in) * 8) - 1; i >= 0; --i) {
+	for (i = (sizeof (in) * 8) - 1; i >= 0; i--) {
 		bit = in >> i;
 		if (bit & 1) {
 			strout[count] = '1';
@@ -244,11 +275,13 @@ R_API const char *r_str_bool(int b) {
 R_API void r_str_case(char *str, bool up) {
 	if (up) {
 		char oc = 0;
-		for (; *str; oc = *str++)
+		for (; *str; oc = *str++) {
 			*str = (*str=='x' && oc=='0') ? 'x': toupper ((int)(ut8)*str);
+		}
 	} else {
-		for (; *str; str++)
+		for (; *str; str++) { 
 			*str = tolower ((int)(ut8)*str);
+		}
 	}
 }
 
@@ -279,6 +312,10 @@ fail:
 	return dst;
 }
 
+R_API char *r_str_r2_prefix(const char *str) {
+	return r_str_newf ("%s%s%s", r_sys_prefix (NULL), R_SYS_DIR, str);
+}
+
 // Compute a 64 bit DJB hash of a string.
 R_API ut64 r_str_hash64(const char *s) {
 	ut64 len, h = 5381;
@@ -525,15 +562,15 @@ R_API int r_str_char_count(const char *string, char ch) {
 }
 
 // Counts the number of words (separted by separator charactors: newlines, tabs,
-// return, space). See r_util.h for more details of the ISSEPARATOR macro.
+// return, space). See r_util.h for more details of the IS_SEPARATOR macro.
 R_API int r_str_word_count(const char *string) {
 	const char *text, *tmp;
 	int word;
 
-	for (text = tmp = string; *text && ISSEPARATOR (*text); text++);
+	for (text = tmp = string; *text && IS_SEPARATOR (*text); text++);
 	for (word = 0; *text; word++) {
-		for (;*text && !ISSEPARATOR (*text); text++);
-		for (tmp = text; *text && ISSEPARATOR (*text); text++);
+		for (;*text && !IS_SEPARATOR (*text); text++);
+		for (tmp = text; *text && IS_SEPARATOR (*text); text++);
 	}
 	return word;
 }
@@ -642,6 +679,17 @@ R_API char *r_str_newlen(const char *str, int len) {
 	return buf;
 }
 
+R_API char *r_str_trunc_ellipsis(const char *str, int len) {
+	char *buf;
+	if (strlen (str) < len) {
+		buf = strdup (str);
+	} else {
+		buf = r_str_newlen (str, len);
+		strcpy (buf + len - 4, "...");
+	}
+	return buf;
+}
+
 // Returns a new heap-allocated string that matches the format-string
 // specification.
 R_API char *r_str_newf(const char *fmt, ...) {
@@ -711,6 +759,9 @@ R_API int r_str_cmp(const char *a, const char *b, int len) {
 	if (a == b) {
 		return true;
 	}
+	if (len < 0) {
+		return strcmp (a, b);
+	}
 	for (;len--;) {
 		if (*a == '\0' || *b == '\0' || *a != *b) {
 			return false;
@@ -733,7 +784,7 @@ R_API int r_str_ccpy(char *dst, char *src, int ch) {
 R_API char *r_str_word_get_first(const char *text) {
 	char *ret;
 	int len = 0;
-	for (;*text && ISSEPARATOR (*text); text++);
+	for (;*text && IS_SEPARATOR (*text); text++);
 	/* strdup */
 	len = strlen (text);
 	ret = (char *)malloc (len + 1);
@@ -827,15 +878,12 @@ R_API char *r_str_appendlen(char *ptr, const char *string, int slen) {
  */
 R_API char *r_str_append(char *ptr, const char *string) {
 	int slen, plen;
-	if (!string && !ptr) {
-		return NULL;
-	}
-	if (!string && ptr) {
-		return ptr;
-	}
 	if (string && !ptr) {
 		return strdup (string);
 	}
+	if (!string) {
+		return ptr;
+	}
 	plen = strlen (ptr);
 	slen = strlen (string);
 	char *newptr = realloc (ptr, slen + plen + 1);
@@ -1125,6 +1173,14 @@ static void r_str_byte_escape(const char *p, char **dst, int dot_nl, bool defaul
 		*q++ = '\\';
 		*q++ = 'b';
 		break;
+	case '\v':
+		*q++ = '\\';
+		*q++ = 'v';
+		break;
+	case '\a':
+		*q++ = '\\';
+		*q++ = 'a';
+		break;
 	default:
 		/* Outside the ASCII printable range */
 		if (!IS_PRINTABLE (*p)) {
@@ -1145,7 +1201,7 @@ static void r_str_byte_escape(const char *p, char **dst, int dot_nl, bool defaul
 
 /* Internal function. dot_nl specifies wheter to convert \n into the
  * graphiz-compatible newline \l */
-static char *r_str_escape_(const char *buf, int dot_nl, bool ign_esc_seq, bool show_asciidot, bool esc_bslash) {
+static char *r_str_escape_(const char *buf, int dot_nl, bool parse_esc_seq, bool ign_esc_seq, bool show_asciidot, bool esc_bslash) {
 	char *new_buf, *q;
 	const char *p;
 
@@ -1163,7 +1219,8 @@ static char *r_str_escape_(const char *buf, int dot_nl, bool ign_esc_seq, bool s
 	while (*p) {
 		switch (*p) {
 		case 0x1b: // ESC
-			if (ign_esc_seq) {
+			if (parse_esc_seq) {
+				const char *start_seq = p;
 				p++;
 				/* Parse the ANSI code (only the graphic mode
 				 * set ones are supported) */
@@ -1176,6 +1233,10 @@ static char *r_str_escape_(const char *buf, int dot_nl, bool ign_esc_seq, bool s
 							goto out;
 						}
 					}
+					if (!ign_esc_seq) {
+						memcpy (q, start_seq, p - start_seq + 1);
+						q += (p - start_seq + 1);
+					}
 				}
 				break;
 			}
@@ -1190,15 +1251,15 @@ out:
 }
 
 R_API char *r_str_escape(const char *buf) {
-	return r_str_escape_ (buf, false, true, false, true);
+	return r_str_escape_ (buf, false, true, true, false, true);
 }
 
 R_API char *r_str_escape_dot(const char *buf) {
-	return r_str_escape_ (buf, true, true, false, true);
+	return r_str_escape_ (buf, true, true, true, false, true);
 }
 
-R_API char *r_str_escape_latin1(const char *buf, bool show_asciidot, bool esc_bslash) {
-	return r_str_escape_ (buf, false, false, show_asciidot, esc_bslash);
+R_API char *r_str_escape_latin1(const char *buf, bool show_asciidot, bool esc_bslash, bool colors) {
+	return r_str_escape_ (buf, false, colors, !colors, show_asciidot, esc_bslash);
 }
 
 static char *r_str_escape_utf(const char *buf, int buf_size, RStrEnc enc, bool show_asciidot, bool esc_bslash) {
@@ -1286,30 +1347,21 @@ R_API char *r_str_escape_utf8(const char *buf, bool show_asciidot, bool esc_bsla
 	return r_str_escape_utf (buf, -1, R_STRING_ENC_UTF8, show_asciidot, esc_bslash);
 }
 
-R_API char *r_str_escape_utf16le(const char *buf, int buf_size, bool show_asciidot) {
-	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF16LE, show_asciidot, true);
+R_API char *r_str_escape_utf16le(const char *buf, int buf_size, bool show_asciidot, bool esc_bslash) {
+	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF16LE, show_asciidot, esc_bslash);
 }
 
-R_API char *r_str_escape_utf32le(const char *buf, int buf_size, bool show_asciidot) {
-	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF32LE, show_asciidot, true);
+R_API char *r_str_escape_utf32le(const char *buf, int buf_size, bool show_asciidot, bool esc_bslash) {
+	return r_str_escape_utf (buf, buf_size, R_STRING_ENC_UTF32LE, show_asciidot, esc_bslash);
 }
 
 /* ansi helpers */
 R_API int r_str_ansi_len(const char *str) {
-	int ch, ch2, i = 0, len = 0, sub = 0;
+	int ch, i = 0, len = 0, sub = 0;
 	while (str[i]) {
 		ch = str[i];
-		ch2 = str[i + 1];
-		if (ch == 0x1b) {
-			if (ch2 == '\\') {
-				i++;
-			} else if (ch2 == ']') {
-				if (!strncmp (str + 2 + 5, "rgb:", 4)) {
-					i += 18;
-				}
-			} else if (ch2 == '[') {
-				for (++i; str[i] && str[i] != 'J' && str[i] != 'm' && str[i] != 'H'; i++);
-			}
+		if (ch == 0x1b && str[i + 1] == '[') {
+			for (++i; str[i] && str[i] != 'J' && str[i] != 'm' && str[i] != 'H'; i++);
 		} else {
 			len++;
 #if 0
@@ -1445,7 +1497,10 @@ R_API int r_str_ansi_filter(char *str, char **out, int **cposs, int len) {
 	int i, j, *cps;
 	char *tmp;
 
-	if (len < 1) {
+	if (len == 0) {
+		return 0;
+	}
+	if (len < 0) {
 		len = strlen (str);
 	}
 	tmp = malloc (len + 1);
@@ -1488,13 +1543,13 @@ R_API int r_str_ansi_filter(char *str, char **out, int **cposs, int len) {
 
 R_API char *r_str_ansi_crop(const char *str, ut32 x, ut32 y, ut32 x2, ut32 y2) {
 	char *r, *r_end, *ret;
-	const char *s;
+	const char *s, *s_start;
 	size_t r_len, str_len = 0, nr_of_lines = 0;
 	ut32 ch = 0, cw = 0;
 	if (x2 < 1 || y2 < 1 || !str) {
 		return strdup ("");
 	}
-	s = str;
+	s = s_start = str;
 	while (*s) {
 		str_len++;
 		if (*s == '\n') {
@@ -1504,6 +1559,9 @@ R_API char *r_str_ansi_crop(const char *str, ut32 x, ut32 y, ut32 x2, ut32 y2) {
 	}
 	r_len = str_len + nr_of_lines * strlen (Color_RESET) + 1;
 	r = ret = malloc (r_len);
+	if (!r) {
+		return NULL;
+	}
 	r_end = r + r_len;
 	while (*str) {
 		/* crop height */
@@ -1524,23 +1582,41 @@ R_API char *r_str_ansi_crop(const char *str, ut32 x, ut32 y, ut32 x2, ut32 y2) {
 			str++;
 			ch++;
 			cw = 0;
-		} else if (*str == 0x1b && *(str + 1) == '[') {
-			const char *ptr = str;
-			if ((r_end - r) > 2) {
-				/* copy 0x1b and [ */
-				*r++ = *str++;
-				*r++ = *str++;
-				for (ptr = str; *ptr && *ptr != 'J' && *ptr != 'm' && *ptr != 'H'; ++ptr) {
-					*r++ = *ptr;
-				}
-				*r++ = *ptr++;
-			}
-			str = ptr;
 		} else {
-			if (ch >= y && ch < y2 && cw >= x && cw < x2) {
-				*r++ = *str;
+			if (ch >= y && ch < y2) {
+				if ((*str & 0xc0) == 0x80) {
+					if (cw > x) {
+						*r++ = *str++;
+					} else {
+						str++;
+					}
+					continue;
+				}
+				if (r_str_char_fullwidth (str, str_len - (str - s_start))) {
+					cw++;
+					if (cw == x) {
+						*r++ = ' ';
+						str++;
+						continue;
+					}
+				}
+				if (*str == 0x1b && *(str + 1) == '[') {
+					const char *ptr = str;
+					if ((r_end - r) > 2) {
+						/* copy 0x1b and [ */
+						*r++ = *str++;
+						*r++ = *str++;
+						for (ptr = str; *ptr && *ptr != 'J' && *ptr != 'm' && *ptr != 'H'; ++ptr) {
+							*r++ = *ptr;
+						}
+						*r++ = *ptr++;
+					}
+					str = ptr;
+					continue;
+				} else if (cw >= x && cw < x2) {
+					*r++ = *str;
+				}
 			}
-
 			/* skip until newline */
 			if (cw >= x2) {
 				while (*str && *str != '\n') {
@@ -1556,6 +1632,38 @@ R_API char *r_str_ansi_crop(const char *str, ut32 x, ut32 y, ut32 x2, ut32 y2) {
 	return ret;
 }
 
+R_API int r_str_utf8_codepoint (const char* s, int left) {
+	bool safe = left >= 0;
+	if ((*s & 0x80) != 0x80) {
+		return 0;
+	} else if ((*s & 0xe0) == 0xc0 && (safe ? left >= 1 : *(s + 1))) {
+		return ((*s & 0x1f) << 6) + (*(s + 1) & 0x3f);
+	} else if ((*s & 0xf0) == 0xe0 && (safe ? left >= 2 : (*(s + 1) && *(s + 2)))) {
+		return ((*s & 0xf) << 12) + ((*(s + 1) & 0x3f) << 6) + (*(s + 2) & 0x3f);
+	} else if ((*s & 0xf8) == 0xf0 && (safe ? left >= 3 : (*(s + 1) && *(s + 2) && *(s + 3)))) {
+		return ((*s & 0x7) << 18) + ((*(s + 1) & 0x3f) << 12) + ((*(s + 2) & 0x3f) << 6) + (*(s + 3) & 0x3f);
+	}
+	return 0;
+}
+
+R_API bool r_str_char_fullwidth (const char* s, int left) {
+	int codepoint = r_str_utf8_codepoint (s, left);
+	return (codepoint >= 0x1100 &&
+		 (codepoint <= 0x115f ||                  /* Hangul Jamo init. consonants */
+			  codepoint == 0x2329 || codepoint == 0x232a ||
+		 (R_BETWEEN (0x2e80, codepoint, 0xa4cf)
+			&& codepoint != 0x303f) ||        /* CJK ... Yi */
+		 R_BETWEEN (0xac00, codepoint, 0xd7a3) || /* Hangul Syllables */
+		 R_BETWEEN (0xf900, codepoint, 0xfaff) || /* CJK Compatibility Ideographs */
+		 R_BETWEEN (0xfe10, codepoint, 0xfe19) || /* Vertical forms */
+		 R_BETWEEN (0xfe30, codepoint, 0xfe6f) || /* CJK Compatibility Forms */
+		 R_BETWEEN (0xff00, codepoint, 0xff60) || /* Fullwidth Forms */
+		 R_BETWEEN (0xffe0, codepoint, 0xffe6) ||
+		 R_BETWEEN (0x20000, codepoint, 0x2fffd) ||
+		 R_BETWEEN (0x30000, codepoint, 0x3fffd)));
+
+}
+
 R_API void r_str_filter_zeroline(char *str, int len) {
 	int i;
 	for (i = 0; i < len && str[i]; i++) {
@@ -1691,7 +1799,7 @@ R_API char **r_str_argv(const char *cmdline, int *_argc) {
 		int doublequoted = 0;
 
 		// Seek the beginning of next argument (skip whitespaces)
-		while (cmdline[cmdline_current] != '\0' && ISWHITECHAR (cmdline[cmdline_current])) {
+		while (cmdline[cmdline_current] != '\0' && IS_WHITECHAR (cmdline[cmdline_current])) {
 			cmdline_current++;
 		}
 
@@ -1862,14 +1970,17 @@ R_API int r_str_len_utf8char(const char *s, int left) {
 }
 
 R_API int r_str_len_utf8(const char *s) {
-	int i = 0, j = 0;
+	int i = 0, j = 0, fullwidths = 0;
 	while (s[i]) {
 		if ((s[i] & 0xc0) != 0x80) {
 			j++;
+			if (r_str_char_fullwidth (s + i, 4)) {
+				fullwidths++;
+			}
 		}
 		i++;
 	}
-	return j;
+	return j + fullwidths;
 }
 
 R_API const char *r_str_casestr(const char *a, const char *b) {
@@ -1878,7 +1989,7 @@ R_API const char *r_str_casestr(const char *a, const char *b) {
 	size_t hay_len = strlen (a);
 	size_t needle_len = strlen (b);
 	while (hay_len >= needle_len) {
-		if (!strncasecmp (a, b, needle_len)) {
+		if (!r_str_ncasecmp (a, b, needle_len)) {
 			return (const char *) a;
 		}
 		a++;
@@ -2166,7 +2277,7 @@ R_API int r_print_format_length(const char *fmt) {
 	int viewflags = 0;
 	nargs = endian = i = j = 0;
 
-	while (*arg && ISWHITECHAR (*arg)) {
+	while (*arg && IS_WHITECHAR (*arg)) {
 		arg++;
 	}
 	/* get times */
@@ -2592,6 +2703,12 @@ R_API char *r_str_between(const char *cmt, const char *prefix, const char *suffi
 }
 
 R_API bool r_str_startswith(const char *str, const char *needle) {
+	if (!str || !needle) {
+		return false;
+	}
+	if (str == needle) {
+		return true;
+	}
 	return !strncmp (str, needle, strlen (needle));
 }
 
@@ -2604,17 +2721,28 @@ R_API bool r_str_endswith(const char *str, const char *needle) {
 	return !strcmp (str + (slen - nlen), needle);
 }
 
-R_API RList *r_str_split_list(char *str) {
-	int i, count, *a = r_str_split_lines (str, &count);
-	if (a) {
-		RList *list = r_list_newf (free);
-		for (i = 0; i < count; i++) {
-			r_list_append (list, str + a[i]);
+// Splits the string <str> by string <c> and returns the result in a list.
+R_API RList *r_str_split_list(char *str, const char *c)  {
+	RList *lst = r_list_new ();
+	char *aux;
+	bool first_loop = true;
+
+	for (;;) {
+		if (first_loop) {
+			aux = strtok (str, c);
+			first_loop = false;
+		} else {
+			aux = strtok (NULL, c);
 		}
-		free (a);
-		return list;
+
+		if (!aux) {
+			break;
+		}
+		r_str_trim (aux);
+		r_list_append (lst, aux);
 	}
-	return NULL;
+
+	return lst;
 }
 
 R_API int *r_str_split_lines(char *str, int *count) {
@@ -2708,7 +2836,7 @@ static int strncmp_skip_color_codes(const char *s1, const char *s2, int n) {
 	int count = 0;
 	for (i = 0, j = 0; s1[i]  && s2[j] && count < n; i++, j++, count++) {
 		while (s1[i] == 0x1b) {
-			while (s1[i] && s1[i] != 'm') { 
+			while (s1[i] && s1[i] != 'm') {
 				i++;
 			}
 			if (s1[i]) {
@@ -2738,7 +2866,7 @@ static int strncmp_skip_color_codes(const char *s1, const char *s2, int n) {
 static char *strchr_skip_color_codes(const char *s, int c) {
 	int i = 0;
 	for (i = 0; s[i]; i++) {
-		while (s[i] == 0x1b) {
+		while (s[i] && s[i] == 0x1b) {
 			while (s[i] && s[i] != 'm') {
 				i++;
 			}
@@ -2746,7 +2874,7 @@ static char *strchr_skip_color_codes(const char *s, int c) {
 				i++;
 			}
 		}
-		if (s[i] == (char)c) {
+		if (!s[i] || s[i] == (char)c) {
 			return (char*)s + i;
 		}
 	}
@@ -2755,20 +2883,20 @@ static char *strchr_skip_color_codes(const char *s, int c) {
 
 // Global buffer to speed up colorizing performance
 
-R_API char* r_str_highlight(char *str, const char *word, const char *color) {
+R_API char* r_str_highlight(char *str, const char *word, const char *color, const char *color_reset) {
 	if (!str || !*str) {
 		return NULL;
 	}
 	ut32 i = 0, j = 0, to_copy;
 	char *start = str;
 	ut32 l_str = strlen (str);
-	ut32 l_reset = strlen (Color_BGRESET);
+	ut32 l_reset = strlen (color_reset);
 	ut32 l_color = color? strlen (color): 0;
 	if (!color) {
 		return strdup (str);
 	}
 	if (!word || !*word) {
-		return r_str_newf ("%s%s%s", color, str, Color_BGRESET);
+		return r_str_newf ("%s%s%s", color, str, color_reset);
 	}
 	ut32 l_word = strlen (word);
 	// XXX dont use static buffers
@@ -2800,11 +2928,11 @@ R_API char* r_str_highlight(char *str, const char *word, const char *color) {
 				copied = strncpy_with_color_codes (o + j, str + i, l_word);
 				i += copied;
 				j += copied;
-				if (j + strlen (Color_BGRESET) >= sizeof (o)) {
+				if (j + strlen (color_reset) >= sizeof (o)) {
 					// XXX. no limits
 					break;
 				}
-				strcpy (o + j, Color_BGRESET);
+				strcpy (o + j, color_reset);
 				j += l_reset;
 			} else {
 				o[j++] = str[i++];
@@ -2895,11 +3023,11 @@ R_API wchar_t* r_str_mb_to_wc(const char *buf) {
 R_API char *r_str_from_ut64(ut64 val) {
 	int i = 0;
 	char *v = (char *)&val;
-	char *str = (char *)calloc(1, 65);
+	char *str = (char *)calloc(1, 9);
 	if (!str) {
 		return NULL;
 	}
-	while (i < 64 && *v && *v >= '!' && *v <= '~') {
+	while (i < 8 && *v) {
 		str[i++] = *v++;
 	}
 	return str;
@@ -2913,3 +3041,40 @@ R_API int r_snprintf(char *string, int len, const char *fmt, ...) {
 	va_end (ap);
 	return ret;
 }
+
+// Strips all the lines in str that contain key
+R_API void r_str_stripLine(char *str, const char *key) {
+	size_t i, j, klen, slen, off;
+	const char *ptr; 
+
+	if (!str || !key) {
+		return;
+	}
+	klen = strlen (key);
+	slen = strlen (str);
+
+	for (i = 0; i < slen; ) {
+		ptr = (char*) r_mem_mem ((ut8*) str + i, slen - i, (ut8*) "\n", 1);
+		if (!ptr) {
+			ptr = (char*) r_mem_mem ((ut8*) str + i, slen - i, (ut8*) key, klen);
+			if (ptr) {
+				str[i] = '\0';
+				break;
+			}
+			break;
+		}
+			
+		off = (size_t) (ptr - (str + i)) + 1;
+
+		ptr = (char*) r_mem_mem ((ut8*) str + i, off, (ut8*) key, klen);
+		if (ptr) {
+			for (j = i; j < slen - off + 1; j++) {
+				str[j] = str[j + off];
+			}
+			slen -= off;
+		} else {
+			i += off;
+		}
+	}
+}
+
