@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2017 - pancake */
+/* radare - LGPL - Copyright 2009-2018 - pancake */
 
 #include <r_userconf.h>
 #include <stdlib.h>
@@ -9,16 +9,32 @@
 #  define NETBSD_WITH_BACKTRACE
 # endif
 #endif
+#if defined(__FreeBSD__)
+# include <sys/param.h>
+# if __FreeBSD_version >= 1000000 
+#  define FREEBSD_WITH_BACKTRACE
+# endif
+#endif
 #include <sys/types.h>
 #include <r_types.h>
 #include <r_util.h>
 #include <r_lib.h>
-#if (__linux__ && __GNU_LIBRARY__) || defined(NETBSD_WITH_BACKTRACE)
+
+static char** env = NULL;
+
+#if (__linux__ && __GNU_LIBRARY__) || defined(NETBSD_WITH_BACKTRACE) || \
+  defined(FREEBSD_WITH_BACKTRACE)
 # include <execinfo.h>
 #endif
 #if __APPLE__
 #include <errno.h>
-#if !__POWERPC__
+#ifdef __MAC_10_8
+#define HAVE_ENVIRON 1
+#else
+#define HAVE_ENVIRON 0
+#endif
+
+#if HAVE_ENVIRON
 #include <execinfo.h>
 #endif
 // iOS dont have this we cant hardcode
@@ -45,16 +61,17 @@ extern char **environ;
 #if __WINDOWS__ && !defined(__CYGWIN__)
 # include <io.h>
 # include <winbase.h>
+#define TMP_BUFSIZE	4096
 #ifdef _MSC_VER
 #include <psapi.h>
 #include <io.h>
 #include <process.h>  // to allow getpid under windows msvc compilation
 #include <direct.h>  // to allow getcwd under windows msvc compilation
 #else
-typedef BOOL WINAPI (*QueryFullProcessImageNameA_t) (HANDLE, DWORD, LPTSTR, PDWORD);
-typedef DWORD WINAPI (*GetProcessImageFileNameA_t) (HANDLE, LPTSTR, DWORD);
-static GetProcessImageFileNameA_t GetProcessImageFileNameA;
-static QueryFullProcessImageNameA_t QueryFullProcessImageNameA;
+typedef BOOL WINAPI (*QueryFullProcessImageName_t) (HANDLE, DWORD, LPTSTR, PDWORD);
+typedef DWORD WINAPI (*GetProcessImageFileName_t) (HANDLE, LPTSTR, DWORD);
+GetProcessImageFileName_t GetProcessImageFileName;
+QueryFullProcessImageName_t QueryFullProcessImageName;
 #endif
 #endif
 
@@ -85,6 +102,7 @@ static const struct {const char* name; ut64 bit;} arch_bit_array[] = {
     {"i8080", R_SYS_ARCH_I8080},
     {"rar", R_SYS_ARCH_RAR},
     {"lm32", R_SYS_ARCH_LM32},
+    {"v850", R_SYS_ARCH_V850},
     {NULL, 0}
 };
 
@@ -189,7 +207,8 @@ R_API char *r_sys_cmd_strf(const char *fmt, ...) {
 #define APPLE_WITH_BACKTRACE 1
 #endif
 
-#if (__linux__ && __GNU_LIBRARY__) || (__APPLE__ && APPLE_WITH_BACKTRACE) || defined(NETBSD_WITH_BACKTRACE)
+#if (__linux__ && __GNU_LIBRARY__) || (__APPLE__ && APPLE_WITH_BACKTRACE) || \
+  defined(NETBSD_WITH_BACKTRACE) || defined(FREEBSD_WITH_BACKTRACE)
 #define HAVE_BACKTRACE 1
 #endif
 
@@ -247,9 +266,19 @@ R_API int r_sys_usleep(int usecs) {
 }
 
 R_API int r_sys_clearenv(void) {
-#if __UNIX__ || __CYGWIN__ && !defined(MINGW32)
-#if __APPLE__ && __POWERPC__
+#if __UNIX__ || (__CYGWIN__ && !defined(MINGW32))
+#if __APPLE__ && !HAVE_ENVIRON
 	/* do nothing */
+	if (!env) {
+		env = r_sys_get_environ ();
+		return 0;
+	}
+	if (env) {
+		char **e = env;
+		while (*e) {
+			*e++ = NULL;
+		}
+	}
 #else
 	if (!environ) {
 		return 0;
@@ -280,7 +309,12 @@ R_API int r_sys_setenv(const char *key, const char *value) {
 	}
 	return setenv (key, value, 1);
 #elif __WINDOWS__
-	SetEnvironmentVariableA (key, (LPSTR)value);
+	LPTSTR key_ = r_sys_conv_utf8_to_utf16 (key);
+	LPTSTR value_ = r_sys_conv_utf8_to_utf16 (value);
+
+	SetEnvironmentVariable (key_, value_);
+	free (key_);
+	free (value_);
 	return 0; // TODO. get ret
 #else
 #warning r_sys_setenv : unimplemented for this platform
@@ -288,9 +322,9 @@ R_API int r_sys_setenv(const char *key, const char *value) {
 #endif
 }
 
+#if __UNIX__
 static char *crash_handler_cmd = NULL;
 
-#if __UNIX__
 static void signal_handler(int signum) {
 	char cmd[1024];
 	if (!crash_handler_cmd) {
@@ -318,11 +352,11 @@ static int checkcmd(const char *c) {
 R_API int r_sys_crash_handler(const char *cmd) {
 #if __UNIX__
 	struct sigaction sigact;
-	void *array[1];
 	if (!checkcmd (cmd)) {
 		return false;
 	}
 #ifdef HAVE_BACKTRACE
+	void *array[1];
 	/* call this outside of the signal handler to init it safely */
 	backtrace (array, 1);
 #endif
@@ -356,23 +390,38 @@ R_API int r_sys_crash_handler(const char *cmd) {
 
 R_API char *r_sys_getenv(const char *key) {
 #if __WINDOWS__ && !__CYGWIN__
-	static char envbuf[4096];
 	DWORD dwRet;
+	LPTSTR envbuf = NULL, key_ = NULL;
+	char *val = NULL;
 
 	if (!key) {
 		return NULL;
 	}
-	dwRet = GetEnvironmentVariableA (key, (LPSTR)&envbuf, sizeof (envbuf));
+	envbuf = (LPTSTR)malloc (sizeof (TCHAR) * TMP_BUFSIZE);
+	if (!envbuf) {
+		goto err_r_sys_get_env;
+	}
+	key_ = r_sys_conv_utf8_to_utf16 (key);
+	dwRet = GetEnvironmentVariable (key_, envbuf, TMP_BUFSIZE);
 	if (dwRet == 0) {
-		/* Variable not found. */
-		return NULL;
+		if (GetLastError () == ERROR_ENVVAR_NOT_FOUND) {
+			goto err_r_sys_get_env;
+		}
+	} else if (TMP_BUFSIZE < dwRet) {
+		envbuf = (LPTSTR)realloc (envbuf, dwRet * sizeof (TCHAR));
+		if (!envbuf) {
+			goto err_r_sys_get_env;
+		}
+		dwRet = GetEnvironmentVariable (key_, envbuf, dwRet);
+		if (!dwRet) {
+			goto err_r_sys_get_env;
+		}
 	}
-	if (dwRet == sizeof(envbuf)) {
-		/* The contents of envbuf are undefined, so return NULL */
-		eprintf ("Buffer too small to read `%s' environment variable.\n", key);
-		return NULL;
-	}
-	return strdup (envbuf);
+	val = r_sys_conv_utf16_to_utf8_l (envbuf, (int)dwRet);
+err_r_sys_get_env:
+	free (key_);
+	free (envbuf);
+	return val;
 #else
 	char *b;
 	if (!key) {
@@ -384,17 +433,11 @@ R_API char *r_sys_getenv(const char *key) {
 }
 
 R_API char *r_sys_getdir(void) {
-	char *ret;
 #if __WINDOWS__ && !__CYGWIN__
-	char *cwd = _getcwd (NULL, 0);
+	return _getcwd (NULL, 0);
 #else
-	char *cwd = getcwd (NULL, 0);
+	return getcwd (NULL, 0);
 #endif
-	ret = cwd ? strdup (cwd) : NULL; 
-	if (cwd) {
-		free (cwd);
-	}
-	return ret;
 }
 
 R_API int r_sys_chdir(const char *s) {
@@ -403,6 +446,10 @@ R_API int r_sys_chdir(const char *s) {
 
 #if __UNIX__ || __CYGWIN__ && !defined(MINGW32)
 R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output, int *len, char **sterr) {
+	char *mysterr = NULL;
+	if (!sterr) {
+		sterr = &mysterr;
+	}
 	char buffer[1024], *outputptr = NULL;
 	char *inputptr = (char *)input;
 	int pid, bytes = 0, status;
@@ -494,7 +541,7 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output,
 				if (!(bytes = read (sh_out[0], buffer, sizeof (buffer)-1))) {
 					break;
 				}
-				buffer[sizeof(buffer) - 1] = '\0';
+				buffer[sizeof (buffer) - 1] = '\0';
 				if (len) {
 					*len += bytes;
 				}
@@ -503,7 +550,7 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output,
 				if (!read (sh_err[0], buffer, sizeof (buffer)-1)) {
 					break;
 				}
-				buffer[sizeof(buffer) - 1] = '\0';
+				buffer[sizeof (buffer) - 1] = '\0';
 				*sterr = r_str_append (*sterr, buffer);
 			} else if (FD_ISSET (sh_in[1], &wfds) && inputptr && *inputptr) {
 				int inputptr_len = strlen (inputptr);
@@ -530,10 +577,11 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output,
 		waitpid (pid, &status, 0);
 		bool ret = true;
 		if (status) {
-			char *escmd = r_str_escape (cmd);
-			eprintf ("error code %d\n", WEXITSTATUS(status));
-			//eprintf ("%s: failed command '%s'\n", __func__, escmd);
-			free (escmd);
+			// char *escmd = r_str_escape (cmd);
+			// eprintf ("error code %d (%s): %s\n", WEXITSTATUS (status), escmd, *sterr);
+			// eprintf ("(%s)\n", output);
+			// eprintf ("%s: failed command '%s'\n", __func__, escmd);
+			// free (escmd);
 			ret = false;
 		}
 
@@ -602,7 +650,7 @@ R_API int r_sys_cmdbg (const char *str) {
 #endif
 }
 
-R_API int r_sys_cmd (const char *str) {
+R_API int r_sys_cmd(const char *str) {
 	if (r_sandbox_enable (0)) {
 		return false;
 	}
@@ -641,14 +689,20 @@ R_API char *r_sys_cmd_str(const char *cmd, const char *input, int *len) {
 }
 
 R_API bool r_sys_mkdir(const char *dir) {
+	bool ret;
+
 	if (r_sandbox_enable (0)) {
 		return false;
 	}
 #if __WINDOWS__ && !defined(__CYGWIN__)
-	return CreateDirectoryA (dir, NULL) != 0;
+	LPTSTR dir_ = r_sys_conv_utf8_to_utf16 (dir);
+
+	ret = CreateDirectory (dir_, NULL) != 0;
+	free (dir_);
 #else
-	return mkdir (dir, 0755) != -1;
+	ret = mkdir (dir, 0755) != -1;
 #endif
+	return ret;
 }
 
 R_API bool r_sys_mkdirp(const char *dir) {
@@ -704,26 +758,22 @@ R_API void r_sys_perror_str(const char *fun) {
 	perror (fun);
 #pragma pop_macro("perror")
 #elif __WINDOWS__
-	char *lpMsgBuf;
-	LPVOID lpDisplayBuf;
-	DWORD dw = GetLastError ();
+	LPTSTR lpMsgBuf;
+	DWORD dw = GetLastError();
 
-	FormatMessage ( FORMAT_MESSAGE_ALLOCATE_BUFFER |
+	if (FormatMessage ( FORMAT_MESSAGE_ALLOCATE_BUFFER |
 			FORMAT_MESSAGE_FROM_SYSTEM |
 			FORMAT_MESSAGE_IGNORE_INSERTS,
 			NULL,
 			dw,
 			MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
-			(LPTSTR) &lpMsgBuf,
-			0, NULL );
-
-	lpDisplayBuf = (LPVOID)LocalAlloc (LMEM_ZEROINIT,
-			(lstrlen ((LPCTSTR)lpMsgBuf)+
-			lstrlen ((LPCTSTR)fun) + 40) * sizeof (TCHAR));
-	eprintf ("%s: %s\n", fun, lpMsgBuf);
-
-	LocalFree (lpMsgBuf);
-	LocalFree (lpDisplayBuf);
+			(LPTSTR)&lpMsgBuf,
+			0, NULL )) {
+		eprintf ("%s: " W32_TCHAR_FSTR "\n", fun, lpMsgBuf);
+		LocalFree (lpMsgBuf);
+	} else {
+		eprintf ("%s\n", fun);
+	}
 #endif
 }
 
@@ -834,50 +884,50 @@ R_API int r_is_heap (void *p) {
 
 R_API char *r_sys_pid_to_path(int pid) {
 #if __WINDOWS__
-	HANDLE kernel32 = LoadLibraryA ("Kernel32.dll");
+	HANDLE kernel32 = GetModuleHandle (TEXT("kernel32"));
 	if (!kernel32) {
-		eprintf ("Error getting the handle to Kernel32.dll\n");
+		eprintf ("Error getting the handle to kernel32.dll\n");
 		return NULL;
 	}
 #ifndef _MSC_VER
-	if (!GetProcessImageFileNameA) {
-		if (!QueryFullProcessImageNameA) {
-			QueryFullProcessImageNameA = (QueryFullProcessImageNameA_t) GetProcAddress (kernel32, "QueryFullProcessImageNameA");
+	if (!GetProcessImageFileName) {
+		if (!QueryFullProcessImageName) {
+			QueryFullProcessImageName = (QueryFullProcessImageName_t) GetProcAddress (kernel32, W32_TCALL ("QueryFullProcessImageName"));
 		}
-		if (!QueryFullProcessImageNameA) {
+		if (!QueryFullProcessImageName) {
 			// QueryFullProcessImageName does not exist before Vista, fallback to GetProcessImageFileName
-			HANDLE psapi = LoadLibrary ("Psapi.dll");
+			HANDLE psapi = LoadLibrary (TEXT("Psapi.dll"));
 			if (!psapi) {
 				eprintf ("Error getting the handle to Psapi.dll\n");
 				return NULL;
 			}
-			GetProcessImageFileNameA = (GetProcessImageFileNameA_t) GetProcAddress (psapi, "GetProcessImageFileNameA");
-			if (!GetProcessImageFileNameA) {
-				eprintf ("Error getting the address of GetProcessImageFileNameA\n");
+			GetProcessImageFileName = (GetProcessImageFileName_t) GetProcAddress (psapi, W32_TCALL ("GetProcessImageFileName"));
+			if (!GetProcessImageFileName) {
+				eprintf ("Error getting the address of GetProcessImageFileName\n");
 				return NULL;
 			}
 		}
 	}
 	HANDLE handle = NULL;
-	CHAR filename[MAX_PATH];
+	TCHAR filename[MAX_PATH];
 	DWORD maxlength = MAX_PATH;
 	handle = OpenProcess (PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, pid);
 	if (handle != NULL) {
-		if (QueryFullProcessImageNameA) {
-			if (QueryFullProcessImageNameA (handle, 0, filename, &maxlength) == 0) {
-				eprintf ("Error calling QueryFullProcessImageNameA\n");
+		if (QueryFullProcessImageName) {
+			if (QueryFullProcessImageName (handle, 0, filename, &maxlength) == 0) {
+				eprintf ("Error calling QueryFullProcessImageName\n");
 				CloseHandle (handle);
 				return NULL;
 			}
 		} else {
-			if (GetProcessImageFileNameA (handle, filename, maxlength) == 0) {
-				eprintf ("Error calling GetProcessImageFileNameA\n");
+			if (GetProcessImageFileName (handle, filename, maxlength) == 0) {
+				eprintf ("Error calling GetProcessImageFileName\n");
 				CloseHandle (handle);
 				return NULL;
 			}
 		}
 		CloseHandle (handle);
-		return strdup (filename);
+		return r_sys_conv_utf16_to_utf8 (filename);
 	}
 	return NULL;
 #else
@@ -927,11 +977,12 @@ R_API char *r_sys_pid_to_path(int pid) {
 #endif
 }
 
-static char** env = NULL;
-
+// TODO: rename to r_sys_env_init()
 R_API char **r_sys_get_environ () {
-#if __APPLE__ && !__POWERPC__
+#if __APPLE__ && !HAVE_ENVIRON
 	env = *_NSGetEnviron();
+#else
+	env = environ;
 #endif
 	// return environ if available??
 	if (!env) {
@@ -983,3 +1034,18 @@ R_API bool r_sys_tts(const char *txt, bool bg) {
 	}
 	return false;
 }
+
+static char prefix[128] = {0};
+
+R_API const char *r_sys_prefix(const char *pfx) {
+	if (!*prefix) {
+		r_str_ncpy (prefix, R2_PREFIX, sizeof (prefix));
+	}
+	if (pfx) {
+		if (strlen (pfx) >= sizeof (prefix) -1) {
+			return NULL;
+		}
+		r_str_ncpy (prefix, pfx, sizeof (prefix) - 1);
+	}
+	return prefix;
+}
