@@ -72,7 +72,9 @@ static char *showfile(char *res, const int nth, const char *fpath, const char *n
 	u_rwx = strdup ("-");
 	fch = isdir? 'd': '-';
 #endif
-	if (printfmt == FMT_RAW) {
+	if (printfmt == 'q') {
+		res = r_str_appendf (res, "%s\n", nn);
+	} else if (printfmt == FMT_RAW) {
 		res = r_str_appendf (res, "%c%s%s%s  1 %4d:%-4d  %-10d  %s\n",
 			isdir?'d': fch,
 			u_rwx? u_rwx: "-",
@@ -85,7 +87,7 @@ static char *showfile(char *res, const int nth, const char *fpath, const char *n
 		}
 		res = r_str_appendf (res, "{\"name\":\"%s\",\"size\":%d,\"uid\":%d,"
 			"\"gid\":%d,\"perm\":%d,\"isdir\":%s}",
-			name, sz, uid, gid, perm, isdir?"true":"false");
+			name, sz, uid, gid, perm, isdir? "true": "false");
 	}
 	free (nn);
 	free (u_rwx);
@@ -110,6 +112,10 @@ R_API char *r_syscmd_ls(const char *input) {
 		input = "";
 		path = ".";
 	}
+	if (*input == 'q') {
+		printfmt = 'q';
+		input++;
+	}
 	if (r_sandbox_enable (0)) {
 		eprintf ("Sandbox forbids listing directories\n");
 		return NULL;
@@ -118,7 +124,15 @@ R_API char *r_syscmd_ls(const char *input) {
 		input++;
 	}
 	if (*input) {
-		if ((!strncmp (input, "-l", 2)) || (!strncmp (input, "-j", 2))) {
+		if ((!strncmp (input, "-h", 2))) {
+			eprintf ("Usage: ls ([-l,-j,-q]) ([path]) # long, json, quiet\n");
+		} else if ((!strncmp (input, "-q", 2))) {
+			printfmt = 'q';
+			path ++;
+			while (*path == ' ') {
+				path++;
+			}
+		} else if ((!strncmp (input, "-l", 2)) || (!strncmp (input, "-j", 2))) {
 			// mode = 'l';
 			if (input[2]) {
 				printfmt = (input[2] == 'j') ? FMT_JSON : FMT_RAW;
@@ -138,14 +152,14 @@ R_API char *r_syscmd_ls(const char *input) {
 	}
 	if (!path || !*path) {
 		path = ".";
-	} else if (*path == '~') {
+	} else if (!strncmp (path, "~/", 2)) {
 		homepath = r_str_home (path + 2);
 		if (homepath) {
 			path = (const char *)homepath;
 		}
 	} else if (*path == '$') {
-		if (!strncmp (path + 1, "home", 4) || !strncmp (path + 1, "HOME", 4)){
-			homepath = r_str_home ((strlen (path)>5)? path + 6: NULL);
+		if (!strncmp (path + 1, "home", 4) || !strncmp (path + 1, "HOME", 4)) {
+			homepath = r_str_home ((strlen (path) > 5)? path + 6: NULL);
 			if (homepath) {
 				path = (const char *)homepath;
 			}
@@ -153,7 +167,7 @@ R_API char *r_syscmd_ls(const char *input) {
 	}
 	if (!r_file_is_directory (path)) {
 		p = strrchr (path, '/');
-		if (p){
+		if (p) {
 			off = p - path;
 			d = (char *) calloc (1, off + 1);
 			if (!d) {
@@ -242,29 +256,27 @@ R_API char *r_syscmd_cat(const char *file) {
 }
 
 R_API char *r_syscmd_mkdir(const char *dir) {
-	bool show_help = true;
-	const char *p = strchr (dir, ' ');
-	if (p) {
-		int ret;
-		char *dirname;
-		if (!strncmp (p + 1, "-p ", 3)) {
-			dirname = r_str_trim (strdup (p + 3));
-			ret = r_sys_mkdirp (dirname);
-		} else {
-			dirname = r_str_trim (strdup (p + 1));
-			ret = r_sys_mkdir (dirname);
-		}
-		if (!ret) {
-			if (r_sys_mkdir_failed ()) {
-				eprintf ("Cannot create \"%s\"\n", dirname);
-			}
-		}
-		free (dirname);
-		show_help = false;
+	const char *suffix = r_str_trim (strchr (dir, ' '));
+	if (!suffix || !strncmp (suffix, "-p", 3)) {
+		return r_str_dup (NULL, "Usage: mkdir [-p] [directory]\n");
 	}
-	if (show_help) {
-		eprintf ("Usage: mkdir [-p] [directory]\n");
+	int ret;
+	char *dirname;
+	if (!strncmp (suffix, "-p ", 3)) {
+		dirname = r_str_trim (strdup (suffix + 3));
+		ret = r_sys_mkdirp (dirname);
+	} else {
+		dirname = r_str_trim (strdup (suffix));
+		ret = r_sys_mkdir (dirname);
 	}
+	if (!ret) {
+		if (r_sys_mkdir_failed ()) {
+			char *res = r_str_newf ("Cannot create \"%s\"\n", dirname);
+			free (dirname);
+			return res;
+		}
+	}
+	free (dirname);
 	return NULL;
 }
 
