@@ -4,13 +4,7 @@ CS_BRA="$2" # branch name
 CS_TIP="$3" # tip commit
 CS_REV="$4" # revert
 
-if [ -d capstone -a ! -d capstone/.git ]; then 
-	echo "[capstone] release with no git?"
-	cd capstone
-	for PATCH in ../capstone-patches/* ; do
-		yes n | patch -Rp1 < $PATCH
-	done
-else 
+clone_capstone() {
 	if [ ! -d capstone ]; then 
 		git clone -b "${CS_BRA}" --depth 10 "${CS_URL}" capstone || exit 1
 	fi
@@ -20,11 +14,31 @@ else
 	else
 		HEAD="`git log|head -n1 | awk '{print $2}'`"
 	fi
+	BRANCH="`git branch | grep '*' | cut -d'*' -f2 | cut -d' ' -f2`"
+
 	if [ "${HEAD}" = "${CS_TIP}" ]; then
 		echo "[capstone] Already in TIP, no need to update from git"
 		exit 0
 	fi
-		echo "[capstone] Updating capstone from git..."
+}
+
+if [ -d capstone -a ! -d capstone/.git ]; then
+	echo "[capstone] release with no git?"
+	cd capstone
+	for PATCH in ../capstone-patches/* ; do
+		yes n | patch -Rp1 < $PATCH
+	done
+else
+	clone_capstone
+
+	if [ "${BRANCH}" != "${CS_BRA}" ]; then
+	    echo "[capstone] Reset capstone"
+	    cd ..
+	    rm -rf capstone
+	    clone_capstone
+	fi
+
+	echo "[capstone] Updating capstone from git..."
 	echo "HEAD ${HEAD}"
 	echo "TIP ${CS_TIP}"
 
