@@ -1,4 +1,4 @@
-/* libgdbr - LGPL - Copyright 2014-2017 - defragger */
+/* libgdbr - LGPL - Copyright 2014-2018 - defragger */
 
 #include "gdbclient/responses.h"
 #include "gdbclient/commands.h"
@@ -23,6 +23,7 @@
 
 extern char hex2char(char *hex);
 
+#if 0
 static int set_interface_attribs (int fd, int speed, int parity) {
 #if defined(_MSC_VER) || defined(__MINGW32__)
 #pragma message("gdbclient/core.c: set_interface_attribs not implemented")
@@ -61,6 +62,7 @@ static int set_interface_attribs (int fd, int speed, int parity) {
 #endif
 	return 0;
 }
+#endif
 
 static struct {
 	ut8  *buf;
@@ -105,7 +107,7 @@ int gdbr_connect(libgdbr_t *g, const char *host, int port) {
 	// Initial max_packet_size for remote target (minimum so far for AVR = 64)
 	g->stub_features.pkt_sz = 64;
 	char *env_pktsz_str;
-	ut32 env_pktsz;
+	ut32 env_pktsz = 0;
 	if ((env_pktsz_str = r_sys_getenv ("R2_GDB_PKTSZ"))) {
 		if ((env_pktsz = (ut32) strtoul (env_pktsz_str, NULL, 10))) {
 			g->stub_features.pkt_sz = R_MAX (env_pktsz, GDB_MAX_PKTSZ);
@@ -477,7 +479,7 @@ static int gdbr_read_registers_lldb(libgdbr_t *g) {
 
 int gdbr_read_registers(libgdbr_t *g) {
 	int ret = -1;
-	if (!g) {
+	if (!g || !g->data) {
 		return -1;
 	}
 	if (reg_cache.init && reg_cache.valid) {
@@ -511,6 +513,9 @@ static int gdbr_read_memory_page(libgdbr_t *g, ut64 address, ut8 *buf, int len)
 	if (!g) {
 		return -1;
 	}
+	if (len < 1) {
+		return len;
+	}
 	g->stub_features.pkt_sz = R_MAX (g->stub_features.pkt_sz, GDB_MAX_PKTSZ);
 	int data_sz = g->stub_features.pkt_sz / 2;
 	int num_pkts = len / data_sz;
@@ -589,7 +594,7 @@ int gdbr_read_memory(libgdbr_t *g, ut64 address, ut8 *buf, int len) {
 	// Read complete pages
 	while (len > page_size) {
 		if ((ret = gdbr_read_memory_page (g, address, buf, page_size)) != page_size) {
-			if (ret < 0) {
+			if (ret < 1) {
 				return ret_len;
 			}
 			return ret_len + ret;
@@ -960,7 +965,7 @@ int send_vcont(libgdbr_t *g, const char *command, const char *thread_id) {
 	return handle_cont (g);
 }
 
-int set_bp(libgdbr_t *g, ut64 address, const char *conditions, enum Breakpoint type) {
+int set_bp(libgdbr_t *g, ut64 address, const char *conditions, enum Breakpoint type, int sizebp) {
 	char tmp[255] = {0};
 	int ret = -1;
 	if (!g) {
@@ -969,17 +974,23 @@ int set_bp(libgdbr_t *g, ut64 address, const char *conditions, enum Breakpoint t
 	switch (type) {
 	case BREAKPOINT:
 		ret = snprintf (tmp, sizeof (tmp) - 1,
-			"%s,%"PFMT64x ",1", CMD_BP, address);
+			"%s,%"PFMT64x ",%d", CMD_BP, address, sizebp);
 		break;
 	case HARDWARE_BREAKPOINT:
 		ret = snprintf (tmp, sizeof (tmp) - 1,
-			"%s,%"PFMT64x ",1", CMD_HBP, address);
+			"%s,%"PFMT64x ",%d", CMD_HBP, address, sizebp);
 		break;
 	case WRITE_WATCHPOINT:
+		ret = snprintf (tmp, sizeof (tmp) - 1,
+			"%s,%"PFMT64x ",%d", CMD_HWW, address, sizebp);
 		break;
 	case READ_WATCHPOINT:
+		ret = snprintf (tmp, sizeof (tmp) - 1,
+			"%s,%"PFMT64x ",%d", CMD_HWR, address, sizebp);
 		break;
 	case ACCESS_WATCHPOINT:
+		ret = snprintf (tmp, sizeof (tmp) - 1,
+			"%s,%"PFMT64x ",%d", CMD_HWA, address, sizebp);
 		break;
 	default:
 		break;
@@ -999,23 +1010,48 @@ int set_bp(libgdbr_t *g, ut64 address, const char *conditions, enum Breakpoint t
 	return 0;
 }
 
-int gdbr_set_bp(libgdbr_t *g, ut64 address, const char *conditions) {
-	return set_bp (g, address, conditions, BREAKPOINT);
+int gdbr_set_bp(libgdbr_t *g, ut64 address, const char *conditions, int sizebp) {
+	return set_bp (g, address, conditions, BREAKPOINT, sizebp);
 }
 
-int gdbr_set_hwbp(libgdbr_t *g, ut64 address, const char *conditions) {
-	return set_bp (g, address, conditions, HARDWARE_BREAKPOINT);
+int gdbr_set_hwbp(libgdbr_t *g, ut64 address, const char *conditions, int sizebp) {
+	return set_bp (g, address, conditions, HARDWARE_BREAKPOINT, sizebp);
 }
 
-int gdbr_remove_bp(libgdbr_t *g, ut64 address) {
-	return remove_bp (g, address, BREAKPOINT);
+int gdbr_set_hww(libgdbr_t *g, ut64 address, const char *conditions, int sizebp) {
+	return set_bp (g, address, conditions, WRITE_WATCHPOINT, sizebp);
 }
 
-int gdbr_remove_hwbp(libgdbr_t *g, ut64 address) {
-	return remove_bp (g, address, HARDWARE_BREAKPOINT);
+int gdbr_set_hwr(libgdbr_t *g, ut64 address, const char *conditions, int sizebp) {
+	return set_bp (g, address, conditions, READ_WATCHPOINT, sizebp);
 }
 
-int remove_bp(libgdbr_t *g, ut64 address, enum Breakpoint type) {
+int gdbr_set_hwa(libgdbr_t *g, ut64 address, const char *conditions, int sizebp) {
+	return set_bp (g, address, conditions, ACCESS_WATCHPOINT, sizebp);
+}
+
+int gdbr_remove_bp(libgdbr_t *g, ut64 address, int sizebp) {
+	return remove_bp (g, address, BREAKPOINT, sizebp);
+}
+
+int gdbr_remove_hwbp(libgdbr_t *g, ut64 address, int sizebp) {
+	return remove_bp (g, address, HARDWARE_BREAKPOINT, sizebp);
+}
+
+int gdbr_remove_hww(libgdbr_t *g, ut64 address, int sizebp) {
+	return remove_bp (g, address, WRITE_WATCHPOINT, sizebp);
+}
+
+int gdbr_remove_hwr(libgdbr_t *g, ut64 address, int sizebp) {
+	return remove_bp (g, address, READ_WATCHPOINT, sizebp);
+}
+
+int gdbr_remove_hwa(libgdbr_t *g, ut64 address, int sizebp) {
+	return remove_bp (g, address, ACCESS_WATCHPOINT, sizebp);
+}
+
+
+int remove_bp(libgdbr_t *g, ut64 address, enum Breakpoint type, int sizebp) {
 	char tmp[255] = {0};
 	int ret = -1;
 	if (!g) {
@@ -1023,16 +1059,19 @@ int remove_bp(libgdbr_t *g, ut64 address, enum Breakpoint type) {
 	}
 	switch (type) {
 	case BREAKPOINT:
-		ret = snprintf (tmp, sizeof (tmp) - 1, "%s,%"PFMT64x ",1", CMD_RBP, address);
+		ret = snprintf (tmp, sizeof (tmp) - 1, "%s,%"PFMT64x ",%d", CMD_RBP, address, sizebp);
 		break;
 	case HARDWARE_BREAKPOINT:
-		ret = snprintf (tmp, sizeof (tmp) - 1, "%s,%"PFMT64x ",1", CMD_RHBP, address);
+		ret = snprintf (tmp, sizeof (tmp) - 1, "%s,%"PFMT64x ",%d", CMD_RHBP, address, sizebp);
 		break;
 	case WRITE_WATCHPOINT:
+		ret = snprintf (tmp, sizeof (tmp) - 1, "%s,%"PFMT64x ",%d", CMD_RHWW, address, sizebp);
 		break;
 	case READ_WATCHPOINT:
+		ret = snprintf (tmp, sizeof (tmp) - 1, "%s,%"PFMT64x ",%d", CMD_RHWR, address, sizebp);
 		break;
 	case ACCESS_WATCHPOINT:
+		ret = snprintf (tmp, sizeof (tmp) - 1, "%s,%"PFMT64x ",%d", CMD_RHWA, address, sizebp);
 		break;
 	default:
 		break;
