@@ -1,4 +1,4 @@
-/* sdb - MIT - Copyright 2011-2016 - pancake */
+/* sdb - MIT - Copyright 2011-2018 - pancake */
 
 #include <stdio.h>
 #include <string.h>
@@ -175,21 +175,24 @@ SDB_API char *sdb_querys (Sdb *r, char *buf, size_t len, const char *_cmd) {
 		return NULL;
 	}
 	out = strbuf_new ();
+	if ((int)len < 1 || !buf) {
+		bufset = 1;
+		buf = malloc ((len = 64));
+		if (!buf) {
+			strbuf_free (out);
+			return NULL;
+		}
+	}
 	if (_cmd) {
 		cmd = original_cmd = strdup (_cmd);
 		if (!cmd) {
 			free (out);
+			if (bufset) {
+				free (buf);
+			}
 			return NULL;
 		}
 	} else {
-		if (len < 1 || !buf) {
-			bufset = 1;
-			buf = malloc ((len = 64));
-			if (!buf) {
-				strbuf_free (out);
-				return NULL;
-			}
-		}
 		cmd = buf;
 	}
 	// if cmd is null, we take buf as cmd
@@ -212,7 +215,7 @@ repeat:
 		if (next) {
 			*next = 0;
 		}
-		out_concat (sdb_fmt (0, "0x%08x\n", sdb_hash (p)));
+		out_concat (sdb_fmt ("0x%08x\n", sdb_hash (p)));
 		if (next) {
 			*next = ';';
 		}
@@ -837,7 +840,7 @@ fail:
 SDB_API int sdb_query (Sdb *s, const char *cmd) {
 	char buf[1024], *out;
 	int must_save = ((*cmd=='~') || strchr (cmd, '='));
-	out = sdb_querys (s, buf, sizeof (buf)-1, cmd);
+	out = sdb_querys (s, buf, sizeof (buf) - 1, cmd);
 	if (out) {
 		if (*out) {
 			puts (out);
