@@ -9,11 +9,28 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
+#ifdef R_API
+#undef R_API
+#endif
+#if R_SWIG
+  #define R_API export
+#elif R_INLINE
+  #define R_API inline
+#else
+  #if defined(__GNUC__) && __GNUC__ >= 4
+    #define R_API __attribute__((visibility("default")))
+  #elif defined(_MSC_VER)
+    #define R_API __declspec(dllexport)
+  #else
+    #define R_API
+  #endif
+#endif
+
 #if defined(EMSCRIPTEN) || defined(__linux__) || defined(__APPLE__) || defined(__GNU__) || defined(__ANDROID__) || defined(__QNX__)
   #define __BSD__ 0
   #define __UNIX__ 1
 #endif
-#if __KFBSD__ || defined(__NetBSD__) || defined(__OpenBSD__)
+#if __KFBSD__ || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD__)
   #define __BSD__ 1
   #define __UNIX__ 1
 #endif
