@@ -1,6 +1,7 @@
 #!/bin/sh
 
 BUILD=1
+FLAGS=""
 PREFIX="/data/data/org.radare.radare2installer/radare2"
 
 type pax
@@ -8,16 +9,27 @@ type pax
 
 cd `dirname $PWD/$0` ; cd ..
 
+# we need a more recent ndk to build the mergedlib for mips
+
+[ -z "${NDK_ARCH}" ] && NDK_ARCH=arm
+
+# ow yeah
+STATIC_BUILD=1
+
 case "$1" in
 "mips")
 	NDK_ARCH=mips
 	STATIC_BUILD=0
 	STRIP=mips-linux-android-strip
+#	FLAGS="-mlong-calls"
+#	export LDFLAGS="-fuse-ld=gold"
 	;;
 "mips64")
 	NDK_ARCH=mips64
 	STATIC_BUILD=0
 	STRIP=mips64el-linux-android-strip
+#	FLAGS="-mlong-calls"
+#	export LDFLAGS="-fuse-ld=gold"
 	;;
 "arm")
 	NDK_ARCH=arm
@@ -73,11 +85,7 @@ local)
 	;;
 esac
 
-[ -z "${NDK_ARCH}" ] && NDK_ARCH=arm
 [ -z "${STATIC_BUILD}" ] && STATIC_BUILD=0
-
-# ow yeah
-STATIC_BUILD=1
 export NDK_ARCH
 export STATIC_BUILD
 PKG=`./configure --version|head -n1 |cut -d ' ' -f 1`
@@ -87,7 +95,7 @@ echo NDK_ARCH: ${NDK_ARCH}
 echo "Using NDK_ARCH: ${NDK_ARCH}"
 echo "Using STATIC_BUILD: ${STATIC_BUILD}"
 
-export CFLAGS="-fPIC -fPIE"
+export CFLAGS="-fPIC -fPIE ${FLAGS}"
 
 if [ "${BUILD}" = 1 ]; then
 	if [ -z "${NDK}" ]; then
@@ -100,7 +108,7 @@ if [ "${BUILD}" = 1 ]; then
 	if [ 1 = 1 ]; then
 		make mrproper
 		if [ $STATIC_BUILD = 1 ]; then
-			CFGFLAGS="--without-pic --with-nonpic"
+			CFGFLAGS="--with-libr"
 		fi
 		# dup
 		echo ./configure --with-compiler=android \
